<?php


namespace App\Configuration;


use App\Exceptions\Api\ServerError;
use App\Http\Resources\LabConfigurationCollection;
use Illuminate\Support\Facades\Config;
/**
 * @group Setari
 *
 * APIs for getting configurations
 */
class LabConfiguration extends Config
{

    /**     *
     * @param  string  $type
     * @param  string  $returnColumn
     *
     * @return array
     */
    public static function config(string $type, string $returnColumn = 'value',$returnAs = 'array'): array
    {
        $configuration = self::getLabConfig();

        $configValues = [];
        if (!isset($configuration[$type])) {
            return $configValues;
        }

        foreach ($configuration[$type] as $item) {
            switch ($returnAs){
                case 'array':
                    $configValues[] = $item[$returnColumn];
                    break;
                case 'assoc':
                    $configValues[$item[$returnColumn]] = $item;
                    break;
                default:
                    throw new ServerError(__('ReturnAs value unknown'));
                    break;
            }
        }

        return $configValues;
    }

    /**
     *
     * @param  string  $type
     * @param  array  $returnColumns
     *
     * @return
     */
    public static function configAllFields(string $type, array $returnColumns = [],$asCollection = true)
    {
        $configuration = self::getLabConfig();

        $configValues = [];
        if (!isset($configuration[$type])) {
            return $configValues;
        }

        foreach ($configuration[$type] as $key => $item) {
            if (empty($returnColumns)) {
                $returnColumns = array_keys($item);
            }
            foreach ($returnColumns as $returnColumn) {
                $configValues[$key][$returnColumn] = $item[$returnColumn] ?? null;
            }
        }

        if (!$asCollection) {
            return $configValues;
        }

        if (request()->expectsJson() || request()->wantsJson()) {
            return new LabConfigurationCollection($configValues);
        }

        return $configValues;
    }


    /**
     * @param  string  $type
     * @param  string  $returnColumn
     *
     * @return string
     */
    public static function configAsString(string $type, string $returnColumn = 'value'): string
    {
        return implode(
            ",",
            LabConfiguration::config($type, $returnColumn)
        );
    }

    /**
     * Work stage types configuration
     *
     * Possible types of work stages
     * @return
     */
    public static function workStageTypes()
    {
        return self::configAllFields('work_stages_types');
    }

    /**
     * Work stage statuses configuration
     *
     * Possible statuses of order stages
     * @return
     */
    public static function orderStageStatuses()
    {
        return self::configAllFields('order_stage_statuses');
    }

    /**
     * Work stage price types configuration
     *
     * Possible values of work stage prices types
     * @return
     */
    public static function workTemplatesPriceTypes()
    {
        return self::configAllFields('work_templates_price_types');
    }

    /**
     * User statuses configuration
     *
     * Possible values of user status
     * @return
     */
    public static function userStatuses()
    {
        return self::configAllFields('user_statuses');
    }

    /**
     * Order statuses configuration
     *
     * Possible values of order status
     * @return
     */
    public static function orderStatuses()
    {
        return self::configAllFields('order_statuses');
    }

    /**
     * Dentition configuration
     *
     * Possible values of dentition
     * @return
     */
    public static function dentition()
    {
        return self::configAllFields('dentition',[],false);
    }

    /**
     * OWS special stages
     *
     * Possible values of special stages for order work stages
     * @return
     */
    public static function getCheckboxStages(){
        $options = self::configAllFields('work_stages_types',[],false);
        $res = [];

        foreach($options as $option){
            if(isset($option['order_checkbox']) && $option['order_checkbox']){
                $res[] = $option['value'];
            }
        }
        return $res;
    }

    /**
     * OWS possible special stages
     *
     * Data of possible special stages for order work stages
     * @return
     */
    public static function getOrderWorkStageSpecialStages(){
        $keys = self::getCheckboxStages();
        $options = self::configAllFields('work_stages_types',[],false);
        $res = [];

        foreach($options as $option){
            if(in_array($option['value'],$keys)){
                $res[] = $option;
            }

        }
        return $res;
    }


    /**
     * Caching lab configuration as is not changing often
     * @return mixed
     * @throws \Exception
     */
    public static function getLabConfig()
    {
        $seconds = 3600;
        $identifier = tenancy()->tenant->id??__('no_use_only_so_you_can_generate_documentation_without_errors');
        //using cache for lab_configuration
        $configuration = cache()->store('file')->remember(
            'lab_configuration'.$identifier,
            $seconds,
            function () {
                return config('lab_configuration');
            }
        );

        return $configuration;

    }
}
