<?php

namespace App\Http\Controllers;

use App\Http\Requests\APBrandRequest;
use App\Http\Resources\APBrandResource;
use App\Models\AdditionalPricesBrand;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

/**
 * @group Comenzi PA brands
 *
 * APIs for managing order additional prices
 */
class AdditionalPricesBrandController extends Controller
{
    /**
     * Brand detailed
     *
     * @param  Request  $request
     * @param $aPBrandId
     *
     * @return Application|ResponseFactory|Response
     */
    public function show(Request $request, $aPBrandId)
    {
        if (!$request->user()->can(['order_additional_prices.read'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.read.missing.permission'));
        }
        $aPBrand = AdditionalPricesBrand::findOrFail((int)$aPBrandId);
        return response(['data' => new APBrandResource($aPBrand)]);
    }

    /**

     * List of brands
     *
     * @param  Request  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function index(Request $request)
    {
        if (!$request->user()->can(['order_additional_prices.read'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.read.missing.permission'));
        }

        $aPBrands = AdditionalPricesBrand::orderBy('name')->get();

        return response(['data' => APBrandResource::collection($aPBrands)]);
    }

    /**
     * Add new brand
     *
     * @param  APBrandRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function store(APBrandRequest $request)
    {
        if (!$request->user()->can(['order_additional_prices.create'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.create.missing.permission'));
        }
        $validated = $request->validated();
        $aPBrand = AdditionalPricesBrand::create($validated);

        return response(['data' => new APBrandResource($aPBrand)], 201);
    }

    /**
     * Update a brand
     *
     * @param  APBrandRequest  $request
     * @param $aPBrandId
     *
     * @return Application|ResponseFactory|Response
     */
    public function update(APBrandRequest $request, $aPBrandId)
    {
        if (!$request->user()->can(['order_additional_prices.update'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.update.missing.permission'));
        }
        $aPBrand = AdditionalPricesBrand::findOrFail((int)$aPBrandId);
        $validated = $request->validated();
        $aPBrand->update($validated);
        return response(['data' => new APBrandResource($aPBrand)]);
    }

    /**
     * Delete a brand
     *
     * @param  Request  $request
     * @param $aPBrandId
     *
     * @return Application|ResponseFactory|Response
     */
    public function destroy(Request $request, $aPBrandId)
    {
        if (!$request->user()->can(['order_additional_prices.delete'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.delete.missing.permission'));
        }
        $aPBrand = AdditionalPricesBrand::findOrFail((int)$aPBrandId);
        $aPBrand->delete();
        return response(['data' => __('Brand successfully deleted')],204);
    }
}
