<?php

namespace App\Http\Controllers;

use App\Http\Requests\APCategoryRequest;
use App\Http\Resources\APCategoryResource;
use App\Models\AdditionalPricesCategory;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

/**
 * @group Comenzi PA categorii
 *
 * APIs for managing order additional prices
 */
class AdditionalPricesCategoryController extends Controller
{
    /**
     * Category detailed
     *
     * @param  Request  $request
     * @param $aPCategoryId
     *
     * @return Application|ResponseFactory|Response
     */
    public function show(Request $request, $aPCategoryId)
    {
        if (!$request->user()->can(['order_additional_prices.read'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.read.missing.permission'));
        }
        $aPCategory = AdditionalPricesCategory::findOrFail((int)$aPCategoryId);
        return response(['data' => new APCategoryResource($aPCategory)]);
    }

    /**
     * List of categories
     *
     * @param  Request  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function index(Request $request)
    {
        if (!$request->user()->can(['order_additional_prices.read'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.read.missing.permission'));
        }

        $aPCategories = AdditionalPricesCategory::orderBy('name')->get();

        return response(['data' => APCategoryResource::collection($aPCategories)]);
    }

    /**
     * Add new category
     *
     * @param  APCategoryRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function store(APCategoryRequest $request)
    {
        if (!$request->user()->can(['order_additional_prices.create'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.create.missing.permission'));
        }
        $validated = $request->validated();
        $aPCategory = AdditionalPricesCategory::create($validated);

        return response(['data' => new APCategoryResource($aPCategory)], 201);
    }

    /**
     * Update a category
     *
     * @param  APCategoryRequest  $request
     * @param $aPBrandId
     *
     * @return Application|ResponseFactory|Response
     */
    public function update(APCategoryRequest $request, $aPCategoryId)
    {
        if (!$request->user()->can(['order_additional_prices.update'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.update.missing.permission'));
        }
        $aPCategory = AdditionalPricesCategory::findOrFail((int)$aPCategoryId);
        $validated = $request->validated();
        $aPCategory->update($validated);
        return response(['data' => new APCategoryResource($aPCategory)]);
    }

    /**
     * Delete a category
     *
     * @param  Request  $request
     * @param $aPBrandId
     *
     * @return Application|ResponseFactory|Response
     */
    public function destroy(Request $request, $aPCategoryId)
    {
        if (!$request->user()->can(['order_additional_prices.delete'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.delete.missing.permission'));
        }
        $aPBrand = AdditionalPricesCategory::findOrFail((int)$aPCategoryId);
        $aPBrand->delete();
        return response(['data' => __('Category successfully deleted')], 204);
    }
}
