<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Exceptions\Api\ValidationException;
use App\Http\Requests\APProductRequest;
use App\Http\Resources\APProductResource;
use App\Models\AdditionalPricesProduct;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

/**
 * @group Comenzi PA produse
 *
 * APIs for managing order additional prices
 */
class AdditionalPricesProductController extends Controller
{
    /**
     * Product detailed
     *
     * @param  Request  $request
     * @param $aPCategoryId
     *
     * @return Application|ResponseFactory|Response
     */
    public function show(Request $request, $aPCategoryId)
    {
        if (!$request->user()->can(['order_additional_prices.read'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.read.missing.permission'));
        }
        $aPCategory = AdditionalPricesProduct::findOrFail((int)$aPCategoryId);
        return response(['data' => new APProductResource($aPCategory)]);
    }

    /**
     * List of products
     *
     * @param  Request  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function index(Request $request)
    {
        if (!$request->user()->can(['order_additional_prices.read'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.read.missing.permission'));
        }

        $aPProducts = AdditionalPricesProduct::orderBy('name')->get();
        $aPProducts->load(['brand', 'category']);

        return response(['data' => APProductResource::collection($aPProducts)]);
    }

    /**
     * Add new product
     *
     * @param  APProductRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function store(APProductRequest $request)
    {
        if (!$request->user()->can(['order_additional_prices.create'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.create.missing.permission'));
        }
        $validated = $request->validated();
        $aPCategory = AdditionalPricesProduct::create($validated);

        return response(['data' => new APProductResource($aPCategory)], 201);
    }

    /**
     * Update a product
     *
     * @param  APProductRequest  $request
     * @param $aPBrandId
     *
     * @return Application|ResponseFactory|Response
     */
    public function update(APProductRequest $request, $aPCategoryId)
    {
        if (!$request->user()->can(['order_additional_prices.update'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.update.missing.permission'));
        }
        $aPCategory = AdditionalPricesProduct::findOrFail((int)$aPCategoryId);
        $validated = $request->validated();
        $aPCategory->update($validated);
        return response(['data' => new APProductResource($aPCategory)]);
    }

    /**
     * Delete a product
     *
     * @param  Request  $request
     * @param $aPBrandId
     *
     * @return Application|ResponseFactory|Response
     */
    public function destroy(Request $request, $aPCategoryId)
    {
        if (!$request->user()->can(['order_additional_prices.delete'])) {
            throw new AccessDeniedException(__('lang_permissions.order_additional_prices.delete.missing.permission'));
        }
        $aPBrand = AdditionalPricesProduct::findOrFail((int)$aPCategoryId);
        $aPBrand->delete();
        return response(['data' => __('Category successfully deleted')], 204);
    }

    public static function generateParametersForAddingAsAdditionalPrice(array $parameters)
    {
        if (
            !isset($parameters['order_id'])
            || !isset($parameters['product_id'])
            || !isset($parameters['quantity'])
        ) {
            throw new ValidationException(__('Must have parameters missing order_id, product_id, quantity'));
        }
        $product = AdditionalPricesProduct::findOrFail((int)$parameters['product_id']);
        unset($parameters['product_id']);
        unset($parameters['order_as_response']);

        if (!isset($parameters['description'])) {
            $parameters['description'] = __(
                'Brand :brand, categorie :category',
                [
                    'brand'    => $product->brand->name ?? null,
                    'category' => $product->category->name ?? null
                ]
            );
        }
        $parameters['name'] = $product->name;
        $parameters['price_per_unit'] = $product->price;
        $parameters['to_be_invoiced'] = $parameters['default_invoicing'] ?? $product->default_invoicing;
        return $parameters;
    }
}
