<?php

namespace App\Http\Controllers\Api;

use App\Actions\Fortify\CreateNewUser;
use App\Exceptions\Api\ModelCouldNotBeCreated;
use App\Exceptions\Api\Unauthorized;
use App\Http\Controllers\Controller;
use App\Http\Resources\UserCompleteWithTokenResource;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;

/**
 * @group Utilizatori
 *
 * APIs for managing users
 */
class AuthController extends Controller
{
    /**
     * Login a user.
     * @bodyParam   username string required.
     * @bodyParam  password string required.
     * @unauthenticated
     */
    public function login(Request $request)
    {
        $request->validate([
            'username' => 'required',
            'password' => 'required'
        ]);
        $user = User::where("username", $request->username)->where('status', 'active')->orWhere('status', 'vacation')->first();

        if (!$user || !Hash::check($request->password, $user->password)) {
            throw new Unauthorized(__('The provided credentials are incorrect.'));
        }

        return response(['data' => new UserCompleteWithTokenResource($user)]);
    }



    public function register(Request $request)
    {
        if (!$user = (new CreateNewUser())->create($request->all())) {
            throw new ModelCouldNotBeCreated('User could not be registered', 500);
        }
        $user = $user->refresh();

        return response(['data' => new UserCompleteWithTokenResource($user)]);
    }



    /**
     * Refresh token.
     *
     * Deletes current token for user and generate 1 new
     */
    public function refresh(Request $request)
    {
        $user = $request->user();
        $user->currentAccessToken()->delete();
        return response()->json(['data' => ['token' => $user->createToken($user->username)->plainTextToken]]);
    }


    /**
     * Logout logged user token.
     *
     * Deletes token
     */
    public function logout(Request $request)
    {
        $user = $request->user();
        $user->currentAccessToken()->delete();
        return response('', 204);
    }
}
