<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Http\Requests\CollaboratorRequest;
use App\Http\Requests\ImportCollaboratorRequest;
use App\Http\Resources\CollaboratorCollection;
use App\Http\Resources\CollaboratorResource;
use App\Models\Collaborator;
use App\Services\ImportCollaboratorService;
use Illuminate\Http\Request;
use Spatie\QueryBuilder\QueryBuilder;

/**
 * @group Colaboratori
 *
 * APIs for managing collaborators
 */
class CollaboratorController extends Controller
{
    /**
     * Create the controller instance.
     *
     * @return void
     */
    public function __construct()
    {
    }

    /**
     * Collaborators listing.
     *
     * @apiResourceCollection App\Http\Resources\CollaboratorCollection
     * @apiResourceModel App\Models\Collaborator with=region
     * @queryParam sort string Field to sort by. Sorting ASC id Sorting DESC -id Example: -id,last_name
     * @queryParam filter[dentists.last_name] string Field to filter by dentist id and/or last_name.  Example: /users?filter[last_name]=id,last_name
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if (!$request->user()->can(['collaborators.read'])) {
            throw new AccessDeniedException(__('lang_permissions.collaborators.read.missing.permission'));
        }
        $collaborators = QueryBuilder::for(Collaborator::class)
            ->allowedFields('dentists.id', 'dentists.last_name')
            ->with(['dentists', 'region'])
            ->allowedIncludes(['dentists', 'region'])
            ->defaultSorts(Collaborator::$defaultSort)
            ->allowedSorts(Collaborator::$sortable)
            ->allowedFilters(Collaborator::$filterable)
            ->get();

        return response(new CollaboratorCollection($collaborators));
    }

    /**
     * Store a newly created collaborator.
     *
     * @apiResource App\Http\Resources\CollaboratorResource
     * @apiResourceModel App\Models\Collaborator with=region
     *
     * @param  \Illuminate\Http\Request  $request
     *
     * @return \Illuminate\Http\Response
     */
    public function store(CollaboratorRequest $request)
    {
        if (!$request->user()->can(['collaborators.create'])) {
            throw new AccessDeniedException(__('lang_permissions.collaborators.create.missing.permission'));
        }
        // Retrieve the validated input data...
        $validated = $request->validated();

        $collaborator = new Collaborator();
        $collaborator = $this->updateFields($validated, $collaborator);

        return response(['data' => new CollaboratorResource($collaborator)], 201);
    }

    /**
     * Display a collaborator.
     *
     * @apiResource App\Http\Resources\CollaboratorResource
     * @apiResourceModel App\Models\Collaborator with=region
     * @urlParam collaborator integer
     *
     * @param  \App\Models\Collaborator  $collaborator
     *
     * @return \Illuminate\Http\Response
     */
    public function show(Request $request, Collaborator $collaborator)
    {
        if (!$request->user()->can(['collaborators.read'])) {
            throw new AccessDeniedException(__('lang_permissions.collaborators.read.missing.permission'));
        }

        return response(['data' => new CollaboratorResource($collaborator)]);
    }

    /**
     * Update  a collaborator.
     *
     * @apiResource App\Http\Resources\CollaboratorResource
     * @apiResourceModel App\Models\Collaborator with=region
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Collaborator  $collaborator
     *
     * @return \Illuminate\Http\Response
     */
    public function update(CollaboratorRequest $request, Collaborator $collaborator)
    {
        if (!$request->user()->can(['collaborators.update'])) {
            throw new AccessDeniedException(__('lang_permissions.collaborators.update.missing.permission'));
        }
        // Retrieve the validated input data...
        $validated = $request->validated();

        $collaborator = $this->updateFields($validated, $collaborator);

        return response(['data' => new CollaboratorResource($collaborator)]);
    }


    /**
     * Remove  a collaborator.
     * @param  \App\Models\Collaborator  $collaborator
     * @urlParam collaborator required integer
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, Collaborator $collaborator)
    {
        if (!$request->user()->can(['collaborators.delete'])) {
            throw new AccessDeniedException(__('lang_permissions.collaborators.delete.missing.permission'));
        }

        $collaborator->delete();
        return response(['message' => __('Successfully deleted')], 204);
    }

    /**
     * @param  array  $validated
     * @param  Collaborator  $collaborator
     *
     * @return Collaborator
     * @throws \Throwable
     */
    public function updateFields(array $validated, Collaborator $collaborator): Collaborator
    {
        foreach ($validated as $fieldName => $fieldValue) {
            $collaborator->$fieldName = $fieldValue;
        }
        $collaborator->saveOrFail();

        return $collaborator->refresh();
    }

    /**
     * Import collaborator's data from Openapi
     *
     * @param ImportCollaboratorRequest $request
     * @return void
     */
    public function importCollaborator(ImportCollaboratorRequest $request)
    {
        if (!$request->user()->can(['collaborators.read'])) {
            throw new AccessDeniedException(__('lang_permissions.collaborators.read.missing.permission'));
        }

        $validated = $request->validated();
        $collaborator = ImportCollaboratorService::getCompanyData($validated['cui']);

        return response()->json($collaborator, $collaborator['status'] ?? 200);
    }
}
