<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Http\Requests\CollaboratorPriceDestroyRequest;
use App\Http\Requests\CollaboratorPriceRequest;
use App\Http\Requests\CollaboratorPriceStoreRequest;
use App\Http\Resources\WorkTemplatePriceResource;
use App\Models\Collaborator;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Response;

/**
 * @group Colaboratori
 *
 * APIs for managing collaborators
 */

class CollaboratorPriceController extends Controller
{

    /**
     * CP List of prices for a collaborator.
     * @param  CollaboratorPriceRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function index(CollaboratorPriceRequest $request)
    {
        if (!$request->user()->can(['collaborators.update'])) {
            throw new AccessDeniedException(__('lang_permissions.collaborators.update.missing.permission'));
        }

        $validated = $request->validated();
        $templates = Collaborator::findOrFail($validated['collaborator_id'])->templatesPrice();
        if(!empty($validated['work_template_id'])){
            $template = $templates->where('work_template_id', $validated['work_template_id'])->firstOrFail();
            return response(['data' => (new WorkTemplatePriceResource($template))]);
        }
        $templates = $templates ->get();

        return response(['data' => (WorkTemplatePriceResource::collection($templates))]);
    }

    /**
     * CP Add/update  a price to a collaborator.
     *
     * @param  CollaboratorPriceStoreRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function storeOrUpdate(CollaboratorPriceStoreRequest $request)
    {
        if (!$request->user()->can(['collaborators.update'])) {
            throw new AccessDeniedException(__('lang_permissions.collaborators.update.missing.permission'));
        }

        $validated = $request->validated();
        // TODO de verificat ca intr-adevar sunt colaborator si nu dentist
        // (redenumeste variabila $dentist)
        $dentist = Collaborator::findOrFail($validated['collaborator_id']);
        $dentist->templatesPrice()->syncWithoutDetaching(
            [
                $validated["work_template_id"] => [
                    "price"  => $validated["price"],
                    "locked" => $validated["locked"],
                ]
            ]
        );
        $templateWithPrice = $dentist->templatesPrice()->where('work_template_id',$validated["work_template_id"])->firstOrFail();
        return response(['data' => new WorkTemplatePriceResource($templateWithPrice)], 201);
    }

    /**
     * CP Delete  a price to a collaborator.
     *
     * @param  DentistPriceStoreRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function destroy(CollaboratorPriceDestroyRequest $request)
    {
        if (!$request->user()->can(['collaborators.update'])) {
            throw new AccessDeniedException(__('lang_permissions.collaborators.update.missing.permission'));
        }

        $validated = $request->validated();
        $dentist = Collaborator::findOrFail($validated['collaborator_id']);
        $templateWithPrice = $dentist->templatesPrice()->where('work_template_id',$validated["work_template_id"])->firstOrFail();
        $templateWithPrice->pivot->delete();
        return response(['data' => new WorkTemplatePriceResource($templateWithPrice)], 204);
    }
}
