<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Exceptions\Api\PossibleUnSyncException;
use App\Models\Order;
use App\Models\OrderWorkStage;
use App\Services\OrderWorkStagesService;
use App\Services\UserService;
use App\Services\UserSkipService;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

/**
 * @group Dashboard
 *
 * APIs for managing dashboard
 */
class DashboardController extends Controller
{
    /**
     * Orders dashboard by status
     *
     * @param  Request  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function getDashboardOrders(Request $request)
    {
        if (!$request->user()->can(['production.order_panel'])) {
            throw new AccessDeniedException(__('lang_permissions.production.order_panel.missing.permission'));
        }

        $orders = Order::with(
            [
                'user' => function ($q) {
                    // TODO: add this to the returned array
                    return $q->where('status', 'vacation')->count();
                }
            ]
        )
            ->select(
                [
                    'id', 'order_no', 'order_status as status', 'patient_id', 'collaborator_id', 'dentist_id', 'start_date', 'finish_date', 'order_details',
                    \DB::raw("IF(order_details IS NULL or order_details = '', FALSE, TRUE) as has_order_details"),
                ],
            )->joinFirstAvailableStage()->with(
                [
                    'stagesWithoutPivot' => function ($q) {
                        $q
                            ->with(
                                [
                                    'user' => function ($q) {
                                        $q->select([
                                            'id',
                                            'status',
                                            \DB::raw("IF(status = 'vacation', TRUE, FALSE) as on_vacation"),
                                        ]);
                                    }
                                ]
                            )
                            ->select(
                                [
                                    'id',
                                    'order_work_stages.order_id',
                                    'order_work_stages.order_work_template_id as order_work_id',
                                    \DB::raw('IF(order_work_stages.status = "not_ready", "ready", order_work_stages.status) as stage_status'),
                                    'order_work_stages.status',
                                    'name',
                                    'type',
                                    'user_name',
                                    'user_id',
                                    'user_id as asigned_user',
                                ]
                            )->join(
                                \DB::raw(
                                    '(
                                select `order_id`,
                                        MIN(`rank`) as `rank`,
                                	    MIN(`order_work_template_id`) as `order_work_template_id`,
                                        true as ready
                                FROM order_work_stages
                                WHERE order_work_template_id IN (
                                    select MIN(`order_work_template_id`) AS `order_work_template_id`
                                    FROM `order_work_stages`
                                    WHERE `status` <> "finished"
                                    GROUP BY `order_id`
                                )
                                AND `status` <> "finished"
                                GROUP BY `order_id`) first_by_rank'
                                ),
                                function ($join) {
                                    $join->on('order_work_stages.order_id', '=', 'first_by_rank.order_id')->on(
                                        'order_work_stages.rank',
                                        '=',
                                        'first_by_rank.rank'
                                    )->on('order_work_stages.order_work_template_id', '=', 'first_by_rank.order_work_template_id');
                                }
                            );
                    }
                ]
            )->where('orders.status', '<>', 'complete')->with(
                [
                    'works' => function ($q) {
                        $q->select(['id', 'order_id', 'name']);
                    }
                ]
            )->with(
                [
                    'patient' => function ($q) {
                        $q->select(['id', \DB::raw('CONCAT_WS(" ", patients.first_name, patients.last_name) as name')]);
                    }
                ]
            )->with(
                [
                    'collaborator' => function ($q) {
                        $q->select(['id', 'name']);
                    }
                ]
            )->with(
                [
                    'dentist' => function ($q) {
                        $q->select(['id', \DB::raw('CONCAT_WS(" ", dentists.first_name, dentists.last_name) as name')]);
                    }
                ]
            )->get()->groupBy(
                function ($item, $key) {
                    switch ($item->order_status) {
                        case 'creating':
                            return 'creating';
                        case 'expecting_completion':
                            return 'complete';
                        default:
                            return 'working';
                    }
                }
            );
        $authUser = auth()->user();
        $canSeeNewAndFinishOrders = UserService::userCanReadOrderDashboardOrdersNewAndFinished($authUser, false);

        //reset creating orders if we dont have the permission to see them
        if (!isset($orders['creating']) || !$canSeeNewAndFinishOrders) {
            $orders['creating'] = [];
        }

        if (!isset($orders['working'])) {
            $orders['working'] = [];
        }

        //reset expecting_completion status  orders if we dont have the permission to see them
        if (!isset($orders['complete']) || !$canSeeNewAndFinishOrders) {
            $orders['complete'] = [];
        }

        $canSeeAllWorkingOrders = UserService::userCanReadOthersStages($authUser, false);
        foreach ($orders['working'] ?? [] as $key => &$order) {
            $order->status = $order->order_status;
            foreach ($order->stagesWithoutPivot as &$stage) {
                $canUserStartStage = UserService::userCanStartStage($authUser, $stage);
                if (!($canSeeAllWorkingOrders || $canUserStartStage)) {
                    unset($orders['working'][$key]);
                    continue;
                }

                $stageStarted = OrderWorkStagesService::isStageStarted($stage);
                $canStageBeFinished = OrderWorkStagesService::canStageBeFinished($stage);
                $stage->actions = new \stdClass();
                $stage->actions->skipping = [
                    'can_skip' => UserService::userCanSkipStage($authUser, $stage, false),
                ];
                $stage->actions->timing_actions = [
                    'start'  => (!$stageStarted) && $canUserStartStage,
                    'pause'  => $stageStarted && $canUserStartStage,
                    'finish' => $canStageBeFinished && $canUserStartStage,
                ];
            }
        }
        //do this because when you do unset to an array, that array becomes an object
        if (!is_array($orders['working'])) {
            $orders['working'] = array_values($orders['working']->toArray());
        }



        unset($order, $stage);
        $orders = $orders->toArray();

        return response(['data' => $orders]);
    }


    /**
     * Skip order work stage
     *
     * @param  Request  $request
     * @param  OrderWorkStage  $orderWorkStage
     *
     * @return Application|ResponseFactory|Response
     * @throws PossibleUnSyncException
     */
    public function skipOrderWorkStage(Request $request, OrderWorkStage $orderWorkStage)
    {
        return UserSkipService::skipStageUser($request, $orderWorkStage);
    }
}
