<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Exceptions\Api\ValidationException;
use App\Http\Requests\DentistCollaboratorRequest;
use App\Http\Requests\DentistCollaboratorsRequest;
use App\Http\Requests\DentistRequest;
use App\Http\Resources\CollaboratorResource;
use App\Http\Resources\DentistCollection;
use App\Http\Resources\DentistResource;
use App\Models\Collaborator;
use App\Models\Dentist;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Spatie\QueryBuilder\QueryBuilder;

/**
 * @group Dentisti
 *
 * APIs for managing Dentists
 */
class DentistController extends Controller
{
    /**
     * Dentists listing.
     *
     * @apiResourceCollection  App\Http\Resources\DentistCollection
     * @apiResourceModel App\Models\Dentist
     * @queryParam sort string Field to sort by. Sorting ASC id Sorting DESC -id Example: -id,first_name,last_name
     * @queryParam filter[collaborators.name] string Field to filter by collaborator id and/or name.  Example: /users?filter[name]=id,name
     * @return Response
     *
     */
    public function index(Request $request)
    {
        if (!$request->user()->can(['dentists.read'])) {
            throw new AccessDeniedException(__('lang_permissions.dentists.read.missing.permission'));
        }
        $dentists = QueryBuilder::for(Dentist::class)
            ->allowedFields('collaborators.id', 'collaborators.name')
            ->with(['collaborators'])
            ->allowedIncludes(['collaborators'])
            ->defaultSorts(Dentist::$defaultSort)
            ->allowedSorts(Dentist::$sortable)
            ->allowedFilters(Dentist::$filterable)
            ->get();
        
        return response(new DentistCollection($dentists));
    }

    /**
     * Store a newly created dentist.
     *
     * @apiResource  App\Http\Resources\DentistResource
     * @apiResourceModel App\Models\Dentist
     *
     * @param  \Illuminate\Http\Request  $request
     *
     * @return Response
     */
    public function store(DentistRequest $request)
    {
        if (!$request->user()->can(['dentists.create'])) {
            throw new AccessDeniedException(__('lang_permissions.dentists.create.missing.permission'));
        }
        // Retrieve the validated input data...
        $validated = $request->validated();

        $dentist = Dentist::create($validated);

        return response(['data' => new DentistResource($dentist)], 201);
    }

    /**
     * Display a dentist.
     *
     * @apiResource  App\Http\Resources\DentistResource
     * @apiResourceModel App\Models\Dentist
     *
     * @param  \App\Models\Dentist  $dentist
     *
     * @return Response
     */
    public function show(Request $request, Dentist $dentist)
    {
        if (!$request->user()->can(['dentists.read'])) {
            throw new AccessDeniedException(__('lang_permissions.dentists.read.missing.permission'));
        }

        $dentist->loadMissing('collaborators');
        return response(['data' => new DentistResource($dentist)]);
    }

    /**
     * Update a dentist.
     *
     * @apiResource  App\Http\Resources\DentistResource
     * @apiResourceModel App\Models\Dentist
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Dentist  $dentist
     *
     * @return Response
     */
    public function update(DentistRequest $request, Dentist $dentist)
    {
        if (!$request->user()->can(['dentists.update'])) {
            throw new AccessDeniedException(__('lang_permissions.dentists.update.missing.permission'));
        }
        // Retrieve the validated input data...
        $validated = $request->validated();

        foreach ($validated as $fieldName => $fieldValue) {
            $dentist->$fieldName = $fieldValue;
        }
        $dentist->save();

        return response(['data' => new DentistResource($dentist)]);
    }


    /**
     * Remove a dentist.
     *
     * @param  \App\Models\Dentist  $dentist
     *
     * @return Response
     */
    public function destroy(Request $request, Dentist $dentist)
    {
        if (!$request->user()->can(['dentists.delete'])) {
            throw new AccessDeniedException(__('lang_permissions.dentists.delete.missing.permission'));
        }

        $dentist->delete();
        return response(['message' => __('Successfully deleted')], 204);
    }

    /**
     * DC Add  a collaborator to a dentist.
     *
     * @param  DentistCollaboratorRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function storeCollaborator(DentistCollaboratorRequest $request)
    {
        if (!$request->user()->can(['dentists.update'])) {
            throw new AccessDeniedException(__('lang_permissions.dentists.update.missing.permission'));
        }

        $validated = $request->validated();
        $dentist = Dentist::with('collaborators')->findOrFail($validated['dentist_id']);
        $collaborator = Collaborator::findOrFail($validated['collaborator_id']);
        $pivot = $dentist->collaborators()->where('collaborator_id',$validated['collaborator_id'])->first();
        if ($pivot !== null) {
            throw new ValidationException('Collaborator already exist.');
        }
        $dentist->collaborators()->syncWithoutDetaching([$validated['collaborator_id']]);
        $dentist->refresh();
        $dentist->loadMissing(['collaborators']);
        return response(['data' => (new DentistResource($dentist))],201);
    }

    /**
     * DC Remove  a collaborator from a dentist.
     *
     * @param  DentistCollaboratorRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function destroyCollaborator(DentistCollaboratorRequest $request)
    {
        if (!$request->user()->can(['dentists.update'])) {
            throw new AccessDeniedException(__('lang_permissions.dentists.update.missing.permission'));
        }

        $validated = $request->validated();
        $dentist = Dentist::with('collaborators')->findOrFail($validated['dentist_id']);
        $collaborator = $dentist->collaborators()->where('collaborator_id',$validated['collaborator_id'])->firstOrFail();
        $dentist->collaborators()->detach([$collaborator->id]);
        $dentist->refresh();
        $dentist->loadMissing(['collaborators']);
        return response(['data' => (new DentistResource($dentist))],204);
    }

    /**
     * DC List of collaborators of a dentist.
     *
     * @param  DentistCollaboratorRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function indexCollaborators(DentistCollaboratorsRequest $request)
    {
        if (!$request->user()->can(['dentists.update'])) {
            throw new AccessDeniedException(__('lang_permissions.dentists.update.missing.permission'));
        }

        $validated = $request->validated();
        $dentist = Dentist::with('collaborators')->findOrFail($validated['dentist_id']);
        return response(['data' => (CollaboratorResource::collection($dentist->collaborators))]);
    }

}
