<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Http\Requests\DentistPriceDestroyRequest;
use App\Http\Requests\DentistPricesRequest;
use App\Http\Requests\DentistPriceStoreRequest;
use App\Http\Resources\DentistPriceResource;
use App\Http\Resources\WorkTemplatePriceResource;
use App\Models\Dentist;
use App\Services\DentistPriceService;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Response;

/**
 * @group Dentisti
 *
 * APIs for managing Dentists
 */

class DentistPriceController extends Controller
{

    /**
     * DP List of prices for a dentist.
     *
     * @param  DentistPricesRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function index(DentistPricesRequest $request)
    {
        if (!$request->user()->can(['dentists.update'])) {
            throw new AccessDeniedException(__('lang_permissions.dentists.update.missing.permission'));
        }
        $validated = $request->validated();
        $preferentialPrices = DentistPriceService::getDentistPreferentialPrices($validated['dentist_id']);
        $colPrefPrice = DentistPriceService::getCollaboratorPreferentialPrices($validated['dentist_id']);

        collect($preferentialPrices)->each(function (&$item) use ($colPrefPrice) {
            $item->collaboratorPreferentialPrices = $colPrefPrice->where('work_template_id',$item->work_template_id);
        });

        return response(['data' => (DentistPriceResource::collection($preferentialPrices))]);
    }

    /**
     * DP Add/update  a price to a dentist.
     *
     * @param  DentistPriceStoreRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function storeOrUpdate(DentistPriceStoreRequest $request)
    {
        if (!$request->user()->can(['dentists.update'])) {
            throw new AccessDeniedException(__('lang_permissions.dentists.update.missing.permission'));
        }

        $validated = $request->validated();
        $dentist = Dentist::findOrFail($validated['dentist_id']);
        $dentist->templatesPrice()->syncWithoutDetaching(
            [
                $validated["work_template_id"] => [
                    "price"  => $validated["price"],
                    "locked" => $validated["locked"],
                ]
            ]
        );
        $templateWithPrice = $dentist->templatesPrice()->where('work_template_id', $validated["work_template_id"])->firstOrFail();
        return response(['data' => new WorkTemplatePriceResource($templateWithPrice)], 201);
    }

    /**
     * DP Delete  a price to a dentist.
     *
     * @param  DentistPriceStoreRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function destroy(DentistPriceDestroyRequest $request)
    {
        if (!$request->user()->can(['dentists.update'])) {
            throw new AccessDeniedException(__('lang_permissions.dentists.update.missing.permission'));
        }

        $validated = $request->validated();
        $dentist = Dentist::findOrFail($validated['dentist_id']);
        $templateWithPrice = $dentist->templatesPrice()->where('work_template_id', $validated["work_template_id"])->firstOrFail();
        $templateWithPrice->pivot->delete();
        return response(['data' => new WorkTemplatePriceResource($templateWithPrice)], 204);
    }
}
