<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Http\Requests\DiscountTypeRequest;
use App\Http\Resources\DiscountTypeResource;
use App\Models\DiscountType;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\JsonResource;
use Illuminate\Http\Response;

/**
 * @group DP Tipuri reduceri
 *
 * APIs for managing discount type
 */
class DiscountTypeController extends Controller
{
    /**
     * Display a listing discount types.
     *
     * @return Response
     */
    public function index(Request $request)
    {
        if (!$request->user()->can(['discount_types.read'])) {
            throw new AccessDeniedException(__('lang_permissions.discount_types.read.missing.permission'));
        }
        $discountTypes = DiscountType::orderBy('name')->get();
        return response(['data' => DiscountTypeResource::collection($discountTypes)]);
    }

    /**
     * Store a newly created discount type.
     *
     * @param  DiscountTypeRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function store(DiscountTypeRequest $request)
    {
        if (!$request->user()->can(['dentists.create'])) {
            throw new AccessDeniedException(__('lang_permissions.dentists.create.missing.permission'));
        }
        $validated = $request->validated();
        $discountType = DiscountType::create($validated);

        return response(['data' => new DiscountTypeResource($discountType)],201);
    }

    /**
     * Show the specified discount type.
     * @param  Request  $request
     * @param  DiscountType  $discountType
     *
     * @return Application|ResponseFactory|Response
     */
    public function show(Request $request,DiscountType $discountType)
    {
        if (!$request->user()->can(['discount_types.read'])) {
            throw new AccessDeniedException(__('lang_permissions.discount_types.read.missing.permission'));
        }
        return response(['data' => new DiscountTypeResource($discountType)]);
    }

    /**
     * Update the specified discount type.
     *
     * @param  DiscountTypeRequest  $request
     * @param  DiscountType  $discountType
     *
     * @return Application|ResponseFactory|Response
     */
    public function update(DiscountTypeRequest $request, DiscountType $discountType)
    {
        if (!$request->user()->can(['discount_types.update'])) {
            throw new AccessDeniedException(__('lang_permissions.discount_types.update.missing.permission'));
        }
        $validated = $request->validated();
        $discountType->update($validated);
        return response(['data' => new DiscountTypeResource($discountType)]);
    }

}
