<?php

namespace App\Http\Controllers;

use App\Http\Requests\InvoiceSeriesRequest;
use App\Http\Resources\InvoiceSeriesResource;
use App\Models\InvoiceSeries;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Exceptions\Api\AccessDeniedException;
use App\Services\HelperService;

/**
 * @group Facturi serii
 *
 * APIs for managing invoice series
 */
class InvoiceSeriesController extends Controller
{
    /**
     *  Display invoice series
     *
     * @param Request $request
     * @param null $invoiceProfileId
     * @return \Illuminate\Contracts\Routing\ResponseFactory|\Illuminate\Http\Response
     * @throws AccessDeniedException
     */
    public function index(Request $request, $invoiceProfileId = null)
    {
        if (!$request->user()->can(['invoice_series.read'])) {
            throw new AccessDeniedException(__('lang_permissions.invoice_series.read.missing.permission'));
        }
        $invoiceSeries = InvoiceSeries::select('invoice_series.*');

        if(isset($invoiceProfileId)){
            $invoiceSeries->where('lab_invoice_profile_id', $invoiceProfileId);
        }
        $invoiceSeries = $invoiceSeries->get();

        return response(['data' => InvoiceSeriesResource::collection($invoiceSeries)]);
    }


    /**
     * Create invoice serie
     *
     * @param InvoiceSeriesRequest $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(InvoiceSeriesRequest $request)
    {
        if (!$request->user()->can(['invoice_series.create'])) {
            throw new AccessDeniedException(__('lang_permissions.invoice_series.create.missing.permission'));
        }
        $validated = $request->validated();
        HelperService::existsDuplicateSeriesOnSameProfile(InvoiceSeries::class, $validated['lab_invoice_profile_id'], $validated['series'], true);
        DB::table('invoice_series')->where('lab_invoice_profile_id', $validated['lab_invoice_profile_id'])->update(['active' => 0]);
        $invoiceSerie = InvoiceSeries::create($validated);

        return response(['data' => new InvoiceSeriesResource($invoiceSerie)], 201);
    }

    /**
     * Activate invoice serie
     *
     * @param InvoiceSeries $invoiceSeries
     * @return \Illuminate\Contracts\Routing\ResponseFactory|\Illuminate\Http\Response
     */
    public function activate(Request $request, InvoiceSeries $invoiceSeries)
    {
        if (!$request->user()->can(['invoice_series.read'])) {
            throw new AccessDeniedException(__('lang_permissions.invoice_series.update.missing.permission'));
        }
        DB::table('invoice_series')->where('lab_invoice_profile_id', $invoiceSeries->lab_invoice_profile_id)->update(['active' => 0]);

        $invoiceSeries->active = 1;
        $invoiceSeries->save();

        return response(['data' => new InvoiceSeriesResource($invoiceSeries)]);
    }
}
