<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Http\Requests\LabInvoiceProfileFilesRequest;
use App\Http\Requests\LabInvoiceProfileRequest;
use App\Http\Resources\LabInvoiceProfileCollection;
use App\Http\Resources\LabInvoiceProfileResource;
use App\Http\Resources\LabProfileResource;
use App\Models\LabInvoiceProfile;
use App\Models\LabProfile;
use App\Services\UploadService;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

/**
 * @group Profiluri facturare
 *
 * APIs for managing lab invoice profiles
 */
class LabInvoiceProfileController extends Controller
{

    /**
     * Display a listing of  invoice profiles.
     *
     * @apiResourceCollection App\Http\Resources\LabInvoiceProfileCollection
     * @apiResourceModel App\Models\LabInvoiceProfile
     * @return Response
     */
    public function index(Request $request)
    {
        if (!$request->user()->can(['lab_invoice_profile.read'])) {
            throw new AccessDeniedException(__('lang_permissions.lab_invoice_profile.read.missing.permission'));
        }

        return response(new LabInvoiceProfileCollection(LabInvoiceProfile::all()));
    }


    /**
     * Store a newly created invoice profile.
     *
     * @apiResource App\Http\Resources\LabInvoiceProfileResource
     * @apiResourceModel App\Models\LabInvoiceProfile
     *
     * @param  LabInvoiceProfileRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function store(LabInvoiceProfileRequest $request)
    {
        if (!$request->user()->can(['lab_invoice_profile.create'])) {
            throw new AccessDeniedException(__('lang_permissions.lab_invoice_profile.create.missing.permission'));
        }

        $validated = $request->validated();
        $profile = LabInvoiceProfile::create($validated);

        return response(['data' => new LabInvoiceProfileResource($profile)], 201);
    }


    /**
     * Display a invoice profile.
     *
     * @apiResource App\Http\Resources\LabInvoiceProfileResource
     * @apiResourceModel App\Models\LabInvoiceProfile
     * @urlParam  lab_invoice_profile integer required
     *
     * @param  Request  $request
     * @param  LabInvoiceProfile  $labInvoiceProfile
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function show(Request $request, LabInvoiceProfile $labInvoiceProfile)
    {
        if (!$request->user()->can(['lab_invoice_profile.read'])) {
            throw new AccessDeniedException(__('lang_permissions.lab_invoice_profile.read.missing.permission'));
        }

        return response(['data' => new LabInvoiceProfileResource($labInvoiceProfile)]);
    }


    /**
     * Update a invoice profile.
     *
     * @apiResource App\Http\Resources\LabInvoiceProfileResource
     * @apiResourceModel App\Models\LabInvoiceProfile
     * @urlParam  lab_invoice_profile integer required
     *
     * @param  LabInvoiceProfileRequest  $request
     * @param  LabInvoiceProfile  $labInvoiceProfile
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function update(LabInvoiceProfileRequest $request, LabInvoiceProfile $labInvoiceProfile)
    {
        if (!$request->user()->can(['lab_invoice_profile.update'])) {
            throw new AccessDeniedException(__('lang_permissions.lab_invoice_profile.update.missing.permission'));
        }

        $validated = $request->validated();
        foreach ($validated as $fieldName => $fieldValue) {
            $labInvoiceProfile->$fieldName = $fieldValue;
        }
        $labInvoiceProfile->save();

        return response(['data' => new LabInvoiceProfileResource($labInvoiceProfile)]);
    }

    /**
     * Remove a invoice profile.
     *
     * @param  \App\Models\LabInvoiceProfile  $labInvoiceProfile
     *
     * @urlParam  lab_invoice_profile integer required
     *
     * @return Response
     */
    public function destroy(Request $request, LabInvoiceProfile $labInvoiceProfile)
    {
        if (!$request->user()->can(['lab_invoice_profile.delete'])) {
            throw new AccessDeniedException(__('lang_permissions.lab_invoice_profile.delete.missing.permission'));
        }
        $labInvoiceProfile->delete();
        return response(['message' => __('Successfully deleted')], 204);
    }

    /**
     * Set as default invoice profile for lab.
     *
     * @apiResource App\Http\Resources\LabProfileResource
     * @apiResourceModel App\Models\LabProfile
     *
     * @param  Request  $request
     * @param  LabInvoiceProfile  $labInvoiceProfile
     *
     * @urlParam  lab_invoice_profile integer required
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function makeDefaultInvoiceProfile(Request $request, LabInvoiceProfile $labInvoiceProfile)
    {
        if (!$request->user()->can(['lab_invoice_profile.update'])) {
            throw new AccessDeniedException(__('lang_permissions.lab_invoice_profile.update.missing.permission'));
        }
        $labProfile = LabProfile::firstOrFail();
        $labProfile->lab_invoice_profile_id = $labInvoiceProfile->id;
        $labProfile->save();

        return response(['data' => new LabProfileResource($labProfile)]);
    }

    /**
     * Update a lab invoice logo and/or cover.
     *
     * @apiResource App\Http\Resources\LabInvoiceProfileResource
     * @apiResourceModel App\Models\LabInvoiceProfile
     * @param  LabInvoiceProfileFilesRequest  $request
     * @urlParam lab_invoice_profile integer required
     * @bodyParam logo file
     * @bodyParam cover file
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws \App\Exceptions\Api\ServerError
     * @throws \App\Exceptions\Api\ValidationException
     */
    public function updateLogoAndCover(LabInvoiceProfileFilesRequest $request, LabInvoiceProfile $labInvoiceProfile)
    {
        if (!$request->user()->can(['lab_invoice_profile.update'])) {
            throw new AccessDeniedException(__('lang_permissions.lab_invoice_profile.update.missing.permission'));
        }
        $validated = $request->validated();

        foreach ($validated as $name => $fileData) {
            (new UploadService())->savefile($labInvoiceProfile, $request->file($name),$name,UploadService::PUBLIC_UPLOAD_PATH);
        }
        $labInvoiceProfile->refresh();

        return response(['data' => new LabInvoiceProfileResource($labInvoiceProfile)]);
    }
}
