<?php

namespace App\Http\Controllers;


use App\Exceptions\Api\AccessDeniedException;
use App\Http\Requests\LabProfileAdditionalDetailsRequest;
use App\Http\Requests\LabProfileFilesRequest;
use App\Http\Requests\LabProfileRequest;
use App\Http\Requests\LabProfileSettingsRequest;
use App\Http\Resources\LabProfileResource;
use App\Models\LabProfile;
use App\Services\LabProfileService;
use App\Services\UploadService;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

/**
 * @group Profil laborator
 *
 * APIs for managing laboratory's profile
 */
class LabProfileController extends Controller
{
    var $labProfile = null;

    public function __construct()
    {
        //        $this->labProfile = LabProfile::firstOrFail();
    }

    /**
     * Lab profile.
     *
     * @apiResource App\Http\Resources\LabProfileResource
     * @apiResourceModel App\Models\LabProfile
     *
     * @param  Request  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function show(Request $request)
    {
        if (!$request->user()->can(['lab_profile.read'])) {
            throw new AccessDeniedException(__('lang_permissions.lab_profile.read.missing.permission'));
        }

        $labProfile = LabProfile::firstOrFail();
        return response(['data' => new LabProfileResource($labProfile)]);
    }


    /**
     * Update lab profile.
     *
     * @apiResource App\Http\Resources\LabProfileResource
     * @apiResourceModel App\Models\LabProfile
     *
     * @param  LabProfileRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function update(LabProfileRequest $request)
    {
        if (!$request->user()->can(['lab_profile.update'])) {
            throw new AccessDeniedException(__('lang_permissions.lab_profile.update.missing.permission'));
        }

        $validated = $request->validated();
        $this->labProfile = LabProfile::firstOrFail();
        $labProfile = $labProfile = $this->updateLabProfileFields($validated, $this->labProfile);

        return response(['data' => new LabProfileResource($labProfile)]);
    }


    /**
     * Update lab settings.
     *
     * @apiResource App\Http\Resources\LabProfileResource
     * @apiResourceModel App\Models\LabProfile
     *
     * @param  LabProfileSettingsRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws \Throwable
     */
    public function updateSettings(LabProfileSettingsRequest $request)
    {
        if (!$request->user()->can(['lab_profile.update'])) {
            throw new AccessDeniedException(__('lang_permissions.lab_profile.update.missing.permission'));
        }

        $validated = $request->validated();
        $this->labProfile = LabProfile::firstOrFail();
        $labProfile  = $this->updateLabProfileFields($validated, $this->labProfile);

        return response(['data' => new LabProfileResource($labProfile)]);
    }


    /**
     * Update lab logo and/or cover.
     *
     * @apiResource App\Http\Resources\LabProfileResource
     * @apiResourceModel App\Models\LabProfile
     * @param  LabProfileFilesRequest  $request
     * @bodyParam logo file
     * @bodyParam cover file
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws \App\Exceptions\Api\ServerError
     * @throws \App\Exceptions\Api\ValidationException
     */
    public function updateLogoAndCover(LabProfileFilesRequest $request)
    {
        if (!$request->user()->can(['lab_profile.update'])) {
            throw new AccessDeniedException(__('lang_permissions.lab_profile.update.missing.permission'));
        }
        $validated = $request->validated();

        $this->labProfile = LabProfile::firstOrFail();

        foreach ($validated as $name => $fileData) {
            (new UploadService())->savefile($this->labProfile, $request->file($name), $name, UploadService::PUBLIC_UPLOAD_PATH);
        }
        $this->labProfile->refresh();

        return response(['data' => new LabProfileResource($this->labProfile)]);
    }

    /**
     * Update lab profile's additional details.
     *
     * @param LabProfileAdditionalDetailsRequest $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function updateAdditionalDetails(LabProfileAdditionalDetailsRequest $request)
    {
        if (!$request->user()->can(['lab_profile.update'])) {
            throw new AccessDeniedException(__('lang_permissions.lab_profile.update.missing.permission'));
        }

        $validated = $request->validated();
        $this->labProfile = LabProfile::firstOrFail();
        $labProfile = $this->updateLabProfileFields($validated, $this->labProfile);

        return response(['data' => new LabProfileResource($labProfile)]);
    }

    /**
     * @param  array  $validated
     * @param  LabProfile  $labProfile
     *
     * @return User
     * @throws \Throwable
     */
    public function updateLabProfileFields(array $validated, LabProfile $labProfile): LabProfile
    {
        return (new LabProfileService)->updateLabProfileFields($validated, $labProfile);
    }
}
