<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Exceptions\Api\ValidationException;
use App\Http\Requests\OAPAddProductRequest;
use App\Http\Requests\OAPriceAddRequest;
use App\Http\Requests\OAPriceListingRequest;
use App\Http\Requests\OAPriceUpdateRequest;
use App\Http\Resources\OAdditionalPriceResource;
use App\Http\Resources\OrderResource;
use App\Models\Order;
use App\Models\OrderAdditionalPrice;
use App\Scopes\OrderAdditionalPriceScope;
use App\Services\OrderService;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Response;

/**
 * @group Comenzi preturi aditionale
 *
 * APIs for managing order additional prices
 */
class OrderAdditionalPriceController extends Controller
{

    /**
     *
     * Specific order additional prices
     *
     * @param  OAPriceListingRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function getOrderAdditionalPrices(OAPriceListingRequest $request)
    {
        if (!$request->user()->can(['orders.read'])) {
            throw new AccessDeniedException(__('lang_permissions.orders.read.missing.permission'));
        }
        $validated = $request->validated();
        $oAPrices = OrderService::getOrderIdAdditionalPricesListing($validated['order_id']);
        return response(['data' => OAdditionalPriceResource::collection($oAPrices)]);
    }

    /**
     * Add a additional price
     *
     * @param  \Illuminate\Http\Request  $request
     *
     * @return Response
     */
    public function store(OAPriceAddRequest $request)
    {
        $validated = $request->validated();
        $order = Order::findOrFail($validated['order_id']);
        OrderService::orderCanBeUpdated($order);
        $oAPrice = $this->addNewAdditionalPrice($validated);

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($oAPrice->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(['data' => new OAdditionalPriceResource($oAPrice)], 201);
    }

    /**
     * Update an additional price
     *
     * @param OAPriceUpdateRequest $request
     * @param OrderAdditionalPrice $orderAdditionalPrice
     * @return Response
     */
    public function update(OAPriceUpdateRequest $request, $orderAdditionalPriceId)
    {
        $validated = $request->validated();

        $orderAdditionalPrice = OrderAdditionalPrice::withoutGlobalScope(OrderAdditionalPriceScope::class)->findOrFail($orderAdditionalPriceId);
        $orderAdditionalPrice->to_be_invoiced = $validated['to_be_invoiced'];
        $orderAdditionalPrice->save();

        return response(['data' => new OAdditionalPriceResource($orderAdditionalPrice)], 200);
    }

    /**
     * @param  array  $validated
     *
     * @return OrderAdditionalPrice
     */
    public function addNewAdditionalPrice(array $parameters): OrderAdditionalPrice
    {
        $oAPrice                    = new OrderAdditionalPrice();
        $oAPrice->order_id          = $parameters['order_id'];
        $oAPrice->name              = $parameters['name'];
        $oAPrice->description       = $parameters['description'] ?? '';
        $oAPrice->quantity          = $parameters['quantity'] ?? 1;
        $oAPrice->price_per_unit    = $parameters['price_per_unit'];
        $oAPrice->to_be_invoiced    = $parameters['to_be_invoiced'] ?? true;

        if ($oAPrice->quantity * $oAPrice->price_per_unit > 999999) {
            throw new ValidationException(__('Total additional price to large. price_per_unit*quantity should be less then 999999'));
        }

        $oAPrice->price             = $oAPrice->quantity * $oAPrice->price_per_unit;

        $oAPrice->save();
        return $oAPrice;
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\OrderAdditionalPrice  $orderAdditionalPrice
     *
     * @return Response
     */
    public function destroy($orderAdditionalPriceId)
    {
        $orderAdditionalPrice = OrderAdditionalPrice::withoutGlobalScope(OrderAdditionalPriceScope::class)->findOrFail($orderAdditionalPriceId);
        OrderService::orderIdCanBeUpdated($orderAdditionalPrice->order_id);
        $orderAdditionalPrice->delete();
        return response(['data' => __('Order additional price deleted')], 204);
    }

    /**
     *
     * Add product as additional price
     *
     * @param  OAPAddProductRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws \App\Exceptions\Api\ValidationException
     */
    public function addProductAsOrderAdditionalPrice(OAPAddProductRequest $request)
    {
        $validated = $request->validated();
        OrderService::orderIdCanBeUpdated($validated['order_id']);

        $oAPParametersFromProduct = AdditionalPricesProductController::generateParametersForAddingAsAdditionalPrice($validated);
        $oAPrice = $this->addNewAdditionalPrice($oAPParametersFromProduct);

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($oAPrice->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(['data' => new OAdditionalPriceResource($oAPrice)], 201);
    }
}
