<?php

namespace App\Http\Controllers;

use App\Configuration\LabConfiguration;
use App\Exceptions\Api\AccessDeniedException;
use App\Exceptions\Api\PossibleUnSyncException;
use App\Exceptions\Api\ServerError;
use App\Exceptions\Api\ValidationException;
use App\Http\Requests\OrderCancelRequest;
use App\Http\Requests\OrderDeleteEmptySpecialStagesIdsRequest;
use App\Http\Requests\OrderDeleteEmptySpecialStagesRequest;
use App\Http\Requests\OrderDetailsRequest;
use App\Http\Requests\OrderEmptySpecialStagesRequest;
use App\Http\Requests\OrderOverviewTimesRequest;
use App\Http\Requests\OrderSendToProductionRequest;
use App\Http\Requests\OrderStoreRequest;
use App\Http\Resources\OrderCollection;
use App\Http\Resources\OrderEmptySpecialStageResource;
use App\Http\Resources\OrderResource;
use App\Models\Order;
use App\Models\OrderWorkStage;
use App\Models\UserWorks;
use App\Scopes\OrderScope;
use App\Services\OrderCancelService;
use App\Services\OrderService;
use App\Services\OrderWorkTemplateService;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Response;


/**
 * @group Comenzi
 *
 * APIs for managing work stages
 */
class OrderController extends Controller
{

    /**
     * Display a listing of orders.
     *
     * @param  Request  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function index(Request $request)
    {
        if (!$request->user()->can(['orders.read'])) {
            throw new AccessDeniedException(__('lang_permissions.orders.read.missing.permission'));
        }

        $orders = $this->ordersList();
        foreach ($orders as &$order) {
            $order->actions = OrderService::possibleOrderActions($order);
        }

        return response(new OrderCollection($orders));
    }

    /**
     * Display a listing of canceled orders.
     *
     * @return Response
     */
    public function indexCancelled(Request $request)
    {
        if (!$request->user()->can(['orders.read'])) {
            throw new AccessDeniedException(__('lang_permissions.orders.read.missing.permission'));
        }

        $orders = $this->orderListQuery()->withoutGlobalScope(OrderScope::class)->where('cancelled', '<>', \DB::raw(0))->get();
        foreach ($orders as &$order) {
            $order->actions = OrderService::possibleOrderActions($order);
        }

        return response(new OrderCollection($orders));
    }


    /**
     * Add new order
     *
     * @param  OrderStoreRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function store(OrderStoreRequest $request)
    {
        if (!$request->user()->can(['orders.create'])) {
            throw new AccessDeniedException(__('lang_permissions.orders.create.missing.permission'));
        }

        $validated = $request->validated();
        // order statuses
        $orderStatuses = LabConfiguration::orderStatuses();
        $firstStatus = $orderStatuses[0]['value'];

        $orderNumber = OrderService::getNextOrderNumber();

        $newOrder = new Order();
        $newOrder->order_no = $orderNumber;
        $newOrder->date_added = now();
        $newOrder->status = $firstStatus;
        foreach ($validated as $fieldName => $fieldValue) {
            $newOrder->$fieldName = $fieldValue;
        }
        $newOrder->save();
        $orderDetails = $this->getOrderWithDetails($newOrder->id);

        return response(['data' => (new OrderResource($orderDetails))], 201);
    }

    /**
     * Display a order.
     *
     * @param  \App\Models\Order  $order
     *
     * @return Response
     */
    public function show(Request $request, $orderId)
    {
        if (!$request->user()->can(['orders.read'])) {
            throw new AccessDeniedException(__('lang_permissions.orders.read.missing.permission'));
        }

        $order = $this->getOrderWithDetails((int)$orderId);

        return response(['data' => (new OrderResource($order))]);
    }

    /**
     * Update a order.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Order  $order
     *
     * @return Response
     */
    public function update(OrderStoreRequest $request, Order $order)
    {
        $validated = $request->validated();
        OrderService::orderCanBeUpdated($order);
        $order->update($validated);
        $updatedOrder = $this->getOrderWithDetails($order->id);
        return response(['data' => new OrderResource($updatedOrder)]);
    }

    /**
     * Delete a order.
     *
     * @param  \App\Models\Order  $order
     *
     * @return Response
     */
    public function destroy(Order $order)
    {
        OrderService::orderCanBeDeleted($order, true);
        //todo delete order with work and stages de verificat daca stergerea este corecta
        $order->forceDelete();
        return response(['data' => new OrderResource($order)], 204);
    }


    /**
     * Finish a order.
     *
     * @param  Request  $request
     * @param  Order  $order
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws PossibleUnSyncException
     */
    public function finishOrder(Order $order)
    {
        OrderService::orderCanBeFinished($order, true);

        \DB::transaction(
            function () use ($order) {
                $order->status = 'complete';
                $order->finish_date = \DB::raw('now()');

                // update order value
                $value = OrderService::calculateTotalPrice($order->id);

                $order->value_finish_order = $value;
                if ($order->payment_document_id == 0) {
                    $order->value = $value;
                }

                $order->save();
            }
        );

        $updatedOrder = $this->getOrderWithDetails($order->id);
        return response(['data' => new OrderResource($updatedOrder)]);
    }

    /**
     * Cancel order
     *
     * @param OrderCancelRequest $request
     * @param Order $order
     * @return ResponseFactory|Response
     * @throws PossibleUnSyncException
     * @throws ServerError
     * @throws ValidationException
     */
    public function cancelOrder(OrderCancelRequest $request, Order $order)
    {
        //todo check this method when implementing paymets
        OrderService::orderCanBeCanceled($order);

        $validated = $request->validated();

        $orderCancel = new OrderCancelService($order->id, $validated['reason'], $validated['appears_on_payment_document'], $validated['canceled_order_stages_updated_costs']);

        $orderCancel->cancelOrder($validated['remake']);

        $updatedOrder = $this->getOrderWithDetails($orderCancel->getOrder()->id);
        return response(['data' => new OrderResource($updatedOrder)]);
    }

    /**
     * @param $dentistId
     * @param $collaboratorId
     * @param $workTemplateId
     *
     * @return \Illuminate\Support\Collection
     */
    public static function getTemplatePrice($dentistId, $collaboratorId, $workTemplateId)
    {
        return WorkTemplateController::getTemplatePrices($dentistId, $collaboratorId, $workTemplateId)->first()->toArray();
    }

    public function getOrderWithDetails($orderId)
    {
        $order = $this->orderQueryDetailed((int)$orderId);

        $order = OrderService::addTotalPriceWithOrder($order);

        $calculatedDuration = OrderService::calculateDays($order, $order->works);

        if (!isset($calculatedDuration['finishDate'])) {
            throw  new  ServerError(__('#EO003 Could not calculate order finish date'));
        }
        $order->finishDate = $calculatedDuration['finishDate'];

        $order = $this->calculateWorksDates($order, $calculatedDuration);

        $order = OrderService::calculation($order);

        return $order;
    }

    public function orderQueryDetailed(int $orderId)
    {
        return Order::select(['orders.*'])->with(
            ['collaborator:id,name', 'dentist:id,first_name,last_name', 'patient:id,first_name,last_name']
        )->with(
            [
                'works' => function ($q) {
                    $q->with(
                        ['stagesWithoutPivot.workStage.workTemplateStageUsers:username,first_name,last_name', 'stagesWithoutPivot.work']
                    )->with('combinations')->where('combination_id', 0);
                }
            ]
        )

            //            ->with('files')
            //            ->with('details')
            ->with(['additionalPrices', 'additionalPricesWithoutGlobalScope'])->withoutGlobalScope(OrderScope::class)->joinFirstAvailableStage()->findOrFail($orderId);
    }

    public function calculateWorksDates(Order $order, $calculatedDuration)
    {
        foreach ($order->works as $work) {
            if (empty($calculatedDuration['works'][$work->id])) {
                continue;
            }

            $work->estimatedDate = $calculatedDuration['works'][$work->id]['estimated_date'];
            $work->minDate = $calculatedDuration['works'][$work->id]['min_date'];
            $work->preferredDate = $calculatedDuration['works'][$work->id]['preferred_date'];
        }

        return $order;
    }

    /**
     *
     * Send a order in production
     *
     * @param  OrderSendToProductionRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws ValidationException
     */
    public function sendOrderToProduction(OrderSendToProductionRequest $request)
    {
        $validated = $request->validated();

        $order = Order::findOrFail($validated['order_id']);

        OrderService::orderCanBeSentToProd($order);

        OrderService::hasAllStagesWithUserAssigned($order, true);

        if (!OrderService::sendOrderStagesToProd($order)) {
            throw new ServerError(__('#OWSESP Could not send stages in production.'));
        }

        $order->status = 'waiting';
        $order->save();

        $updatedOrder = $this->getOrderWithDetails($order->id);
        return response(['data' => new OrderResource($updatedOrder)]);
    }


    /**
     * Order list possibles actions
     *
     * @return Application|ResponseFactory|Response
     */
    public function ordersListPossibleActions()
    {
        $orders = $this->ordersList();

        foreach ($orders as &$order) {
            $order->actions = OrderService::possibleOrderActions($order);
        }

        return response(new OrderCollection($orders));
    }

    /**
     * @return mixed
     */
    public function ordersList()
    {
        $orders = $this->orderListQuery();

        return $orders->get();
    }


    /**
     *
     * Order empty special stages
     *
     * @param  OrderEmptySpecialStagesRequest  $request
     * @param  bool  $returnAsResource
     *
     * @return array|Application|ResponseFactory|Response
     */
    public function orderEmptySpecialStages(OrderEmptySpecialStagesRequest $request, $returnAsResource = true)
    {
        $validated = $request->validated();
        $emptySpecialStages = $this->getOrderEmptySpecialStages($validated['order_id']);
        if ($returnAsResource) {
            return response(['data' => OrderEmptySpecialStageResource::collection($emptySpecialStages)]);
        }

        return $emptySpecialStages;
    }

    public function getOrderEmptySpecialStages($orderId)
    {
        $order = $this->orderQueryDetailed($orderId);
        $emptySpecialStages = [];
        $works = $order->works ?? [];
        foreach ($works as $work) {
            $stages = $work->stagesWithoutPivot ?? [];
            foreach ($stages as $stage) {
                if (!$stage->multi_factor && OrderService::isSpecialStage($stage->type)) {
                    $emptySpecialStages[] = $stage;
                }
            }
        }

        return $emptySpecialStages;
    }

    /**
     *
     * Delete specific id`a of empty special stages from order
     *
     * @param  OrderDeleteEmptySpecialStagesIdsRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws ValidationException
     */
    public function deleteOrderEmptySpecialStagesIds(OrderDeleteEmptySpecialStagesIdsRequest $request)
    {
        $validated = $request->validated();
        OrderService::orderIdCanBeUpdated($validated['order_id']);
        $emptySpecialStages = collect($this->getOrderEmptySpecialStages($validated['order_id']));

        $worksToReorder = [];
        foreach ($validated['stages_ids'] as $stageId) {
            $stage = $emptySpecialStages->where('id', $stageId)->first();
            if ($stage === null) {
                throw new ValidationException(__('Stage id: :stageId is not from order or can not be deleted', ['stageId' => $stageId]));
            }
            $worksToReorder[] = $stage->order_work_template_id;
        }

        \DB::transaction(
            function () use ($emptySpecialStages, $validated) {
                $stagesToDelete = $emptySpecialStages->whereIn('id', $validated['stages_ids'])->all();
                foreach ($stagesToDelete as $item) {
                    $item->delete();
                }
            }
        );

        OrderWorkTemplateService::reorderStagesForOrderWorks($worksToReorder);

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($validated['order_id']);
            return response(['data' => new OrderResource($order)]);
        }

        return response(['message' => __("Empty order stages deleted"), 204]);
    }

    /**
     *
     * Delete specific id`a of empty special stages from order
     *
     * @param  OrderDeleteEmptySpecialStagesRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws ValidationException
     */
    public function deleteOrderEmptySpecialStages(OrderDeleteEmptySpecialStagesRequest $request)
    {
        $validated = $request->validated();
        OrderService::orderIdCanBeUpdated($validated['order_id']);
        $emptySpecialStages = collect($this->getOrderEmptySpecialStages($validated['order_id']));

        $worksToReorder = $emptySpecialStages->pluck('order_work_template_id')->toArray();

        \DB::transaction(
            function () use ($emptySpecialStages) {
                foreach ($emptySpecialStages as $item) {
                    $item->delete();
                }
            }
        );

        OrderWorkTemplateService::reorderStagesForOrderWorks($worksToReorder);

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($validated['order_id']);
            return response(['data' => new OrderResource($order)]);
        }

        return response(['message' => __("Empty order stages deleted"), 204]);
    }

    /**
     * @return mixed
     */
    public function orderListQuery()
    {
        $orders = Order::select(['orders.*']);
        $orders->joinFirstAvailableStage();
        $orders->with(['collaborator:id,name', 'dentist:id,first_name,last_name', 'patient:id,first_name,last_name', 'worksNameAndSymbol']);
        $orders->with(
            [
                'hasRemake' => function ($q) {
                    $q->withoutGlobalScope(OrderScope::class);
                }
            ]
        )->with(
            [
                'remakeOf' => function ($q) {
                    $q->withoutGlobalScope(OrderScope::class);
                }
            ]
        )->orderByDesc('date_added');
        return $orders;
    }

    /**
     * Get overview times
     *
     * @param  OrderOverviewTimesRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    function getOrderOverviewTimes(OrderOverviewTimesRequest $request)
    {
        if (!$request->user()->can(['orders.read'])) {
            throw new AccessDeniedException(__('lang_permissions.orders.read.missing.permission'));
        }
        $validated = $request->validated();
        $orderOverviewTimes = $this->getOrderOverviewTimesQueryResult($validated['order_id']);

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($validated['order_id']);
            $order->overview_times = $orderOverviewTimes;
            return response(['data' => new OrderResource($order)]);
        }

        return response(['data' => $orderOverviewTimes]);
    }

    function getOrderOverviewTimesQueryResult(int $orderId)
    {
        $ids = OrderWorkStage::where('order_id', $orderId)->pluck('id')->toArray();

        $query = UserWorks::select(
            'order_work_stage_id',
            \DB::raw('MIN(user_works.start_time) as start'),
            \DB::raw("MAX(IFNULL(user_works.stop_time, 'working')) as stop"),
            \DB::raw(
                "SUM(
                    TIMESTAMPDIFF(SECOND, user_works.start_time, IFNULL(user_works.stop_time, NOW()))
                    ) AS seconds"
            )
        )->groupBy('order_work_stage_id')->whereIn('order_work_stage_id', $ids)->get()->keyBy('order_work_stage_id');

        return $query;
    }

    /**
     * Update order details.
     *
     * @param OrderDetailsRequest $request
     * @param Order $order
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    function updateOrderDetails(OrderDetailsRequest $request, Order $order)
    {
        OrderService::orderCanBeUpdated($order);

        $validated = $request->validated();
        $order->update([
            'dentition_color'   => $validated['dentition_color'] ?? "",
            'dentition_shape'   => $validated['dentition_shape'] ?? "",
            'face_shape'        => $validated['face_shape'] ?? "",
            'order_details'     => $validated['order_details'] ?? "",
        ]);

        $updatedOrder = $this->getOrderWithDetails($order->id);
        return response(['data' => new OrderResource($updatedOrder)]);
    }
}
