<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Exceptions\Api\PossibleUnSyncException;
use App\Exceptions\Api\ServerError;
use App\Exceptions\Api\ValidationException;
use App\Http\Requests\OrderStagesPossibleActionsRequest;
use App\Http\Requests\OWSSpecialStagesRequest;
use App\Http\Requests\OWStageDeleteRequest;
use App\Http\Requests\OWStageRemakeRequest;
use App\Http\Requests\OWStageReorderRequest;
use App\Http\Requests\OWStageTimerRequest;
use App\Http\Requests\OWStageUpdateUserRequest;
use App\Http\Requests\OWStageUpdateUserWithWorkPriceRequest;
use App\Http\Requests\OWSUpdateDateRequest;
use App\Http\Requests\OWTAddNewStageRequest;
use App\Http\Resources\OrderResource;
use App\Http\Resources\OrderWorkStageResource;
use App\Http\Resources\OrderWorkTemplateResource;
use App\Models\OrderWorkStage;
use App\Models\OrderWorkTemplate;
use App\Models\User;
use App\Models\WorkStage;
use App\Services\OrderService;
use App\Services\OrderWorkStagesService;
use App\Services\OrderWorkTemplateService;
use App\Services\UserService;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Response;

/**
 * @group Comenzi etape lucrare
 *
 * APIs for managing order work stages
 */
class OrderWorkStageController extends Controller
{

    /**
     * Reorder order work stages
     *
     * @param  OWStageReorderRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws ValidationException
     */
    public function reorderWorkStages(OWStageReorderRequest $request)
    {
        $validated = $request->validated();

        $orderT = OrderWorkTemplate::findOrFail($validated['order_work_template_id']);
        OrderService::orderIdCanBeUpdated($orderT->order_id);

        $owtStages = $orderT->stagesWithoutPivot()->with(['workStage.workTemplateStageUsers'])->get();
        $owtStagesIds = $owtStages->pluck('id')->toArray();
        $this->validateOrderStageIds($validated['order_work_stages'], $owtStagesIds);

        $rank = 0;
        $stageCanBeFinished = true;
        $stagesOrder = [];
        foreach ($validated['order_work_stages'] as $index => $element) {
            $stagesOrder[$index] = $owtStages->where('id', $element['stage_id'])->where(
                'order_work_template_id',
                $validated['order_work_template_id']
            )->first();
            if ($stagesOrder[$index]->status === 'finished') {
                if (!$stageCanBeFinished) {
                    throw new PossibleUnSyncException(__('Supplied ranks are invalid. Finished stage rank invalid'));
                }
            }
            if ($stagesOrder[$index]->status !== 'finished') {
                $stageCanBeFinished = false;
            }
            $stagesOrder[$index]->rank = $rank++;
        }

        foreach ($stagesOrder as $stage) {
            $stage->save();
        }

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($orderT->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(['message' => __("Work stages reordered")]);
    }

    /**
     * Set order stage user with default price
     *
     * <aside> user_id can be only a user assigned to the stage of work template</aside>
     *
     * @param  OWStageUpdateUserWithWorkPriceRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws ValidationException
     */
    public function updateUserWithWorkTemplatePrice(OWStageUpdateUserWithWorkPriceRequest $request)
    {
        $validated = $request->validated();
        $orderT = OrderWorkTemplate::findOrFail($validated['order_work_id']);
        $owtStage = $orderT->stagesWithoutPivot()->where('id', $validated['order_work_stage_id'])->with(
            ['workStage.workTemplateStageUsers']
        )->findOrFail($validated['order_work_stage_id']);

        OrderWorkStagesService::canStageUserBeModified($owtStage, true);

        $this->validateIsUserAttachedToOrderStage($validated['order_work_stage_id'], $validated['user_id'], $owtStage, true);

        $user = $owtStage->workStage->workTemplateStageUsers->where('id', $validated['user_id'])->first();
        $owtStage->user_id = $user->id;
        $owtStage->user_name = $user->first_name.' '.$user->last_name;
        $owtStage->cost = $user->pivot->price;
        $owtStage->save();

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($orderT->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(['data' => new OrderWorkStageResource($owtStage)]);
    }

    /**
     * @param  array  $toBeValidatedIds
     * @param  array  $dbOrderTemplateStagesIds
     *
     * @throws ValidationException
     */
    public function validateOrderStageIds(array $toBeValidatedIds, array $dbOrderTemplateStagesIds): void
    {
        if (count($toBeValidatedIds) !== count($dbOrderTemplateStagesIds)) {
            throw new ValidationException(__('Invalid number of stages'));
        }
        $existentIds = [];
        foreach ($toBeValidatedIds as $element) {
            $stageId = $element['stage_id'];
            if (!in_array($stageId, $dbOrderTemplateStagesIds)) {
                throw new ValidationException(__('Invalid order stage id:').$stageId);
            }
            if (in_array($stageId, $existentIds)) {
                throw new ValidationException(__('Duplicate order stage id:').$stageId);
            }

            $existentIds[] = $stageId;
        }
    }


    /**
     * @param $stageId
     * @param $userId
     * @param $stages
     *
     * @throws ValidationException
     */
    public function validateIsUserAttachedToOrderStage($stageId, $userId, $stage, $throwException = false): void
    {
        if (!isset($stage->workStage->workTemplateStageUsers)) {
            throw new ValidationException(__('Invalid user_id for stage :stageId', ['stageId' => $stageId]));
        }

        $userExist = false;
        foreach ($stage->workStage->workTemplateStageUsers as $workTemplateStageUser) {
            if ($workTemplateStageUser->id == $userId) {
                $userExist = true;
            }
        }

        if (!$userExist) {
            throw new ValidationException(__('User_id does not exists for stage :stageId', ['stageId' => $stageId]));
        }
    }

    /**
     * Add new order work stage.
     *
     * @param  OWTAddNewStageRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws ValidationException
     */
    public function newOrderWorkStage(OWTAddNewStageRequest $request)
    {
        $validated = $request->validated();

        $orderWork = OrderWorkTemplate::findOrFail($validated['order_work_template_id']);
        OrderService::orderIdCanBeUpdated($orderWork->order_id, true);

        $order = $orderWork->order()->firstOrFail();
        //setting order stage status by order status
        $stageStatus = 'not_ready';
        if ($order->status == 'creating') {
            $stageStatus = 'config';
        }

        $userId = 0;
        $workTemplate = $orderWork->workTemplate()->first();
        $workStage = $workTemplate->getStagesWithUsers($validated['work_stage_id']);

        if (!empty($workTemplate) && !empty($workStage[0])) {
            $workStage = $workStage[0];
            $userId = $workStage['user_id'];
        }

        // find out if work stage exists
        $workStage = !empty($workStage) ? $workStage : WorkStage::findOrFail($validated['work_stage_id'])->toArray();
        if (empty($workStage)) {
            throw new ValidationException(__("Could not fount work stage: ").$validated['work_stage_id']);
        }

        $rank = $validated['rank'];

        $dentConfig = OrderWorkStagesService::getDentConfig($orderWork, (array)$workStage);

        $orderWork->stages()->attach(
            $workStage['id'],
            [
                'work_template_id'       => $orderWork->work_template_id,
                'rank'                   => $rank,
                'user_id'                => $userId,
                'order_id'               => $orderWork->order_id,
                'price'                  => $workStage['special_cost'],
                'remake_price'           => $workStage['remake_cost'],
                'extra_cost_applied'     => $workStage['extra_cost_applied'],
                'price_type'             => $workStage['price_type'],
                'name'                   => $workStage['name'],
                'type'                   => $workStage['type'],
                'to_send'                => $workStage['to_send'],
                'from_template'          => 0,
                'arcada_dentition'       => $dentConfig['arcada_dentition'],
                'status'                 => $stageStatus,
                'multi_factor'           => OrderWorkStagesService::calculateMultiFactor(
                    $workStage['type'],
                    $workStage['price_type'],
                    json_decode($dentConfig['dentition']),
                    json_decode($dentConfig['arcada_dentition'])
                ),
                'user_name'              => $workStage['user_name'] ?? '',
                'cost'                   => !$userId ? 0 : $workStage['cost'],
                'duration'               => $workStage['min_duration'],
                'elements'               => $dentConfig['elements'],
                'multi_factor_dentition' => $workStage['multi_factor_dentition'],
                'dentition'              => $dentConfig['dentition'],
                'works'                  => json_encode(OrderWorkStagesService::getWorksByWorkId($orderWork->id))
            ]
        );

        OrderWorkTemplateService::reorderStages($orderWork->id);

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($orderWork->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        if ($orderWork->isDirty()) {
            $orderWork->refresh();
        }
        return response(['data' => new OrderWorkTemplateResource($orderWork)]);
    }

    /**
     *
     * Delete a order work stage.
     *
     * @param  OWStageDeleteRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws ValidationException
     * @throws \App\Exceptions\Api\PossibleUnSyncException
     */
    function deleteOrderWorkStage(OWStageDeleteRequest $request)
    {
        $validated = $request->validated();
        $owStage = OrderWorkStage::where('id', $validated['order_work_stage_id'])->where(
            'order_work_template_id',
            $validated['order_work_id']
        )->first();

        if ($owStage === null) {
            throw new ValidationException(__("Order stage does not exists"));
        }

        OrderService::orderIdCanBeUpdated($owStage->order_id, true);
        OrderWorkStagesService::canStageBeDeleted($owStage, true);

        $owStage->forceDelete();

        OrderWorkTemplateService::reorderStages($owStage->order_work_template_id);

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($owStage->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(["data" => __('Successfully deleted')], 204);
    }


    /**
     * @param  OrderWorkStage  $orderStage
     * @param $elementsData
     *
     * @throws ValidationException
     */
    public function confirmOrderStage(OrderWorkStage $orderStage, $elementsData): void
    {
        if ($orderStage->price_type !== 'fix') {
            throw new ValidationException(__("Order work stage price type mismatch"));
        }

        $confirmed = $elementsData ?? false;
        $orderStage->confirmed = $confirmed;
        $orderStage->multi_factor = 0;
        if ($confirmed) {
            $orderStage->multi_factor = 1;
        }
        $orderStage->save();
    }


    /**
     * @param  OrderWorkStage  $orderStage
     * @param $elementsData
     *
     * @throws ValidationException
     */
    public function updateStageArcadaDentition(OrderWorkStage $orderStage, $elementsData): void
    {
        if ($orderStage->price_type !== 'arcada') {
            throw new ValidationException(__("Order work stage price type mismatch"));
        }

        $elements = 0;
        foreach ($elementsData as $arcada) {
            if ($arcada) {
                $elements++;
            }
        }

        $orderStage->arcada_dentition = json_encode($elementsData);
        $orderStage->elements = $elements;
        $orderStage->multi_factor = $elements;
        $orderStage->save();
    }


    /**
     * @param  OrderWorkStage  $orderStage
     * @param $elementsData
     *
     * @throws ValidationException
     */
    public function updateStageByElements(OrderWorkStage $orderStage, $elementsData): void
    {
        if ($orderStage->price_type !== 'elemente') {
            throw new ValidationException(__("Order work stage price type mismatch"));
        }

        $elements = 0;
        foreach ($elementsData as $dent) {
            if ($dent) {
                $elements++;
            }
        }

        $orderStage->dentition = json_encode($elementsData);
        $orderStage->elements = $elements;

        $multiFactor = 1;
        if ($orderStage->price_type !== 'fix') {
            $multiFactor = $elements;
        }
        $orderStage->multi_factor = $multiFactor;

        $orderStage->save();
    }


    /**
     * Confirm(check/uncheck) element type special order stage
     *
     * @param  OWSSpecialStagesRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws ServerError
     * @throws ValidationException
     */
    public function updateSpecialStages(OWSSpecialStagesRequest $request)
    {
        $validated = $request->validated();
        $work = OrderWorkTemplate::findOrFail($validated['order_work_id']);
        OrderService::orderIdCanBeUpdated($work->order_id);

        $orderStage = $work->stagesWithoutPivot()->where('id', $validated['order_work_stage_id'])->firstOrFail();

        if ($orderStage->price_type !== $validated['price_type']) {
            throw new ValidationException(__("Order work stage price type mismatch"));
        }

        if (!OrderService::isSpecialStage($orderStage->type)) {
            throw new ValidationException(__("Order work stage is not a special stage :stageType", ['stageType' => ($orderStage->type)]));
        }

        OrderWorkStagesService::canStageUserBeModified($orderStage, true);

        OrderWorkStagesService::specialStageIsNotFinished($orderStage,true);

        switch ($orderStage->price_type) {
            case 'fix':
                $this->confirmOrderStage($orderStage, $validated['elements']['fix'] ?? false);
                break;
            case 'arcada':
                $this->updateStageArcadaDentition($orderStage, $validated['elements']);
                break;
            case 'elemente':
                $this->updateStageByElements($orderStage, $validated['elements']);
                break;
            default:
                throw  new ServerError($orderStage->price_type.__(' price type is not implemented.'));
                break;
        }

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($work->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(["data" => new OrderWorkStageResource($orderStage)]);
    }

    /**
     * Manual update order stage user and cost
     *
     * @param  OWStageUpdateUserRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws ValidationException
     */
    public function updateStageUser(OWStageUpdateUserRequest $request)
    {
        $validated = $request->validated();

        $orderStage = OrderWorkStage::where('id', $validated['order_work_stage_id'])->where(
            'order_work_template_id',
            $validated['order_work_id']
        )->first();

        if ($orderStage === null) {
            throw new ValidationException(
                __(
                    'Order stage id :orderWS for order work id :workId does not exists',
                    [
                        'orderWS' => $validated['order_work_stage_id'],
                        'workId'  => $validated['order_work_id']
                    ]
                )
            );
        }

        OrderWorkStagesService::canStageUserBeManuallyModified($request, $orderStage, true);


        $user = User::where('id', $validated['user_id'])->firstOrFail();
        $orderStage->user_id = $user->id;
        $orderStage->user_name = $user->first_name.' '.$user->last_name;
        $orderStage->cost = $validated['cost'];

        $orderStage->save();

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($orderStage->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(["data" => new OrderWorkStageResource($orderStage)]);
    }


    /**
     * Start a order stage
     *
     * @param  OWStageTimerRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws ValidationException
     */
    public function startStageTimer(OWStageTimerRequest $request)
    {
        $validated = $request->validated();

        $orderStage = OrderWorkStage::findOrFail($validated['order_work_stage_id']);
        $isStarted = OrderWorkStagesService::isStageStarted($orderStage, true);
        if ($isStarted) {
            throw new ValidationException(__('Stage already started'));
        }

        UserService::userCanStartStage($request->user(), $orderStage, true);

        $log = null;
        \DB::transaction(
            function () use (&$orderStage, &$log) {
                $orderStage->status = 'working';
                $orderStage->save();

                // start timer
                $log = OrderWorkStagesService::startStageTimer($orderStage);
                $orderUpdateSuccess = OrderService::updateStatus($orderStage->order_id);
            },
            2
        );

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($orderStage->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(["data" => $log]);
    }

    /**
     * Pause a order stage
     *
     * @param  OWStageTimerRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function pauseStageTimer(OWStageTimerRequest $request)
    {
        $validated = $request->validated();

        $orderStage = OrderWorkStage::findOrFail($validated['order_work_stage_id']);
        $isStarted = OrderWorkStagesService::isStageStarted($orderStage, true);
        if (!$isStarted) {
            throw new ValidationException(__('Stage is not started'));
        }

        UserService::userCanStartStage($request->user(), $orderStage, true);

        $log = null;
        \DB::transaction(
            function () use (&$orderStage, &$log) {
                $orderStage->status = 'paused';
                $orderStage->save();

                // stop timer
                $log = OrderWorkStagesService::stopStageTimer($orderStage);
                $orderUpdateSuccess = OrderService::updateStatus($orderStage->order_id);
            },
            2
        );

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($orderStage->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(["data" => $log]);
    }

    /**
     * Finish a order stage
     *
     * @param  OWStageTimerRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function finishStageTimer(OWStageTimerRequest $request)
    {
        $validated = $request->validated();

        $orderStage = OrderWorkStage::findOrFail($validated['order_work_stage_id']);

        OrderWorkStagesService::canStageBeFinished($orderStage, true, true);

        UserService::userCanStartStage($request->user(), $orderStage);

        $logs = null;
        \DB::transaction(
            function () use (&$orderStage, &$logs) {
                $logs = OrderWorkStagesService::endAllTimersForStage($orderStage);

                $orderStage->status = 'finished';
                $orderStage->finished_time = \DB::raw('NOW()');
                $orderStage->save();

                $orderUpdateSuccess = OrderService::updateStatus($orderStage->order_id);
                OrderWorkStagesService::markReadyTime($orderStage->order_id);
            },
            2
        );

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($orderStage->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(["data" => $logs]);
    }

    /**
     *
     * Order stages possibles actions
     *
     * @param  OrderStagesPossibleActionsRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function stagesPossibleActions(OrderStagesPossibleActionsRequest $request)
    {
        $validated = $request->validated();
        $owStages = OrderWorkStage::whereIn('id', $validated['order_work_stage_ids'])->get();

        if ($owStages->count() !== count($validated['order_work_stage_ids'])) {
            throw new PossibleUnSyncException(__('Could not find all order work stages'));
        }
        OrderService::orderIdCanBeUpdated($owStages->order_id);

        $possibleActions = [];
        foreach ($owStages as $index => $owStage) {
            $possibleActions[$index] = OrderWorkStagesService::possibleStageActions($request, $owStage);
        }

        return response(["data" => $possibleActions]);
    }


    /**
     * Remake a order stage
     *
     * @param  OWStageRemakeRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws ServerError
     * @throws ValidationException
     */
    function remakeStage(OWStageRemakeRequest $request)
    {

        $validated = $request->validated();

        $owStage = OrderWorkStage::where('order_work_template_id',$validated['order_work_id'])->find($validated['order_work_stage_id']);
        if ($owStage === null) {
            throw new ValidationException(__('Could not find order stage of work'));
        }
        if ($request->has('remake_parameters.user_is_paying') && !$validated['remake_parameters']['user_is_paying']) {
            $validated['remake_parameters']['user_id_paying'] = 0;
        }

        \DB::transaction(
            function () use ($owStage, $validated) {
                OrderWorkStagesService::canStageBeRemaked($owStage, true);
                OrderWorkTemplateService::remakeStages(
                    $owStage->order_work_template_id,
                    $validated['order_work_stage_id'],
                    $validated['remake_parameters'],
                    true
                );
            }
        );

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($owStage->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(["data" => new OrderWorkTemplateResource(OrderWorkTemplate::with([''])->findOrFail($validated['']))]);
    }

    /**
     * Update date and time of order work stage
     *
     * @param  OWSUpdateDateRequest  $request
     * @param  OrderWorkStage  $orderWorkStage
     *
     * @return Application|ResponseFactory|Response
     * @throws ServerError
     * @throws \Throwable
     */
    public function updateStageDateAndTime(OWSUpdateDateRequest $request,OrderWorkStage $orderWorkStage)
    {
        $validated = $request->validated();
        $orderWorkStage = OrderWorkStagesService::updateDateAndTime($validated, $orderWorkStage);
        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($orderWorkStage->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(["data" => new OrderWorkStageResource($orderWorkStage)]);

    }
}
