<?php

namespace App\Http\Controllers;


use App\Exceptions\Api\AccessDeniedException;
use App\Exceptions\Api\ValidationException;
use App\Http\Requests\OrderWorkCombineRequest;
use App\Http\Requests\OrderWTListingRequest;
use App\Http\Requests\OrderWTStoreRequest;
use App\Http\Requests\OrderWTUpdateSettingsRequest;
use App\Http\Requests\OWTDentitionRequest;
use App\Http\Resources\OrderResource;
use App\Http\Resources\OrderWorkArrayResource;
use App\Http\Resources\OrderWorkTemplateResource;
use App\Models\Order;
use App\Models\OrderWorkStage;
use App\Models\OrderWorkTemplate;
use App\Models\WorkStage;
use App\Models\WorkTemplate;
use App\Services\DentitionService;
use App\Services\OrderService;
use App\Services\OrderWorkStagesService;
use App\Services\OrderWorkTemplateService;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\JsonResource;
use Illuminate\Http\Response;
use App\Services\OrderDatesCalculatorService;

/**
 * @group Comenzi lucrari
 *
 * APIs for managing work stages
 */
class OrderWorkTemplateController extends Controller
{
    /**
     * OT Display a listing of order works.
     *
     * @return Response
     */
    public function index(OrderWTListingRequest $request)
    {
        if (!$request->user()->can(['orders.read'])) {
            throw new AccessDeniedException(__('lang_permissions.orders.read.missing.permission'));
        }

        $validated = $request->validated();
        $order = Order::findOrFail($validated['order_id']);
        $templates = $order->templates()->select(
            'order_work_templates.id as id',
            'order_work_templates.price as price',
            'order_work_templates.extra_price as extra_price',
            'order_work_templates.dentition as dentition',
            'order_work_templates.min_duration as min_duration',
            'order_work_templates.estimated_duration as estimated_duration',
            'order_work_templates.preferred_duration as preferred_duration',
            'order_work_templates.price_type as price_type',
            'order_work_templates.elements as elements',
            'order_work_templates.name as name',
            'order_work_templates.original_name as original_name',
            'work_template_id',
            'combination_id'
        )->orderBy('order_work_templates.created_at')->get()->toArray();


        $combinations = [];
        foreach ($templates as &$template) {
            if ($template['combination_id'] != 0) {
                if (!isset($combinations[$template['combination_id']])) {
                    $combinations[$template['combination_id']] = [];
                }

                $combinations[$template['combination_id']][] = $template;
            }
        }

        $days = OrderService::calculateDays($order, $templates);

        foreach ($templates as $key => &$template) {
            $template['min_date'] = $days['works'][$template['id']]['min_date'];
            $template['preferred_date'] = $days['works'][$template['id']]['preferred_date'];
            $template['estimated_date'] = $days['works'][$template['id']]['estimated_date'];

            $template['combinations'] = [];
            if (isset($combinations[$template['id']])) {
                $template['combinations'] = $combinations[$template['id']];
            }

            if ($template['combination_id'] != 0) {
                unset($templates[$key]);
            }
        }


        return response(['data' => OrderWorkArrayResource::collection($templates)]);
    }

    /**
     * OT Add new work on order.
     *
     * @param  \Illuminate\Http\Request  $request
     *
     * @return Response
     */
    public function store(OrderWTStoreRequest $request)
    {
        if (!$request->user()->can(['orders.create'])) {
            throw new AccessDeniedException(__('lang_permissions.orders.create.missing.permission'));
        }
        $validated = $request->validated();
        $order = Order::findOrFail($validated['order_id']);
        $workTemplate = WorkTemplate::findOrFail($validated['work_template_id']);
        $newOWTemplate = null;
        \DB::transaction(
            function () use ($validated, $order, $workTemplate, &$newOWTemplate) {
                $newOWTemplate = $this->addWorkTemplateToOrder($order, $workTemplate);
                OrderWorkStagesService::updateStagesDates($newOWTemplate->order_id, $newOWTemplate->id);
                OrderWorkStagesService::markReadyTime($validated['order_id']);
            }
        );


        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($newOWTemplate->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(['data' => new OrderWorkTemplateResource($newOWTemplate)], 201);
    }

    /**
     * OT Remove a order work.
     *
     * @param  \App\Models\OrderWorkTemplate  $orderWorkTemplate
     *
     * @return Response
     */
    public function destroy(Request $request, $orderWorkTemplateId)
    {
        if (!$request->user()->can(['orders.delete'])) {
            throw new AccessDeniedException(__('lang_permissions.orders.delete.missing.permission'));
        }

        $orderWorkTemplate = OrderWorkTemplate::findOrFail($orderWorkTemplateId);

        // @todo implement it when implementing calendar
//        $stages = $orderWorkTemplate->stages()->get();
//
//        foreach($stages as $stage){
//            Calendar::where('type', 'trialStage')
//                ->where('external_id', $stage->pivot->id)
//                ->delete();
//        }

        $workId = $orderWorkTemplate->id;
        $orderId = $orderWorkTemplate->order_id;
        //todo seems does not delete properly. to investigate 02.08.2021 Leo
        \DB::transaction(
            function () use ($orderWorkTemplate) {
                OrderWorkTemplate::where('combination_id', $orderWorkTemplate->id)->delete();
                $orderWorkTemplate->stagesWithoutPivot()->delete();
                $orderWorkTemplate->delete();
            }
        );

        // @todo implement it when implementing calendar
//        CalendarLib::updateOrder($orderId);
//        Calendar::deleteWork($workId);

        OrderWorkStagesService::updateStagesDates($orderId, $workId);

        OrderWorkStagesService::markReadyTime($orderId);

        return response(['message' => __('Successfully deleted')], 204);
    }


    /**
     * @param  Order  $order
     * @param  WorkTemplate  $workTemplate
     *
     * @return OrderWorkTemplate
     * @throws \Throwable
     */
    public function addWorkTemplateToOrder(Order $order, WorkTemplate $workTemplate): OrderWorkTemplate
    {
        $stageStatus = 'not_ready';
        if ($order->status === 'creating') {
            $stageStatus = 'config';
        }

        $price = OrderController::getTemplatePrice($order->dentist_id, $order->collaborator_id, $workTemplate->id);

        $newOrderWork = new OrderWorkTemplate();
        $newOrderWork->work_template_id = $workTemplate->id;
        $newOrderWork->name = $workTemplate->name;
        $newOrderWork->original_name = $workTemplate->name;
        $newOrderWork->symbol = $workTemplate->symbol;
        $newOrderWork->order_id = $order->id;
        $newOrderWork->price = $price['preferential_price_if_any'];
        $newOrderWork->extra_price = $workTemplate->extra_cost;
        $newOrderWork->dentition = DentitionService::emptyDentitionWithAdditionalFields();
        $newOrderWork->min_duration = $workTemplate->min_duration;
        $newOrderWork->estimated_duration = $workTemplate->estimated_duration;
        $newOrderWork->preferred_duration = $workTemplate->estimated_duration;
        $newOrderWork->price_type = $workTemplate->price_type;
        $newOrderWork->multi_factor_dentition = $workTemplate->multi_factor_dentition;
        $newOrderWork->arcada_dentition = json_encode(config("lab_configuration.default_arcada_dentition"));
        $newOrderWork->save();
        $this->addTemplateWorkStagesToOrder($workTemplate, $newOrderWork, $stageStatus);
        return $newOrderWork;
    }

    /**
     * @param  WorkTemplate  $workTemplate
     * @param  OrderWorkTemplate  $orderWorkTemplate
     * @param  string  $stageStatus
     *
     * @throws \Throwable
     */
    public function addTemplateWorkStagesToOrder(WorkTemplate $workTemplate, OrderWorkTemplate $orderWorkTemplate, string $stageStatus)
    {
        OrderService::orderIdCanBeUpdated($orderWorkTemplate->order_id);
        $stages = $workTemplate->getStagesWithUsers();
        \DB::transaction(
            function () use ($workTemplate, $orderWorkTemplate, $stageStatus, $stages) {
            }
        );
        $rank = 0;
        foreach ($stages as $stage) {
            $orderWorkTemplate->workStagesOrdered()->attach(
                $stage['id'],
                [
                    'work_template_id'       => $stage['work_template_id'],
                    'rank'                   => $rank++,
                    'order_id'               => $orderWorkTemplate->order_id,
                    'price'                  => $stage['special_cost'],
                    'remake_price'           => $stage['remake_cost'],
                    'extra_cost_applied'     => $stage['extra_cost_applied'],
                    'name'                   => $stage['name'],
                    'type'                   => $stage['type'],
                    'price_type'             => $stage['price_type'],
                    'to_send'                => $stage['to_send'],
                    'from_template'          => 1,
                    'dentition'              => json_encode([]),
                    'status'                 => $stageStatus,
                    'remake_id'              => 0,
                    'user_fault_id'          => 0,
                    'collaborator_fault_id'  => 0,
                    'lab_paying'             => 0,
                    'multi_factor'           => WorkStage::calculateMultiFactor($stage['type'], $stage['price_type'], []),
                    'multi_factor_dentition' => $orderWorkTemplate->multi_factor_dentition,
                    'arcada_dentition'       => json_encode(config("lab_configuration.default_arcada_dentition")),
                    'user_id'                => $stage['user_id'],
                    'user_name'              => is_null($stage['user_name']) ? 0 : $stage['user_name'],
                    'cost'                   => is_null($stage['cost']) ? 0 : $stage['cost'],
                    'day_number'             => $stage['day_number'],
                    'duration'               => $stage['min_duration'],
                    'works'                  => json_encode([$orderWorkTemplate->id])
                ]
            );
        }
        unset($rank);
    }

    /**
     * OT Preferred settings order work.
     *
     * @param  OrderWTUpdateSettingsRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    function updateSettings(OrderWTUpdateSettingsRequest $request)
    {
        $validated = $request->validated();

        $work = OrderWorkTemplate::findOrFail($validated['order_work_template_id']);

        OrderService::orderIdCanBeUpdated($work->order_id, true);

        if ($request->has('preferred_duration')) {
            $order = Order::findOrFail($work->order_id);
            $toFinishDate = OrderDatesCalculatorService::calculateDays($order->start_date, $request->preferred_duration);
            OrderWorkStagesService::checkIfExceedDateRange($work->order_id, $toFinishDate, $order->start_date, $validated['order_work_template_id']);
            OrderWorkTemplateService::updatePreferedDate($work, $validated['preferred_duration']);
        }

        if ($request->has('ignore_extra_price')) {
            OrderWorkTemplateService::updateIgnoreExtraPrice($work, $validated['ignore_extra_price']);
        }

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($work->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(['data' => new OrderWorkTemplateResource($work)]);
    }

    //

    /**
     * OT Update dentition and dentition arcada for order work.
     *
     * @param  OWTDentitionRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws ValidationException
     */
    function updateDentition(OWTDentitionRequest $request)
    {
        $validated = $request->validated();

        $work = OrderWorkTemplate::find($validated['order_work_template_id']);

        OrderService::orderIdCanBeUpdated($work->order_id, true);

        \DB::transaction(
            function () use (&$work, $validated) {
                if (isset($validated['arcada_dentition'])) {
                    $work = DentitionService::updateWorkArcada($work, $validated['arcada_dentition']);
                }
                $work = DentitionService::updateWorkForm($work, $validated['dentition'], true);
                $work = DentitionService::updateOrderXtra($work, $validated['dentition']);
            }
        );


        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($work->order_id);
            return response(['data' => new OrderResource($order)]);
        }

        if ($work->isDirty()) {
            $work->refresh();
        }
        return response(['data' => new OrderWorkTemplateResource($work)]);
    }

    /**
     * OT Combine a order work with a work template
     *
     * @param  OrderWorkCombineRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws ValidationException
     */
    function combineWorks(OrderWorkCombineRequest $request)
    {
        $validated = $request->validated();

        $orderWork = OrderWorkTemplate::findOrFail($validated['order_work_template_id']);
        $order = Order::findOrFail($orderWork->order_id);
        OrderWorkTemplateService::orderWorkCanBeCombined($orderWork);

        $workTemplateToCombineWith = WorkTemplate::findOrFail($validated['work_template_id']);

        $newWork = null;
        \DB::transaction(
            function () use ($order, $orderWork, $workTemplateToCombineWith, &$newWork) {
                $newWork = $this->addWorkTemplateToOrder($order, $workTemplateToCombineWith);
                OrderWorkTemplateService::updateCombination($orderWork, $newWork);
                OrderWorkTemplateService::updateCombinationDentition($newWork, $orderWork->dentition);
                OrderWorkTemplateService::updateCombinationArcadaDentition($newWork, $orderWork->arcada_dentition);
                OrderWorkTemplateService::moveStages($newWork->id, $orderWork->id);
            }
        );

        if ($request->has('order_as_response') && $validated['order_as_response']) {
            $order = (new OrderController())->getOrderWithDetails($order->id);
            return response(['data' => new OrderResource($order)]);
        }

        return response(['data' => new OrderWorkTemplateResource($newWork)]);
    }


}
