<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Http\Requests\PatientRequest;
use App\Http\Resources\PatientCollection;
use App\Http\Resources\PatientResource;
use App\Models\Patient;
use Illuminate\Http\Request;
/**
 * @group Pacienti
 *
 * APIs for managing patients
 */

class PatientController extends Controller
{

    /**
     * Create the controller instance.
     *
     * @return void
     */
    public function __construct()
    {

    }

    /**
     * Patients listing.
     * @apiResourceCollection App\Http\Resources\PatientCollection
     * @apiResourceModel App\Models\Patient
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request): \Illuminate\Http\Response
    {
        if (!$request->user()->can(['patients.read'])) {
            throw new AccessDeniedException(__('lang_permissions.patients.read.missing.permission'));
        }

        return response(new PatientCollection(Patient::all()));
    }

    /**
     * Store a newly created patient.
     * @apiResource App\Http\Resources\PatientResource
     * @apiResourceModel App\Models\Patient
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(PatientRequest $request)
    {
        if (!$request->user()->can(['patients.create'])) {
            throw new AccessDeniedException(__('lang_permissions.patients.create.missing.permission'));
        }
        // Retrieve the validated input data...
        $validated = $request->validated();

        $patient = Patient::create($validated);

        return response(['data' => new PatientResource($patient)],201);
    }

    /**
     * Display a patient.
     * @apiResource App\Http\Resources\PatientResource
     * @apiResourceModel App\Models\Patient
     *
     * @param  \App\Models\Patient  $patient
     * @return \Illuminate\Http\Response
     */
    public function show(Request $request,Patient $patient)
    {
        if (!$request->user()->can(['patients.read'])) {
            throw new AccessDeniedException(__('lang_permissions.patients.read.missing.permission'));
        }

        return response(['data' => new PatientResource($patient)]);
    }

    /**
     * Update a patient.
     * @apiResource App\Http\Resources\PatientResource
     * @apiResourceModel App\Models\Patient
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Patient  $patient
     * @return \Illuminate\Http\Response
     */
    public function update(PatientRequest $request, Patient $patient)
    {
        if (!$request->user()->can(['patients.update'])) {
            throw new AccessDeniedException(__('lang_permissions.patients.update.missing.permission'));
        }
        // Retrieve the validated input data...
        $validated = $request->validated();

        foreach ($validated as $fieldName => $fieldValue) {
            $patient -> $fieldName  = $fieldValue;
        }
        $patient -> save();

        return response(['data' => new PatientResource($patient)]);

    }

    /**
     * Remove a patient.
     *
     * @urlParam  patient numeric required
     * @param  \App\Models\Patient  $patient
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request,Patient $patient)
    {
        if (!$request->user()->can(['patients.delete'])) {
            throw new AccessDeniedException(__('lang_permissions.patients.delete.missing.permission'));
        }

        $patient -> delete();
        return response(['message' => __('Successfully deleted')],204);
    }
}
