<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Exceptions\Api\PossibleUnSyncException;
use App\Exceptions\Api\ValidationException;
use App\Http\Requests\PaymentDocumentStoreRequest;
use App\Http\Requests\PDAddOrdersRequest;
use App\Http\Requests\PDAvailableOrdersRequest;
use App\Http\Requests\PDLockRequest;
use App\Http\Requests\PDOrderDetailsRequest;
use App\Http\Requests\PDRemoveOrderRequest;
use App\Http\Resources\OrderResource;
use App\Http\Resources\PaymentDocumentResource;
use App\Models\Order;
use App\Models\PaymentDocument;
use App\Services\InvoiceService;
use App\Services\OrderService;
use App\Services\PaymentDocumentService;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use App\Http\Requests\PaymentDocumentAddPaymentRequest;
use App\Models\Payment;
use App\Http\Requests\PaymentDocumentRemovePaymentRequest;
use App\Http\Resources\PaymentResource;
use App\Http\Requests\GenerateInvoiceRequest;
use App\Http\Resources\InvoiceResource;
use App\Services\ReceiptService;
use App\Http\Resources\ReceiptResource;
use App\Http\Requests\GenerateReceiptRequest;
use App\Services\PaymentService;

/**
 * @group Documente de plata
 *
 * APIs for managing payment documents
 */
class PaymentDocumentsController extends Controller
{
    const FILTER_BY_START_DATE  = 'by_start_date';
    const FILTER_BY_END_DATE    = 'by_end_date';



    /**
     * Display a listing of payment documents.
     *
     * @param  Request  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function index(Request $request)
    {
        if (!$request->user()->can(['payment_documents.read'])) {
            throw new AccessDeniedException(__('lang_permissions.payment_documents.read.missing.permission'));
        }
        $paymentDocuments = PaymentDocument::orderBy('payment_documents.id', 'DESC')->with(['collaborator', 'dentist', 'payments'])->get();
        return response(['data' => PaymentDocumentResource::collection($paymentDocuments)]);
    }


    /**
     * Add a new payment document.
     *
     * @param  PaymentDocumentStoreRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function store(PaymentDocumentStoreRequest $request)
    {
        $validated = $request->validated();
        PaymentDocumentService::canAddPaymentDocument($validated['collaborator_id'], true);

        $paymentDocument = PaymentDocumentService::createThePaymentDocument($validated);

        if ($validated['detailed_payment']) {
            $paymentDocument = static::getDetailedPaymentDocument($paymentDocument, [], true);
        }

        return response(['data' => new PaymentDocumentResource($paymentDocument)]);
    }

    /**
     * Display a payment document.
     *
     * @param  Request  $request
     * @param  PaymentDocument  $paymentDocument
     *
     * @return Application|ResponseFactory|Response
     */
    public function show(Request $request, PaymentDocument $paymentDocument)
    {
        if (!$request->user()->can(['payment_documents.read'])) {
            throw new AccessDeniedException(__('lang_permissions.payment_documents.read.missing.permission'));
        }

        $paymentDocument = static::getDetailedPaymentDocument($paymentDocument, [], true);
        return response(['data' => new PaymentDocumentResource($paymentDocument)]);
    }

    /**
     * Remove a payment document..
     *
     * @param  Request  $request
     * @param  PaymentDocument  $paymentDocument
     *
     * @return Application|ResponseFactory|Response
     * @throws ValidationException
     */
    public function destroy(Request $request, PaymentDocument $paymentDocument)
    {
        if (!$request->user()->can(['payment_documents.delete'])) {
            throw new AccessDeniedException(__('lang_permissions.payment_documents.delete.missing.permission'));
        }

        PaymentDocumentService::canBeDeleted($paymentDocument, true);
        $paymentDocument->delete();
        return response(['data' => __('Payment document deleted')], 204);
    }

    /**
     * Available orders for a payment document
     *
     * @param  PDAvailableOrdersRequest  $request
     * @param  PaymentDocument  $paymentDocument
     *
     * @return Application|ResponseFactory|Response
     */
    public function getPaymentAvailableOrders(PDAvailableOrdersRequest $request, PaymentDocument $paymentDocument)
    {
        $validated = $request->validated();
        $paymentDocument->availableOrders = $paymentDocument->availableOrders ?? static::getAvailableOrders($paymentDocument, $validated['filters'] ?? [], true);

        return response(['data' => OrderResource::collection($paymentDocument->availableOrders)]);
    }

    /**
     * @param  PaymentDocument  $paymentDocument
     * @param  array  $filters
     *
     * @return mixed
     */
    public static function getAvailableOrders(PaymentDocument $paymentDocument, $filters = [], bool $withWorksAndStaged = false)
    {
        $availableOrdersSql = PaymentDocumentService::sqlAvailableOrdersForCollaborator(
            $paymentDocument->collaborator_id,
            $paymentDocument->dentist_id,
            $filters
        );

        if ($withWorksAndStaged) {
            static::addOrderWorksSql($availableOrdersSql);
        }

        return $availableOrdersSql->get();
    }


    /**
     * Orders detailed works
     *
     * @param  Request  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function getOrdersDetails(PDOrderDetailsRequest $request)
    {
        if (!$request->user()->can(['payment_documents.read'])) {
            throw new AccessDeniedException(__('lang_permissions.payment_documents.read.missing.permission'));
        }
        $validated = $request->validated();
        $ordersDetails = OrderService::getExtraDetails($validated['order_ids']);

        return response(['data' => $ordersDetails]);
    }

    /**
     * @param  PaymentDocument  $paymentDocument
     * @param  array  $filters
     * @param  false  $detailedOrders
     *
     * @return mixed
     */
    public static function getDetailedPaymentDocument(PaymentDocument $paymentDocument, $filters = [], $detailedOrders = false)
    {
        $paymentDocument = PaymentDocument::where('payment_documents.id', $paymentDocument->id)->firstOrFail();
        if ($detailedOrders) {
            $paymentDocument->ordersAttached = static::getOrdersAttachedToPaymentDocument($paymentDocument);
        }
        $paymentDocument->load(['discounts', 'collaborator', 'dentist']);
        return $paymentDocument;
    }

    /**
     * Lock a payment document
     *
     * @param  PDLockRequest  $request
     * @param  PaymentDocument  $paymentDocument
     *
     * @return Application|ResponseFactory|Response
     * @throws ValidationException
     */
    public function lockPaymentDocument(PDLockRequest $request, PaymentDocument $paymentDocument)
    {
        PaymentDocumentService::canBeLocked($paymentDocument, true);
        $validated = $request->validated();
        $paymentDocument->status = 'locked';
        $paymentDocument->save();
        $paymentDocument = static::getDetailedPaymentDocument($paymentDocument, [], $validated['detailed_payment']);

        return response(['data' => new PaymentDocumentResource($paymentDocument)]);
    }

    /**
     * Add orders to the payment document.
     *
     * @param PaymentDocument $paymentDocument
     * @param array $validated
     * @return PaymentDocument
     */
    public static function addOrdersToPaymentDocument(PaymentDocument $paymentDocument, array $validated)
    {
        PaymentDocumentService::canBeUpdated($paymentDocument, true);

        $availableOrders = static::getAvailableOrders($paymentDocument);
        $availableOrdersIds = $availableOrders->pluck('id')->toArray();

        foreach ($validated['order_ids_to_be_added'] as $order) {
            $orderId = $order['order_id'] ?? -1;
            if (!in_array($orderId, $availableOrdersIds)) {
                throw new ValidationException(__('Order id can not be added to the payment document'));
            }
            $order = $availableOrders->where('id', $orderId)->first();

            if ($paymentDocument->dentist_id != 0 && $paymentDocument->dentist_id != $order->dentist_id) {
                throw new ValidationException(__('Payment dentist is different from the order dentist'));
            }
            $order->payment_document_id = $paymentDocument->id;

            // update value
            if ($order->status !== 'complete') {
                $value = OrderService::calculateTotalPrice($order->id);
                $order->value = $value;
            }
            $order->save();
        }

        $paymentDocument->refresh();
        $paymentDocument = static::getDetailedPaymentDocument($paymentDocument, [], $validated['detailed_payment']);
        return $paymentDocument;
    }

    /**
     * Add orders to a payment document.
     *
     * @param  PDAddOrdersRequest  $request
     * @param  PaymentDocument  $paymentDocument
     *
     * @return Application|ResponseFactory|Response
     * @throws ValidationException
     */
    public function addOrders(PDAddOrdersRequest $request, PaymentDocument $paymentDocument)
    {
        $validated = $request->validated();

        $paymentDocument = static::addOrdersToPaymentDocument($paymentDocument, $validated);

        return response(['data' => new PaymentDocumentResource($paymentDocument)]);
    }

    /**
     * Remove order from payment document
     *
     * @param  PDRemoveOrderRequest  $request
     * @param  PaymentDocument  $paymentDocument
     *
     * @return Application|ResponseFactory|Response
     * @throws PossibleUnSyncException
     * @throws ValidationException
     */
    public function removeOrder(PDRemoveOrderRequest $request, PaymentDocument $paymentDocument)
    {
        PaymentDocumentService::canBeUpdated($paymentDocument, true);
        $validated = $request->validated();
        $order = Order::findOrFail($validated['order_id_to_be_removed']);
        if ((int)$paymentDocument->to_pay < (int)$order->value) {
            throw new PossibleUnSyncException(__('lang_permissions.orders.delete.payments.conflict'));
        }

        $order->payment_document_id = 0;
        $order->save();
        $paymentDocument = static::getDetailedPaymentDocument($paymentDocument, [], $validated['detailed_payment']);

        return response(['data' => new PaymentDocumentResource($paymentDocument)]);
    }


    /**
     * Mark payment document as paid
     *
     * @param  PaymentDocument  $paymentDocument
     *
     * @return Application|ResponseFactory|Response
     * @throws ValidationException
     */
    public function markAsPaid(PDLockRequest $request, PaymentDocument $paymentDocument)
    {
        PaymentDocumentService::canBeMarkedAsPaid($paymentDocument);
        $validated = $request->validated();
        $paymentDocument->status = 'paid';
        $paymentDocument->save();
        $paymentDocument->refresh();
        $paymentDocument = static::getDetailedPaymentDocument($paymentDocument, [], $validated['detailed_payment']);
        return response(['data' => new PaymentDocumentResource($paymentDocument)]);
    }

    public static function getOrdersAttachedToPaymentDocument(PaymentDocument $paymentDocument)
    {
        $sqlWorks = Order::select('orders.*')->where('orders.payment_document_id', $paymentDocument->id);
        static::addOrderWorksSql($sqlWorks);
        $sqlWorks = $sqlWorks->get();
        $sqlWorks->load(['dentist', 'collaborator', 'patient', 'additionalPrices' => function ($query) {
            return $query->where('to_be_invoiced', 1);
        }]);

        return $sqlWorks;
    }

    /**
     * @param $ordersSql
     */
    public static function addOrderWorksSql($ordersSql)
    {
        OrderService::addOrderWorksSql($ordersSql);
    }

    /**
     * Add the payment to the payment document's due orders.
     *
     * @param PaymentDocument $paymentDocument
     * @param array $parameters
     */
    public static function addPaymentValueToDueOrders(PaymentDocument $paymentDocument, array $parameters)
    {
        PaymentService::canAddPayment($paymentDocument, $parameters['value'], true);

        $parameters['payment_document_id'] = $paymentDocument->id;
        $orders = $paymentDocument->orders;
        $paymentsArray = PaymentDocumentService::distributePaymentToOrders($orders, $parameters, $paymentDocument);

        Payment::insert($paymentsArray);
        $paymentDocument->refresh();

        return $paymentDocument;
    }

    /**
     * Distribute unattached payments to the payment document's due orders.
     *
     * @param PaymentDocument $paymentDocument
     * @return void
     */
    public static function distributeUnattachedPaymentsTotalToDueOrders(PaymentDocument $paymentDocument)
    {
        $orders = $paymentDocument->orders;
        $payments = Payment::where('payment_document_id', $paymentDocument->id)->where('order_id', 0)
            ->whereNull('receipt_series')->WhereNull('receipt_no');
        $getPayments = $payments->get();

        $payments->delete();
        $paymentsArray = PaymentDocumentService::distributePayments($getPayments, $orders, $paymentDocument);
        Payment::insert($paymentsArray);
        $paymentDocument->refresh();

        return $paymentDocument;
    }

    /**
     * Add payment to payment document
     *
     * @param PaymentDocumentAddPaymentRequest $request
     * @param PaymentDocument $paymentDocument
     * @return mixed
     * @throws PossibleUnSyncException
     * @throws ValidationException
     */
    public function addPayment(PaymentDocumentAddPaymentRequest $request, PaymentDocument $paymentDocument)
    {
        $validated = $request->validated();

        $paymentDocument = $paymentDocument->invoice
            ? self::addPaymentValueToDueOrders($paymentDocument, $validated)
            : PaymentDocumentService::addPayment($paymentDocument, $validated);
        $paymentDocument->refresh();
        return response(['data' => new PaymentDocumentResource($paymentDocument)]);
    }


    /**
     * Remove payment from payment document
     *
     * @param PaymentDocumentAddPaymentRequest $request
     * @param PaymentDocument $paymentDocument
     * @return mixed
     * @throws ValidationException
     */
    public function removePayment(PaymentDocumentRemovePaymentRequest $request, PaymentDocument $paymentDocument)
    {
        $validated = $request->validated();
        PaymentService::canRemovePayment($validated['payment_id'], true);
        PaymentDocumentService::removePaymentFromPD($validated['payment_id'], $paymentDocument);

        return response(['data' => new PaymentDocumentResource($paymentDocument)]);
    }

    /**
     * Get all payments of a payment document
     *
     * @param PaymentDocument $paymentDocument
     * @return ResponseFactory|Response
     */
    public function getPayments(PaymentDocument $paymentDocument)
    {
        $payments = Payment::where('payment_document_id', $paymentDocument->id)
            ->get();

        foreach ($payments as $payment) {
            $payment->can_issue_receipt = PaymentService::canIssueReceipt($payment);
        }

        return response(['data' => PaymentResource::collection($payments)]);
    }

    /**
     * Generate invoice
     *
     * @param GenerateInvoiceRequest $request
     * @param PaymentDocument $paymentDocument
     * @throws ValidationException
     */
    public function generateInvoice(GenerateInvoiceRequest $request, PaymentDocument $paymentDocument)
    {
        $validated = $request->validated();
        if (!request()->user()->can(['payment_documents.update'])) {
            throw new AccessDeniedException(__('lang_permissions.payment_documents.update.missing.permission'));
        }
        PaymentDocumentService::canGenerateInvoice($paymentDocument, $validated['invoice_date'], $validated['invoice_series_id'], $validated['invoice_services'], true);
        self::distributeUnattachedPaymentsTotalToDueOrders($paymentDocument);
        $invoice = InvoiceService::generateInvoice($paymentDocument, $validated['invoice_series_id'], $validated['collaborator_id'], $validated['invoice_date'], $validated['invoice_services']);

        return response(['data' => new InvoiceResource($invoice)], 201);
    }

    /**
     * Generate receipt
     *
     * @param GenerateReceiptRequest $request
     * @param PaymentDocument $paymentDocument
     * @return ResponseFactory|Response
     * @throws ValidationException
     * @throws \App\Exceptions\Api\AccessDeniedException
     */
    public function generateReceipt(GenerateReceiptRequest $request, PaymentDocument $paymentDocument)
    {
        $validated = $request->validated();
        PaymentDocumentService::canGenerateReceipt($paymentDocument, $validated['payment_id'], $validated['receipt_date'], $validated['receipt_series_id'], true);
        $receipt = ReceiptService::generateReceipt($paymentDocument, $validated['payment_id'], $validated['receipt_series_id'], $validated['receipt_date']);
        return response(['data' => new ReceiptResource($receipt)], 201);
    }
}
