<?php

namespace App\Http\Controllers;

use App\Http\Requests\ReceiptSeriesRequest;
use App\Models\InvoiceSeries;
use App\Models\ReceiptSeries;
use App\Services\HelperService;
use App\Services\ReceiptSeriesService;
use Illuminate\Http\Request;
use App\Http\Resources\ReceiptSeriesResource;
use Illuminate\Support\Facades\DB;
use App\Exceptions\Api\AccessDeniedException;

/**
 * @group Chitante serii
 *
 * APIs for managing receipt series
 */
class ReceiptSeriesController extends Controller
{
    /**
     * Display receipt series
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request, $invoiceProfileId = null)
    {
        if (!$request->user()->can(['receipt_series.read'])) {
            throw new AccessDeniedException(__('lang_permissions.receipt_series.read.missing.permission'));
        }
        $receiptSeries = ReceiptSeries::select('receipt_series.*');

        if(isset($invoiceProfileId)){
            $receiptSeries->where('lab_invoice_profile_id', $invoiceProfileId);
        }
        $receiptSeries = $receiptSeries->get();

        return response(['data' => ReceiptSeriesResource::collection($receiptSeries)]);
    }

    /**
     * Create receipt serie
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(ReceiptSeriesRequest $request)
    {
        if (!$request->user()->can(['receipt_series.create'])) {
            throw new AccessDeniedException(__('lang_permissions.receipt_series.create.missing.permission'));
        }
        $validated = $request->validated();
        HelperService::existsDuplicateSeriesOnSameProfile(ReceiptSeries::class, $validated['lab_invoice_profile_id'], $validated['series'], true);
        DB::table('receipt_series')->where('lab_invoice_profile_id', $validated['lab_invoice_profile_id'])->update(['active' => 0]);
        $invoiceSerie = ReceiptSeries::create($validated);

        return response(['data' => new ReceiptSeriesResource($invoiceSerie)], 201);
    }

    /**
     * Activate receipt series
     *
     * @param ReceiptSeries $receiptSeries
     * @return \Illuminate\Contracts\Routing\ResponseFactory|\Illuminate\Http\Response
     */
    public function activate(Request $request, ReceiptSeries $receiptSeries)
    {
        if (!$request->user()->can(['receipt_series.read'])) {
            throw new AccessDeniedException(__('lang_permissions.receipt_series.update.missing.permission'));
        }
        DB::table('receipt_series')->where('lab_invoice_profile_id', $receiptSeries->lab_invoice_profile_id)->update(['active' => 0]);

        $receiptSeries->active = 1;
        $receiptSeries->save();

        return response(['data' => new ReceiptSeriesResource($receiptSeries)]);
    }
}
