<?php

namespace App\Http\Controllers\Reports;

use App\Exceptions\Api\AccessDeniedException;
use App\Http\Controllers\Controller;
use App\Http\Requests\CompletedStagesChartRequest;
use App\Http\Resources\CompletedStagesChartResource;
use App\Models\User;
use App\Models\WorkStage;
use App\Services\Reports\CompletedStagesChartService;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;
use Illuminate\Support\Collection;

/**
 * @group Grafice
 *
 * APIs for managing charts
 */
class CompletedStagesChartController extends Controller
{
    /**
     * Retrieve completed stages info for the chart.
     *
     * @param User $user
     * @param WorkStage $workStage
     * @param integer $numberOfMonths
     * @return AnonymousResourceCollection
     */
    public function getCompletedStagesChartInfo(User $user, WorkStage $workStage, int $numberOfMonths = 6): AnonymousResourceCollection
    {
        $stages = CompletedStagesChartService::calculateTotalValuePerMonth($user, $workStage, $numberOfMonths);
        $dates = CompletedStagesChartService::generateIntervalDatesWithDefaultValues($stages, $numberOfMonths);

        return CompletedStagesChartResource::collection(['data' => $dates]);
    }

    /**
     * Listing of technician's completed stage(s) value per months number.
     *
     * @param CompletedStagesChartRequest $request
     * @param User $user
     * @param WorkStage $workStage
     * @return Collection
     */
    public function getCompletedStagesChartData(CompletedStagesChartRequest $request, User $user, WorkStage $workStage)
    {
        if (!$request->user()->can(['reports.read'])) {
            throw new AccessDeniedException(__('lang_permissions.reports.read.missing.permission'));
        }

        $validated = $request->validated();

        if (isset($validated['number_of_months'])) {
            return $this->getCompletedStagesChartInfo($user, $workStage, $validated['number_of_months']);
        }
        return $this->getCompletedStagesChartInfo($user, $workStage);
    }

    /**
     * Retrieve user's total per order work stage.
     *
     * @param User $user
     * @param integer $numberOfMonths
     * @return AnonymousResourceCollection
     */
    public function getUserTotalPerOrderWorkStage(User $user, $numberOfMonths = 6): AnonymousResourceCollection
    {
        $orderWorkStages = CompletedStagesChartService::calculateUserTotalPerOrderWorkStage($user, $numberOfMonths);

        return CompletedStagesChartResource::collection($orderWorkStages);
    }

    /**
     * Ranking of technician's completed stages value.
     *
     * @param CompletedStagesChartRequest $request
     * @param User $user
     * @return AnonymousResourceCollection
     */
    public function getUserTotalPerOrderWorkStageData(CompletedStagesChartRequest $request, User $user): AnonymousResourceCollection
    {
        if (!$request->user()->can(['reports.read'])) {
            throw new AccessDeniedException(__('lang_permissions.reports.read.missing.permission'));
        }

        $validated = $request->validated();

        if (isset($validated['number_of_months'])) {
            return $this->getUserTotalPerOrderWorkStage($user, $validated['number_of_months']);
        }
        return $this->getUserTotalPerOrderWorkStage($user);
    }
}
