<?php

namespace App\Http\Controllers\Reports;

use App\Exceptions\Api\AccessDeniedException;
use App\Http\Controllers\Controller;
use App\Http\Controllers\PaymentDocumentsController;
use App\Http\Requests\CalculateTotalPriceForOrdersIdsRequest;
use App\Http\Requests\OrdersReportRequest;
use App\Http\Resources\OrderResource;
use App\Models\Collaborator;
use App\Models\LabProfile;
use App\Models\Order;
use App\Services\OrderService;
use App\Services\PaymentDocumentService;
use App\Services\Reports\DiscountsService;
use App\Services\Reports\OrdersService;
use App\Services\Reports\PaymentsService;

/**
 * @group Rapoarte comenzi
 *
 * APIs for managing order additional prices
 */
class OrderController extends Controller
{

    /**
     * Orders report.
     *
     * <aside class="notice">Invoice filters combined could give wrong data. Use only one with true</aside>
     *
     * @param  OrdersReportRequest  $request
     * @param  Collaborator  $collaborator
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\Routing\ResponseFactory|\Illuminate\Http\Response
     * @throws AccessDeniedException
     */
    public function getCollaboratorOrdersNotCanceled(OrdersReportRequest $request, Collaborator $collaborator)
    {
        if (!request()->user()->can(['reports.read'])) {
            throw new AccessDeniedException(__('lang_permissions.reports.read.missing.permission'));
        }
        $validated = $request->validated();
        $validated['filters']['collaborator_id'] = $collaborator->id;

        return response(['data' => OrderResource::collection(OrdersService::getOrdersNotCanceled($validated['filters']))]);
    }

    /**
     * Send an order to invoicing.
     *
     **/
    public function sendToInvoicing(Order $order)
    {
        OrdersService::canBeSentToInvoicing($order);

        $unlockedPaymentDocument = PaymentDocumentService::getTheUnlockedPaymentDocument($order);
        $validated['order_ids_to_be_added'] = [
            0 => [
                'order_id' => $order->id
            ]
        ];
        $validated['detailed_payment'] = false;

        if ($unlockedPaymentDocument) {
            PaymentDocumentsController::addOrdersToPaymentDocument($unlockedPaymentDocument, $validated);
        } else {
            unset($order['dentist_id']);
            $order['lab_invoice_profile_id'] = LabProfile::firstOrFail()->lab_invoice_profile_id;
            $paymentDocument = PaymentDocumentService::createThePaymentDocument($order->toArray());
            PaymentDocumentsController::addOrdersToPaymentDocument($paymentDocument, $validated);
        }

        PaymentsService::moveOrderPaymentsToPayments($order);
        DiscountsService::moveOrderDiscountsToDiscounts($order);
        return response(['data' => 'Comanda a fost trimisa in facturare.']);
    }

    /**
     * Calculate current total value
     *
     * @param  CalculateTotalPriceForOrdersIdsRequest  $request
     *
     * @return
     */
    public function calculateTotalPriceForOrdersIds(CalculateTotalPriceForOrdersIdsRequest $request)
    {
        $validated = $request->validated();
        $ordersIds = array_column($validated['orders_ids'], 'order_id');
        return response(['data' => OrderService::calculateTotalPriceForIds($ordersIds)]);
    }
}
