<?php

namespace App\Http\Controllers\Reports;

use App\Exceptions\Api\AccessDeniedException;
use App\Http\Controllers\Controller;
use App\Http\Requests\OrdersChartRequest;
use App\Http\Resources\OrdersChartCollaboratorsResource;
use App\Http\Resources\OrdersChartResource;
use App\Models\Collaborator;
use App\Services\Reports\ChartService;
use App\Services\Reports\OrdersChartService;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;
use Illuminate\Support\Collection;

/**
 * @group Grafice
 *
 * APIs for managing charts
 */
class OrdersChartController extends Controller
{
    /**
     * Get the total value per month.
     *
     * @param Collaborator $collaborator
     * @param integer $numberOfMonths
     * @return Collection
     */
    public static function getTotalValuePerMonth(Collaborator $collaborator, int $numberOfMonths): Collection
    {
        $tempTable = ChartService::createMonthsTemporaryTable($numberOfMonths);
        $tempTable = OrdersChartService::calculateTotalValuePerMonth($collaborator);
        return $tempTable->get();
    }

    /**
     * Retrieve orders info for the chart.
     *
     * @param Collaborator $collaborator
     * @param integer $numberOfMonths
     * @return AnonymousResourceCollection
     */
    public function getOrdersChartInfo(Collaborator $collaborator, int $numberOfMonths = 6): AnonymousResourceCollection
    {
        $totalValuesPerMonth = self::getTotalValuePerMonth($collaborator, $numberOfMonths);

        return OrdersChartResource::collection(['data' => $totalValuesPerMonth]);
    }

    /**
     * Listing of orders total per month.
     *
     * @param OrdersChartRequest $request
     * @param Collaborator $collaborator
     * @return AnonymousResourceCollection
     */
    public function getOrdersChartData(OrdersChartRequest $request, Collaborator $collaborator): AnonymousResourceCollection
    {
        if (!$request->user()->can(['reports.read'])) {
            throw new AccessDeniedException(__('lang_permissions.reports.read.missing.permission'));
        }

        $validated = $request->validated();

        if (isset($validated['number_of_months'])) {
            return $this->getOrdersChartInfo($collaborator, $validated['number_of_months']);
        }
        return $this->getOrdersChartInfo($collaborator);
    }

    /**
     * Retrieve collaborators info for the modal.
     *
     * @param integer $numberOfMonths
     * @return AnonymousResourceCollection
     */
    public function getOrdersChartCollaborators(int $numberOfMonths = 6): AnonymousResourceCollection
    {
        $query = OrdersChartService::calculateOrdersChartCollaboratorsTotal($numberOfMonths);

        return OrdersChartCollaboratorsResource::collection($query);
    }

    /**
     * Ranking of collaborators per orders value.
     *
     * @param OrdersChartRequest $request
     * @return AnonymousResourceCollection
     */
    public function getOrdersChartCollaboratorsData(OrdersChartRequest $request): AnonymousResourceCollection
    {
        if (!$request->user()->can(['reports.read'])) {
            throw new AccessDeniedException(__('lang_permissions.reports.read.missing.permission'));
        }

        $validated = $request->validated();

        if (isset($validated['number_of_months'])) {
            return $this->getOrdersChartCollaborators($validated['number_of_months']);
        }
        return $this->getOrdersChartCollaborators();
    }
}
