<?php

namespace App\Http\Controllers\Reports;

use App\Exceptions\Api\AccessDeniedException;
use App\Http\Controllers\Controller;
use App\Http\Requests\WorksChartRequest;
use App\Http\Resources\WorksChartOrdersResource;
use App\Http\Resources\WorksChartResource;
use App\Models\WorkTemplate;
use App\Services\Reports\ChartService;
use App\Services\Reports\WorksChartService;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;
use Illuminate\Support\Collection;

/**
 * @group Grafice
 *
 * APIs for managing charts
 */
class WorksChartController extends Controller
{
    /**
     * Get the total value per month.
     *
     * @param WorkTemplate $workTemplate
     * @param integer $numberOfMonths
     * @return Collection
     */
    public static function getTotalValuePerMonth(WorkTemplate $workTemplate, int $numberOfMonths): Collection
    {
        $tempTable = ChartService::createMonthsTemporaryTable($numberOfMonths);
        $tempTable = WorksChartService::calculateTotalValuePerMonth($workTemplate);

        return $tempTable->get();
    }

    /**
     * Retrieve works info for the chart.
     *
     * @param WorkTemplate $workTemplate
     * @param integer $numberOfMonths
     * @return AnonymousResourceCollection
     */
    public function getWorksChartInfo(WorkTemplate $workTemplate, $numberOfMonths = 6): AnonymousResourceCollection
    {
        $totalValuesPerMonth = self::getTotalValuePerMonth($workTemplate, $numberOfMonths);

        return WorksChartResource::collection(['data' => $totalValuesPerMonth]);
    }

    /**
     * Listing of works per days number.
     *
     * @param WorksChartRequest $request
     * @param WorkTemplate $workTemplate
     * @return AnonymousResourceCollection
     */
    public function getWorksChartData(WorksChartRequest $request, WorkTemplate $workTemplate): AnonymousResourceCollection
    {
        if (!$request->user()->can(['reports.read'])) {
            throw new AccessDeniedException(__('lang_permissions.reports.read.missing.permission'));
        }

        $validated = $request->validated();

        if (isset($validated['number_of_months'])) {
            return $this->getWorksChartInfo($workTemplate, $validated['number_of_months']);
        }
        return $this->getWorksChartInfo($workTemplate);
    }

    /**
     * Retrieve orders info for the works chart.
     *
     * @param integer $numberOfMonths
     * @return AnonymousResourceCollection
     */
    public function getWorksChartOrders(int $numberOfMonths = 6): AnonymousResourceCollection
    {
        $query = WorksChartService::calculateWorksChartOrdersTotal($numberOfMonths);

        return WorksChartOrdersResource::collection(['data' => $query]);
    }

    /**
     * Ranking of work orders number.
     *
     * @param WorksChartRequest $request
     * @return AnonymousResourceCollection
     */
    public function getWorksChartOrdersData(WorksChartRequest $request): AnonymousResourceCollection
    {
        if (!$request->user()->can(['reports.read'])) {
            throw new AccessDeniedException(__('lang_permissions.reports.read.missing.permission'));
        }

        $validated = $request->validated();

        if (isset($validated['number_of_months'])) {
            return $this->getWorksChartOrders($validated['number_of_months']);
        }

        return $this->getWorksChartOrders();
    }
}
