<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Http\Requests\RoleCreateRequest;
use App\Http\Requests\RoleUpdateRequest;
use App\Http\Resources\RoleCollection;
use App\Http\Resources\RoleResource;
use App\Models\Role;
use Illuminate\Http\Request;

/**
 * @group Roluri
 *
 * APIs for managing roles
 */
class RoleController extends Controller
{
    /**
     * Roles listing.
     *
     * @apiResourceCollection App\Http\Resources\RoleCollection
     * @apiResourceModel \App\Models\Role
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if (!$request->user()->can(['roles.read'])) {
            throw new AccessDeniedException(__('lang_permissions.roles.read.missing.permission'));
        }

        return response(new RoleCollection(Role::where('name','<>',config('custom.api.super_admin_role'))->get()));
    }

    /**
     * Store a newly created role.
     *
     * @apiResource App\Http\Resources\RoleResource
     * @apiResourceModel \App\Models\Role
     *
     * @param  \Illuminate\Http\Request  $request
     *
     * @return \Illuminate\Http\Response
     */
    public function store(RoleCreateRequest $request)
    {
        if (!$request->user()->can(['roles.create'])) {
            throw new AccessDeniedException(__('lang_permissions.roles.create.missing.permission'));
        }
        // Retrieve the validated input data...
        $validated = $request->validated();

        $role = new Role();
        $role = $this->updateFields($validated, $role);

        return response(['data' => new RoleResource($role)], 201);
    }

    /**
     * Display a role.
     *
     * @apiResource App\Http\Resources\RoleResource
     * @apiResourceModel \App\Models\Role
     * @urlParam role integer required
     *
     * @param  \App\Models\Role  $role
     *
     * @return \Illuminate\Http\Response
     */
    public function show(Request $request, Role $role)
    {
        if (!$request->user()->can(['roles.read'])) {
            throw new AccessDeniedException(__('lang_permissions.roles.read.missing.permission'));
        }

        return response(['data' => new RoleResource($role)]);
    }

    /**
     * Update  a role.
     *
     * @apiResource App\Http\Resources\RoleResource
     * @apiResourceModel \App\Models\Role
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  Role $role
     *
     * @return \Illuminate\Http\Response
     */
    public function update(RoleUpdateRequest $request, Role $role)
    {
        if (!$request->user()->can(['roles.update'])) {
            throw new AccessDeniedException(__('lang_permissions.roles.update.missing.permission'));
        }
        if ( $role->name == config('custom.api.super_admin_role')) {
            throw new AccessDeniedException(__('lang_permissions.roles.delete.missing.permission.superadmin'));
        }

        // Retrieve the validated input data...
        $validated = $request->validated();

        $role = $this->updateFields($validated, $role);

        return response(['data' => new RoleResource($role)]);
    }


    /**
     * Remove  a role.
     * @response  1
     * @param  Role $role
     *
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, Role $role)
    {
        if (!$request->user()->can(['roles.delete'])) {
            throw new AccessDeniedException(__('lang_permissions.roles.delete.missing.permission'));
        }
        if ( $role->name == config('custom.api.super_admin_role')) {
            throw new AccessDeniedException(__('lang_permissions.roles.delete.missing.permission.superadmin'));
        }

        $role ->permissions()->detach();
        $role ->delete();
        return response(['message' => __('Successfully deleted')], 204);
    }

    /**
     * @param  array  $validated
     * @param  Role $role
     *
     * @return Role
     * @throws \Throwable
     */
    public function updateFields(array $validated, Role $role): Role
    {
        $updateRolePermissions = false;
        foreach ($validated as $fieldName => $fieldValue) {
            if ($fieldName === 'permissions') {
                $updateRolePermissions = true;
                continue;
            }
            $role->$fieldName = $fieldValue;
        }
        if($updateRolePermissions){
            $role->updateRolePermissions($role,$validated['permissions']);
        }
        $role->saveOrFail();
        $role ->refresh()->givePermissionTo(config('custom.api.default_role_permissions'));
        return $role;
    }
}
