<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\ServerError;
use App\Exceptions\Api\ValidationException;
use App\Http\Requests\AddTenantRequest;
use App\Models\Tenant;
use Artisan;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\DB;
use Spatie\MediaLibrary\MediaCollections\Models\Media;

/**
 * @group Rute tehnice
 *
 */
class TechnicalRoutesController extends Controller
{

    public function migrateCentral()
    {
        try {
            Artisan::queue('migrate', ['--quiet' => false, '--force' => true, '--step' => true]);
            echo 'migrating main complete</br>';
        } catch (Exception $e) {
            echo 'Caught exception: ', $e->getMessage(), "\n";
            return false;
        }
    }

    public function migrateFreshCentral()
    {
        try {
            Artisan::queue('migrate:fresh', ['--quiet' => false, '--force' => true, '--step' => true]);
            echo 'migrating fresh main complete</br>';
        } catch (Exception $e) {
            echo 'Caught exception: ', $e->getMessage(), "\n";
            return false;
        }
    }

    public function migrateCentralRollback()
    {
        try {
            Artisan::queue('migrate:rollback', ['--quiet' => true, '--force' => true, '--step' => '1']);
            echo 'migrate main back 1 step complete</br>';
        } catch (Exception $e) {
            echo 'Caught exception: ', $e->getMessage(), "\n";
            return false;
        }
    }

    public function seedCentral()
    {
        try {
            Artisan::call('db:seed');
            echo 'seed main complete</br>';
        } catch (Exception $e) {
            echo 'Caught exception: ', $e->getMessage(), "\n";
            return false;
        }
    }

    /**
     * Migrating tenants
     *
     * @unauthenticated
     *
     * @response {"message": "clear cache complete"}
     */
    public function migrateTenants()
    {
        try {
            echo Artisan::call('tenants:migrate', ['--quiet' => false, '--force' => true, '--step' => true]);
            return response(["message" => 'migrating tenants complete']);
        } catch (Exception $e) {
            echo 'Caught exception: ', $e->getMessage(), "\n";
            return false;
        }
    }

    public function migrateTenantsRollback()
    {
        try {
            echo Artisan::call('tenants:rollback', ['--quiet' => false, '--force' => true, '--step' => '1']);
            echo 'migrate tenants back 1 step complete</br>';
        } catch (Exception $e) {
            echo 'Caught exception: ', $e->getMessage(), "\n";
            return false;
        }
    }

    public function seedTenants()
    {
        try {
            Artisan::call('tenants:seed', ['--quiet' => false]);
            echo "seed tenants complete</br>";
        } catch (Exception $e) {
            echo 'Caught exception: ', $e->getMessage(), "\n";
            return false;
        }
    }

    public function migrateTenantRollback($tenant_name)
    {
        try {
            Artisan::call('tenants:migrate-fresh', ['--tenants' => [$tenant_name], '--quiet' => false]);
            echo "fresh migrating tenant  >> $tenant_name << complete</br>";
        } catch (Exception $e) {
            echo 'Caught exception: ', $e->getMessage(), "\n";
            return false;
        }
    }

    public function migrateTenant($tenant_name)
    {
        try {
            Artisan::call('tenants:migrate', ['--tenants' => [$tenant_name], '--quiet' => false]);
            echo "migrating tenant  >> $tenant_name << complete</br>";
            return true;
        } catch (Exception $e) {
            echo 'Caught exception: ', $e->getMessage(), "\n";
            return false;
        }
    }

    public function migrateFreshTenant($tenant_name, $verbose = true)
    {
        try {
            Artisan::call('tenants:migrate-fresh', ['--tenants' => [$tenant_name], '--quiet' => false]);
            if ($verbose) {
                echo "migrating tenant  >> $tenant_name << complete</br>";
            }
            return true;
        } catch (Exception $e) {
            echo $e->getMessage(), "\n";
            \Log::error("migrate tenant:".$tenant_name.' with error:'.$e->getMessage());
            return false;
        }
    }

    public function seedTenant($tenant_name, $verbose = true)
    {
        try {
            Artisan::call('tenants:seed', ['--tenants' => [$tenant_name], '--quiet' => false]);
            if ($verbose) {
                echo "seed tenant >> $tenant_name << complete</br></br>";
            }
            return true;
        } catch (Exception $e) {
            echo $e->getMessage(), "\n";
            \Log::error("seed tenant:".$tenant_name.' with error:'.$e->getMessage());
            return false;
        }
    }


    public function createTenant($tenant_name)
    {
        $tenant = Tenant::where('id', $tenant_name)->first();
        if ($tenant !== null) {
            throw new ValidationException('Tenant already exists');
        }

        $tenant = Tenant::create(['id' => strtolower($tenant_name)]);
        if ($tenant === null) {
            throw new ServerError('Could not create tenant');
        }
        return $tenant;
    }

    /**
     * Clear cache
     *
     * @unauthenticated
     *
     * @response {"message": "clear cache complete"}
     */
    public function clearCache()
    {
        try {
            if (!defined('STDIN')) {
                define('STDIN', fopen("php://stdin", "rb"));
            }
            App::loadDeferredProviders();
            Artisan::call('optimize:clear');
            return response(["message" => 'clear cache complete']);
        } catch (Exception $e) {
            echo $e->getMessage(), "\n";
        }
    }

    /**
     * Install a tenant
     *
     * <aside>You need to have the database already created , even if is empty. Will not install if there is already a tenant with
     * the same name.</aside>
     *
     * @param  Request  $request
     * @param $tenant_name
     *
     * @unauthenticated
     * @response {"message": "Tenant $tenant_name created"}
     *
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\Routing\ResponseFactory|\Illuminate\Http\Response
     * @throws ServerError
     * @throws \Throwable
     */
    public function installTenant(Request $request, $tenant_name)
    {
        try {
            DB::transaction(
                function () use ($tenant_name) {
                    $responseCreateTenant = $this->createTenant($tenant_name);
                    if ($responseCreateTenant === null) {
                        throw new ServerError("Could not create tenant $tenant_name");
                    }
                    //initializare tenant
                    tenancy()->initialize($responseCreateTenant);

                    if (!$this->migrateFreshTenant(\tenant('id'), false)) {
                        throw new ServerError("Could not migrate tenant $tenant_name");
                    }

                    if (!$this->seedTenant(\tenant('id'), false)) {
                        throw new ServerError("Could not seed tenant $tenant_name");
                    }
                }
            );
        } catch (Exception $e) {
            throw new ServerError($e->getMessage());
        }

        return response(["message" => "Tenant $tenant_name created"]);
    }

    /**
     * Reset database fo a tenant
     *
     * <aside>You need to have the tenant and the database already created.Will not work if tenant is not already present.</aside>
     *
     * @param  Request  $request
     * @param $tenant_name
     *
     * @unauthenticated
     * @response {"message": "Tenant $tenant_name refreshed"}
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\Routing\ResponseFactory|\Illuminate\Http\Response
     * @throws ServerError
     */
    public function resetTenant(Request $request, $tenant_name)
    {
        try {
            $tenant = Tenant::where('id', $tenant_name)->first();
            if ($tenant === null) {
                throw new ServerError("$tenant_name does not exist");
            }
            //initializare tenant
            tenancy()->initialize($tenant);

            if (!$this->migrateFreshTenant(\tenant('id'), false)) {
                throw new ServerError("Could not migrate tenant $tenant_name");
            }

            $this->deleteTenantFiles($tenant);

            if (!$this->seedTenant(\tenant('id'), false)) {
                throw new ServerError("Could not seed tenant $tenant_name");
            }
        } catch (Exception $e) {
            throw new ServerError($e->getMessage());
        }

        return response(["message" => "Tenant $tenant_name reseted"]);
    }

    public function deleteTenantFiles(Tenant $tenant)
    {
        $initialised = false;
        if (tenant('id') !== $tenant->id) {
            $initialised = true;
            tenancy()->initialize($tenant);
        }
        $tenantMedia = Media::all();
        foreach ($tenantMedia as $media) {
            $media->delete();
        }
        if($initialised){
            tenancy()->end();
        }
    }


}
