<?php

namespace App\Http\Controllers;


use App\Exceptions\Api\AccessDeniedException;
use App\Http\Requests\EndCollaborationRequest;
use App\Http\Requests\ProfileFilesRequest;
use App\Http\Requests\UserProfileCreateRequest;
use App\Http\Requests\UserProfileUpdateRequest;
use App\Http\Requests\UserSelfProfileUpdateRequest;
use App\Http\Resources\UserCollection;
use App\Http\Resources\UserCompleteResource;
use App\Http\Resources\UsernameCollection;
use App\Models\Tenant;
use App\Models\User;
use App\Services\UploadService;
use App\Services\UserService;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Hash;
use Spatie\QueryBuilder\QueryBuilder;

/**
 * @group Utilizatori
 *
 * APIs for managing users
 */
class UserController extends Controller
{
    private $possibleUsernamesCollection;

    /**
     * All possible users on all clients
     *
     * This endpoint allows you to see all possible users on all clients.
     *
     * @unauthenticated
     */
    public function allCustomersIdsIfDebugOn(Request $request)
    {
        $customersIdentifiers = Tenant::all();
        $this->possibleUsernamesCollection = new Collection();
        foreach ($customersIdentifiers as $customersIdentifier) {
            $customersIdentifier->run(
                function () {
                    $this->possibleUsernamesCollection = $this->possibleUsernamesCollection->merge(User::all());
                }
            );
        }

        return response(new UsernameCollection($this->possibleUsernamesCollection));
    }


    /**
     * All possible users on specific client
     *
     * This endpoint allows you to see all possible users on specific client
     *
     * @apiResourceCollection  App\Http\Resources\UserCollection
     * @apiResourceModel App\Models\User
     * @param  Request  $request
     *
     * @queryParam sort string Field to sort by. Sorting ASC id Sorting DESC -id Example: -id,first_name,last_name
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function index(Request $request)
    {
        if (!$request->user()->can(['users.read'])) {
            throw new AccessDeniedException(__('lang_permissions.users.read.missing.permission'));
        }

        $users = QueryBuilder::for(User::notSuperAdmin())
            ->defaultSorts(User::$defaultSort)
            ->allowedSorts(User::$sortable)
            ->get();

        return response(new UserCollection($users));
    }

    /**
     * Store a newly created user.
     *
     * @apiResource App\Http\Resources\UserCompleteResource
     * @apiResourceModel App\Models\User
     * @bodyParam  password_confirmation required
     *
     * @param  \Illuminate\Http\Request  $request
     *
     * @return Response
     */
    public function store(UserProfileCreateRequest $request)
    {
        if (!$request->user()->can(['users.create'])) {
            throw new AccessDeniedException(__('lang_permissions.users.create.missing.permission'));
        }
        // Retrieve the validated input data...
        $validated = $request->validated();
        $user = new User();
        $user = $this->updateFields($validated, $user);

        return response(['data' => new UserCompleteResource($user)], 201);
    }

    /**
     * Display a user.
     *
     * @apiResource App\Http\Resources\UserCompleteResource
     * @apiResourceModel App\Models\User
     * @urlParam user integer required
     *
     * @param  \App\Models\User  $user
     *
     * @return Response
     */
    public function show(Request $request, User $user)
    {
        if (!$request->user()->can(['users.read'])) {
            throw new AccessDeniedException(__('lang_permissions.users.read.missing.permission'));
        }

        return response(['data' => new UserCompleteResource($user)]);
    }


    /**
     * Update  a user.
     *
     * @apiResource App\Http\Resources\UserCompleteResource
     * @apiResourceModel App\Models\User
     * @urlParam user integer required
     * @bodyParam  password_confirmation required
     *
     * @param  UserProfileCreateRequest  $request
     * @param  User  $user
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function update(UserProfileUpdateRequest $request, User $user)
    {
        if (!$request->user()->can(['users.update'])) {
            throw new AccessDeniedException(__('lang_permissions.users.update.missing.permission'));
        }
        // Retrieve the validated input data...
        $validated = $request->validated();

        $user = $this->updateFields($validated, $user);

        return response(['data' => new UserCompleteResource($user)]);
    }

    /**
     * Logged user profile.
     *
     * @apiResource  App\Http\Resources\UserCompleteResource
     * @apiResourceModel App\Models\User
     */
    public function selfProfile(Request $request)
    {
        if (!$request->user()->can(['self.profile.read'])) {
            throw new AccessDeniedException(__('lang_permissions.self.profile.read.missing.permission'));
        }
        return response(['data' => new UserCompleteResource($request->user())]);
    }


    /**
     * Update logged user profile.
     *
     * @apiResource  App\Http\Resources\UserCompleteResource
     * @apiResourceModel App\Models\User
     * @bodyParam  password_confirmation required
     *
     * @param  UserSelfProfileUpdateRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws \Throwable
     */
    public function selfProfileUpdate(UserSelfProfileUpdateRequest $request)
    {
        if (!$request->user()->can(['self.profile.update'])) {
            throw new AccessDeniedException(__('lang_permissions.self.profile.update.missing.permission'));
        }

        $validated = $request->validated();
        $user = $request->user();

        $user = $this->updateFields($validated, $user);

        return response(['data' => new UserCompleteResource($user->load(['roles', 'roles.permissions']))]);
    }


    /**
     * @param  array  $validated
     * @param  User  $user
     *
     * @return User
     * @throws \Throwable
     */
    public function updateFields(array $validated, User $user): User
    {
        $updateRole = false;
        foreach ($validated as $fieldName => $fieldValue) {
            if ($fieldName === 'username_suffix') {
                $fieldValue = tenant()->id . '.' . $fieldValue;
                $fieldName = "username";
            }
            if ($fieldName === 'password') {
                $fieldValue = Hash::make($fieldValue);
                if ($user->currentAccessToken() !== null) {
                    $user->currentAccessToken()->delete();
                }
            }
            if ($fieldName === 'roles_ids') {
                $updateRole = true;
                continue;
            }
            $user->$fieldName = $fieldValue;
        }

        if ($updateRole) {
            $user->updateUserRoles($user, $validated['roles_ids']);
        }
        $user->saveOrFail();


        return $user;
    }

    /**
     * Update user's profile picture.
     *
     * @apiResource App\Http\Resources\UserCompleteResource
     * @apiResourceModel App\Models\User
     * @param  ProfileFilesRequest  $request
     * @urlParam user integer required
     * @bodyParam picture file
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws \App\Exceptions\Api\ServerError
     * @throws \App\Exceptions\Api\ValidationException
     */
    public function updateProfilePhoto(ProfileFilesRequest $request, User $user)
    {
        if (!$request->user()->can(['self.profile.update'])) {
            throw new AccessDeniedException(__('lang_permissions.self.profile.update.missing.permission'));
        }
        $validated = $request->validated();

        foreach ($validated as $name => $fileData) {
            $fileName = (new UploadService())->savefile($user, $request->file($name), $name, UploadService::PUBLIC_UPLOAD_PATH);
            $user->profile_pic = $fileName;
        }

        $user->use_profile_pic = true;
        $user->save();

        return response(['data' => new UserCompleteResource($user)]);
    }

    /**
     * End the collaboration with the given user.
     *
     * @param EndCollaborationRequest $request
     * @param User $user
     * @return Response
     */
    public function endCollaboration(EndCollaborationRequest $request, User $user)
    {
        if (!$request->user()->can(['users.update'])) {
            throw new AccessDeniedException(__('lang_permissions.users.update.missing.permission'));
        }
        $validated = $request->validated();

        UserService::endCollaboration($user, $validated, $request);

        return response(['data' => new UserCompleteResource($user)]);
    }
}
