<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Http\Requests\WorkStageRequest;
use App\Http\Resources\WorkStageCollection;
use App\Http\Resources\WorkStageResource;
use App\Models\WorkStage;
use App\Traits\HasMultiFactorDentition;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

/**
 * @group Etape lucru
 *
 * APIs for managing work stages
 */
class WorkStageController extends Controller
{
    use HasMultiFactorDentition;

    /**
     * Create the controller instance.
     *
     * @return void
     */
    public function __construct()
    {
    }


    /**
     * Work stages listing.
     *
     * @apiResourceCollection  App\Http\Resources\WorkStageCollection
     * @apiResourceModel App\Models\WorkStage
     *
     * @param  Request  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function index(Request $request)
    {
        if ($request->user()->cannot('work_stages.read')) {
            throw new AccessDeniedException(__('lang_permissions.work_stages.read.missing.permission'));
        }

        return response(new WorkStageCollection(WorkStage::with('users')->get()));
    }


    /**
     * Store a newly created work stage.
     *
     * @apiResource  App\Http\Resources\WorkStageResource
     * @apiResourceModel App\Models\WorkStage
     *
     * @param  WorkStageRequest  $request
     *
     * @return Response
     * @throws AccessDeniedException
     */
    public function store(WorkStageRequest $request): Response
    {
        if ($request->user()->cannot('work_stages.create')) {
            throw new AccessDeniedException(__('lang_permissions.work_stages.create.missing.permission'));
        }
        // Retrieve the validated input data...
        $validated = $request->validated();
        $validated['multi_factor_dentition'] = $this->encodeMultiFactorDentition($validated['multi_factor_dentition']);

        $workStage = WorkStage::create($validated);

        return response(['data' => new WorkStageResource($workStage)], 201);
    }


    /**
     * Display a work stages.
     *
     * @apiResource  App\Http\Resources\WorkStageResource
     * @apiResourceModel App\Models\WorkStage
     * @urlParam  work_stage integer required
     *
     * @param  Request  $request
     * @param  WorkStage  $workStage
     *
     * @return Response
     * @throws AccessDeniedException
     */
    public function show(Request $request, WorkStage $workStage): Response
    {
        if ($request->user()->cannot('work_stages.read')) {
            throw new AccessDeniedException(__('lang_permissions.work_stages.read.missing.permission'));
        }

        return response(['data' => new WorkStageResource($workStage->loadMissing('users'))]);
    }


    /**
     * Update a work stages.
     *
     * @apiResource  App\Http\Resources\WorkStageResource
     * @apiResourceModel App\Models\WorkStage
     * @urlParam  work_stage integer required
     *
     * @param  WorkStageRequest  $request
     * @param  WorkStage  $workStage
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function update(WorkStageRequest $request, WorkStage $workStage)
    {
        if ($request->user()->cannot('work_stages.update')) {
            throw new AccessDeniedException(__('lang_permissions.work_stages.update.missing.permission'));
        }
        // Retrieve the validated input data...
        $validated = $request->validated();
        $validated['multi_factor_dentition'] = $this->encodeMultiFactorDentition($validated['multi_factor_dentition']);

        foreach ($validated as $fieldName => $fieldValue) {
            $workStage->$fieldName = $fieldValue;
        }
        $workStage->save();

        return response(['data' => new WorkStageResource($workStage)]);
    }


    /**
     * Remove a work stages.
     *
     * @param  Request  $request
     * @param  WorkStage  $workStage
     * @urlParam  work_stage integer required
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function destroy(Request $request, WorkStage $workStage)
    {
        if ($request->user()->cannot('work_stages.delete')) {
            throw new AccessDeniedException(__('lang_permissions.work_stages.delete.missing.permission'));
        }

        $this->deleteWorkStage($workStage);
        
        return response(['message' => __('Successfully deleted')], 204);
    }

    /**
     * 
     */
    private function deleteWorkStage(WorkStage $workStage){
        $workStage->delete();
    }


    /**
     * Reset all work stages
     * 
     * @param  Request  $request
     */
    public function resetWorkStages(Request $request){
        if ($request->user()->cannot('work_stages.delete')) {
            throw new AccessDeniedException(__('lang_permissions.work_stages.delete.missing.permission'));
        }

        $workStages = WorkStage::all();
        foreach($workStages as $ws){
            $this->deleteWorkStage($ws);
        }

        return response(['message' => __('Successfully deleted')], 204);
    }


}
