<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Exceptions\Api\ValidationException;
use App\Http\Requests\WorkStageUpdateUsersRankRequest;
use App\Http\Requests\WorkStageUserCreateRequest;
use App\Http\Requests\WorkStageUserRequestUpdate;
use App\Http\Requests\WorkStageUsers;
use App\Http\Resources\WorkStageUserCollection;
use App\Http\Resources\WorkStageUserResource;
use App\Models\WorkStage;
use App\Models\WorkStageUser;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

/**
 * @group Etape lucru utilizatori
 *
 * APIs for managing work stage users
 */
class WorkStageUserController extends Controller
{


    /**
     * Display all users attached to a work stage.
     *
     * @apiResourceCollection   App\Http\Resources\WorkStageUserCollection
     * @apiResourceModel App\Models\WorkStageUser
     * @param  Request  $request
     * @param  WorkStage  $workStage
     *
     * @urlParam  work_stage integer required work stage id
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function index(Request $request, WorkStage $workStage)
    {
        if ($request->user()->cannot('work_stages.read')) {
            throw new AccessDeniedException(__('lang_permissions.work_stages_users.read.missing.permission'));
        }

        $workStage->loadMissing(['workStageUsers.user.roles']);
        $usersForWorkStage = $workStage -> workStageUsers;
        return response(new WorkStageUserCollection($usersForWorkStage));
    }


    /**
     * Store a newly user to a work stage.
     *
     * @apiResource  App\Http\Resources\WorkStageUserResource
     * @apiResourceModel App\Models\WorkStageUser
     *
     * @param  WorkStageUserCreateRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function store(WorkStageUserCreateRequest $request)
    {
        if ($request->user()->cannot('work_stages.create')) {
            throw new AccessDeniedException(__('lang_permissions.work_stages_users.create.missing.permission'));
        }
        // Retrieve the validated input data...
        $validated = $request->validated();
        $workStageUser = WorkStageUser::where('work_stage_id', '=', $validated['work_stage_id'])->where(
            'user_id',
            '=',
            $validated['user_id']
        )->firstOr(
            function () use ($validated) {
                $newWorkStageUser = new WorkStageUser();
                foreach ($validated as $fieldName => $fieldValue) {
                    $newWorkStageUser->$fieldName = $fieldValue;
                }
                $newWorkStageUser->save();
                return $newWorkStageUser;
            }
        );
        $workStageUser->rank = $workStageUser->rank ?? $this->nextRankForWorkStage($validated['work_stage_id']);
        if ($workStageUser->isDirty()) {
            $workStageUser->save();
        }

        $workStageUser->fresh();
        $workStageUser->loadMissing(['user.roles']);

        return response(new WorkStageUserResource($workStageUser), 201);
    }


    /**
     * Display a user attached to a work stage.
     *
     * @apiResource  App\Http\Resources\WorkStageUserResource
     * @apiResourceModel App\Models\WorkStageUser
     * @urlParam  work_stage_user integer required
     *
     * @param  Request  $request
     * @param  WorkStageUser  $workStageUser
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function show(Request $request, WorkStageUser $workStageUser)
    {
        if ($request->user()->cannot('work_stages.read')) {
            throw new AccessDeniedException(__('lang_permissions.work_stages_users.read.missing.permission'));
        }

        return response(new WorkStageUserResource($workStageUser));
    }


    /**
     * Update a user attached to a work stage.
     *
     * <aside>Updates only price and rank</aside>
     *
     * @apiResource  App\Http\Resources\WorkStageUserResource
     * @apiResourceModel App\Models\WorkStageUser
     *
     * @param  WorkStageUserRequestUpdate  $request
     * @param  WorkStageUser  $workStageUser
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function update(WorkStageUserRequestUpdate $request, WorkStageUser $workStageUser)
    {
        if ($request->user()->cannot('work_stages.update')) {
            throw new AccessDeniedException(__('lang_permissions.work_stages_users.update.missing.permission'));
        }
        $validated = $request->validated();
        foreach ($validated as $fieldName => $fieldValue) {
            $workStageUser->$fieldName = $fieldValue;
        }
        $workStageUser->save();

        return response(new WorkStageUserResource($workStageUser));
    }


    /**
     * Remove a user attached to a work stage.
     *
     * @urlParam  work_stage_user integer required
     *
     * @param  Request  $request
     * @param  WorkStageUser  $workStageUser
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function destroy(Request $request, WorkStageUser $workStageUser)
    {
        if ($request->user()->cannot('work_stages.delete')) {
            throw new AccessDeniedException(__('lang_permissions.work_stages_users.delete.missing.permission'));
        }
        $workStageUser->delete();
        return response(['message' => __('Successfully deleted')], 204);
    }

    /**
     * @param $workStageId
     *
     * @return mixed
     */
    public function lastRankForWorkStage($workStageId)
    {
        return WorkStageUser::where('work_stage_id', $workStageId)->get()->pluck('rank')->max();
    }

    /**
     * @param $workStageId
     *
     * @return int
     */
    public function nextRankForWorkStage($workStageId): int
    {
        $nextRank = 0;
        $lastRank = $this->lastRankForWorkStage($workStageId);
        if ($lastRank !== null) {
            $nextRank = $lastRank + 1;
        }
        return $nextRank;
    }


    /**
     * Updates work stage users on specified work stage
     * @apiResourceCollection   App\Http\Resources\WorkStageUserCollection
     * @apiResourceModel App\Models\WorkStageUser
     * @param  WorkStageUpdateUsersRankRequest  $request
     *
     * @bodyParam work_stage_users required
     * @return Application|ResponseFactory|Response
     * @throws ValidationException
     */
    public function updateWorkStageUsers(WorkStageUpdateUsersRankRequest $request)
    {
        $validated = $request->validated();

        $workStageUsers = $this->validateUpdatingUsersRequest($validated);

        foreach ($validated['work_stage_users'] as $rank => $workStageUserId) {
            $workStageUser = $workStageUsers->where('user_id', $workStageUserId['user_id'])->first();
            $workStageUser->rank = $rank;
            $workStageUser->save();
        }

        return response(new WorkStageUserCollection($workStageUsers->load('user.roles')->sortBy('rank')->values()));
    }

    private function validateUpdatingUsersRequest($validatedRequestData)
    {
        //unique ranks
        $requestRanks = array_column($validatedRequestData['work_stage_users'], 'rank');
        $uniqueRequestRanks = array_unique($requestRanks);
        if ($requestRanks !== $uniqueRequestRanks) {
            throw new ValidationException('Invalid ranks');
        }
        //unique users
        $requestUsersIds = array_column($validatedRequestData['work_stage_users'], 'user_id');
        $uniqueRequestUsersIds = array_unique($requestUsersIds);
        if ($requestUsersIds !== $uniqueRequestUsersIds) {
            throw new ValidationException('Invalid users id');
        }

        $workStageUsers = WorkStageUser::where('work_stage_id', $validatedRequestData['work_stage_id'])->get();

        //same users in request as in db
        $workStageUsersIds = $workStageUsers->pluck('user_id')->toArray();

        if (count($workStageUsers) !== count($requestUsersIds) ||
            count(array_intersect($requestUsersIds, $workStageUsersIds)) !== count($workStageUsersIds)) {
            throw new ValidationException('Updating users not the same as on work stage.');
        }

        return $workStageUsers;
    }
}
