<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Http\Requests\CloneWorkTemplateRequest;
use App\Http\Requests\WorkTemplateRequest;
use App\Http\Resources\WorkTemplateCollection;
use App\Http\Resources\WorkTemplateResource;
use App\Models\WorkTemplate;
use App\Traits\HasMultiFactorDentition;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

/**
 * @group Sabloane
 *
 * APIs for managing work templates
 */
class WorkTemplateController extends Controller
{
    use HasMultiFactorDentition;


    /**
     * Work template listing.
     *
     * @apiResourceCollection  App\Http\Resources\WorkTemplateCollection
     * @apiResourceModel App\Models\WorkTemplate
     *
     * @param  Request  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function index(Request $request)
    {
        if ($request->user()->cannot('work_templates.read')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.read.missing.permission'));
        }

        return response(new WorkTemplateCollection(WorkTemplate::get()));
    }


    /**
     * WT Store a newly created work template.
     *
     * @apiResource  App\Http\Resources\WorkTemplateResource
     * @apiResourceModel App\Models\WorkTemplate
     *
     * @param  WorkTemplateRequest  $request
     *
     * @return Response
     * @throws AccessDeniedException
     */
    public function store(WorkTemplateRequest $request): Response
    {
        if ($request->user()->cannot('work_templates.create')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.create.missing.permission'));
        }
        // Retrieve the validated input data...
        $validated = $request->validated();
        $validated['multi_factor_dentition'] = $this->encodeMultiFactorDentition($validated['multi_factor_dentition']);


        $workTemplate = WorkTemplate::create($validated);

        return response(['data' => new WorkTemplateResource($workTemplate)], 201);
    }


    /**
     * WT  Display a work template.
     *
     * @apiResource  App\Http\Resources\WorkTemplateResource
     * @apiResourceModel App\Models\WorkTemplate
     * @urlParam  work_template integer required
     *
     * @param  Request  $request
     * @param  WorkTemplate  $workTemplate
     *
     * @return Response
     * @throws AccessDeniedException
     */
    public function show(Request $request, WorkTemplate $workTemplate): Response
    {
        if ($request->user()->cannot('work_templates.read')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.read.missing.permission'));
        }

        return response(['data' => new WorkTemplateResource($workTemplate)]);
    }


    /**
     * WT Update a work template.
     *
     * @apiResource  App\Http\Resources\WorkTemplateResource
     * @apiResourceModel App\Models\WorkTemplate
     * @urlParam  work_template integer required
     *
     * @param  WorkTemplateRequest  $request
     * @param  WorkTemplate  $workTemplate
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function update(WorkTemplateRequest $request, WorkTemplate $workTemplate)
    {
        if ($request->user()->cannot('work_templates.update')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.update.missing.permission'));
        }
        // Retrieve the validated input data...
        $validated = $request->validated();
        $validated['multi_factor_dentition'] = $this->encodeMultiFactorDentition($validated['multi_factor_dentition']);

        foreach ($validated as $fieldName => $fieldValue) {
            $workTemplate->$fieldName = $fieldValue;
        }
        $workTemplate->save();

        $currentWorkTemplate = WorkTemplate::findOrFail($workTemplate->id);
        $templatesWithPrice = $currentWorkTemplate->dentistsPrice()
            ->where('work_template_id', $workTemplate->id)
            ->where('locked', 0)
            ->get();

        foreach ($templatesWithPrice as $templateWithPrice) {
            $templateWithPrice->pivot->delete();
        }

        return response(['data' => new WorkTemplateResource($workTemplate)]);
    }


    /**
     * WT Remove a work template.
     *
     * @param  Request  $request
     * @param  WorkTemplate  $workTemplate
     *
     * @urlParam  work_template integer required
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function destroy(Request $request, WorkTemplate $workTemplate)
    {
        if ($request->user()->cannot('work_templates.delete')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.delete.missing.permission'));
        }

        $this->deleteWorkTemplate($workTemplate);

        return response(['message' => __('Successfully deleted')], 204);
    }

    /**
     *
     */
    private function deleteWorkTemplate(WorkTemplate $workTemplate)
    {
        $workTemplate->workStagesOrdered()->detach();
        $workTemplate->forceDelete();
    }


    /**
     * WT Reset all work templates.
     *
     * @param  Request  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function resetWorKTemplates(Request $request)
    {
        if (!$request->user()->hasRole([config('custom.api.super_admin_role')])) {
            throw new AccessDeniedException(__('lang_permissions.roles.missing.role.superadmin'));
        }

        $workTemplates = WorkTemplate::all();
        foreach ($workTemplates as $wt) {
            $this->deleteWorkTemplate($wt);
        }

        return response(['message' => __('Successfully deleted')], 204);
    }


    /**
     * @param  int  $dentistId
     * @param  int  $collaboratorId
     * @param  int|null  $workTemplateId
     *
     * @return \Illuminate\Support\Collection
     */
    public static function getTemplatePrices(int $dentistId, int $collaboratorId, int $workTemplateId = null)
    {
        $pricesSql = WorkTemplate::select(
            [
                'work_templates.id as id',
                'work_templates.name',
                'work_templates.cost',
                \DB::RAW('IFNULL(dprices.price, IFNULL(cprices.price, work_templates.cost)) as preferential_price_if_any'),
                \DB::RAW('IF(IFNULL(dprices.locked,cprices.locked) = 1, "true", "false") as price_locked'),
            ]
        )->leftJoin(
            'collaborator_prices as cprices',
            function ($join) use ($collaboratorId) {
                $join->on('work_templates.id', '=', 'cprices.work_template_id')->where('cprices.collaborator_id', '=', $collaboratorId);
            }
        )->leftJoin(
            'dentist_prices as dprices',
            function ($join) use ($dentistId) {
                $join->on('work_templates.id', '=', 'dprices.work_template_id')->where('dprices.dentist_id', '=', $dentistId);
            }
        )->orderBy('work_templates.name');

        if ($workTemplateId !== null) {
            $pricesSql->where('work_templates.id', $workTemplateId);
        }

        return $pricesSql->get();
    }

    /**
     * Clone a work template.
     *
     * @param  CloneWorkTemplateRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function cloneWorkTemplates(CloneWorkTemplateRequest $request)
    {
        if ($request->user()->cannot('work_templates.create')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.create.missing.permission'));
        }
        $validated = $request->validated();
        $toBeClonedWorkTemplate = WorkTemplate::findOrFail(
            $validated['work_template_id']
        );

        $workTemplate = null;
        \DB::transaction(
            function () use ($toBeClonedWorkTemplate, &$workTemplate) {
                $workTemplate = $toBeClonedWorkTemplate->replicate()->fill(['name' => __('Clone of ') . $toBeClonedWorkTemplate->name]);
                $workTemplate->save();
                foreach ($toBeClonedWorkTemplate->templateStagesOrdered as $toBeClonedStagePivot) {
                    $stagePivot = $toBeClonedStagePivot->replicate();
                    $stagePivot->work_template_id = $workTemplate->id;
                    $stagePivot->save();
                    foreach ($toBeClonedStagePivot->templateStageUsersOrdered as $toBeClonedUserPivot) {
                        $userPivot = $toBeClonedUserPivot->replicate();
                        $userPivot->work_template_id = $workTemplate->id;
                        $userPivot->work_template_stage_id = $stagePivot->id;
                        $userPivot->save();
                    }
                }
            }
        );
        $workTemplate->refresh();

        return response(['data' => new WorkTemplateResource($workTemplate)], 201);
    }
}
