<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Exceptions\Api\ValidationException;
use App\Http\Requests\WorkTemplateStageDeleteRequest;
use App\Http\Requests\WorkTemplateStageRequest;
use App\Http\Requests\WorkTemplateStagesIndexRequest;
use App\Http\Requests\WorkTemplateStagesUpdateRequest;
use App\Http\Requests\WTStageDayNumberRequest;
use App\Http\Resources\WorkTemplateStageCollection;
use App\Http\Resources\WorkTemplateStageResource;
use App\Models\WorkStage;
use App\Models\WorkTemplate;
use App\Models\WorkTemplateStage;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Resources\Json\ResourceCollection;
use Illuminate\Http\Response;

/**
 * @group Sabloane - etape
 *
 * APIs for managing work templates
 */
class WorkTemplateStageController extends Controller
{
    private $workTemplateStageFields
        = [
            'work_template_id',
            'work_stage_id',
            'rank',
            'day_number',
        ];


    /**
     * WTS Listing of work template stages
     *
     * @apiResourceCollection   App\Http\Resources\WorkTemplateStageCollection
     * @apiResourceModel App\Models\WorkTemplateStage
     *
     * @param  WorkTemplateStagesIndexRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function index(WorkTemplateStagesIndexRequest $request)
    {
        if ($request->user()->cannot('work_templates.read')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.read.missing.permission'));
        }

        $validated = $request->validated();
        $workTemplate = WorkTemplate::where('id', $validated['work_template_id'])->firstOrFail();
        $workTemplateStages = $workTemplate->workStagesOrdered()->get();
        return response(['data' => new WorkTemplateStageCollection($workTemplateStages)]);
    }

    /**
     * WTS creates a work template stage.
     *
     * <aside> Add`s automatically work stage users to work template stage</aside>
     *
     * @apiResource  App\Http\Resources\WorkTemplateStageResource
     * @apiResourceModel App\Models\WorkTemplateStage
     *
     * @param  WorkTemplateStageRequest  $request
     *
     *
     * @return Application|ResponseFactory|Response
     */
    public function store(WorkTemplateStageRequest $request)
    {
        if ($request->user()->cannot('work_templates.update')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.update.missing.permission'));
        }
        $workTemplate = null;
        $validated = $request->validated();
        \DB::transaction(function () use ($validated, &$workTemplate) {
            $workTemplate = WorkTemplate::where('id', $validated['work_template_id'])->firstOrFail();
            $this->addNewWorkTemplateStage($workTemplate, $validated);
            $this->reorderStages($workTemplate);
        });

        $workTemplateStage = $workTemplate->workStagesOrdered()->where('work_stage_id', $validated['work_stage_id'])->first();
        return response(['data' => new WorkTemplateStageResource($workTemplateStage)], 201);
    }


    /**
     * WTS Remove a work template stage.
     *
     * @param  \App\Models\WorkTemplateStage  $workTemplateStage
     *
     * @response
     * @return Response
     */
    public function destroy(WorkTemplateStageDeleteRequest $request)
    {
        if ($request->user()->cannot('work_templates.update')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.update.missing.permission'));
        }

        $validated = $request->validated();

        $wtStage = WorkTemplateStage::where('work_stage_id', $validated['work_stage_id'])->where(
            'work_template_id',
            $validated['work_template_id']
        )->firstOrFail();

        if ($wtStage === null) {
            throw new ValidationException(__('Record does not exist'));
        }

        $workTemplate = WorkTemplate::where('id', $validated['work_template_id'])->firstOrFail();
        $workTemplate->workStagesOrdered()->detach([$validated['work_stage_id']]);
        $this->reorderStages($workTemplate);
        return response(['data' => __("work template stage ").$validated['work_template_id'].__(" has been deleted")], 204);
    }


    /**
     * WTS update work template stages.
     *
     * @apiResourceCollection   App\Http\Resources\WorkTemplateStageCollection
     * @apiResourceModel App\Models\WorkTemplateStage
     *
     * @param  WorkTemplateStagesUpdateRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws ValidationException
     */
    public function updateWorkTemplateStages(WorkTemplateStagesUpdateRequest $request)
    {
        if ($request->user()->cannot('work_templates.update')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.update.missing.permission'));
        }

        $validated = $request->validated();

        $workTemplate = WorkTemplate:: findOrFail($validated['work_template_id']);
        $templateWorkStages = $this->validateUpdatingWorkStagesRequest($workTemplate, $validated);

        foreach ($validated['work_template_stages'] as $rank => $workTemplateStage) {
            $updatingWorkTemplateStage = $templateWorkStages->where('work_stage_id', $workTemplateStage['work_stage_id'])->first();
            $updatingWorkTemplateStage->rank = $workTemplateStage['rank'];
            $updatingWorkTemplateStage->save();
        }

        $this->reorderStages($workTemplate);

        return response(['data' => new WorkTemplateStageCollection($workTemplate->workStagesOrdered()->get())]);
    }


    /**
     * WTS update work template stage day number.
     *
     * @param  WTStageDayNumberRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     */
    public function updateWTStageDayNumber(WTStageDayNumberRequest $request)
    {
        if ($request->user()->cannot('work_templates.update')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.update.missing.permission'));
        }
        $validated = $request->validated();
        $workTemplateStage = WorkTemplateStage::where('work_template_id', $validated['work_template_id'])->where('work_stage_id',$validated['work_template_stage_id']
        )->get();
        if (count($workTemplateStage) !== 1) {
            throw new ValidationException(__('There is none or more then one combination of work_stage_id and work_template_id'));
        }
        $workTemplateStage = $workTemplateStage->first();
        $workTemplateStage->day_number = $validated['day_number'];
        $workTemplateStage->save();
        $workTemplateStage->refresh();

        return response(['data' => __('Work template stage day number updated')]);
    }


    public function addNewWorkTemplateStage(WorkTemplate $workTemplate, array $newWorkStageParameters)
    {
        $workStageExists = $workTemplate->templateStagesOrdered()->where('work_stage_id', $newWorkStageParameters['work_stage_id'])->first(
        );
        if ($workStageExists !== null) {
//            throw  new ValidationException(__('Work stage already exists. Use update.'));
        }

        $reorderedWTS = $this->parseRequestWorkStageChange($workTemplate, $newWorkStageParameters);
        $workTemplate->workStagesOrdered()->sync($reorderedWTS);

        $this->addWorkStageUsersToTemplateStage($workTemplate, $newWorkStageParameters);
    }

    public function parseRequestWorkStageChange(WorkTemplate $workTemplate, array $newWorkStageParameters)
    {
        $workTemplateStageFields = $this->workTemplateStageFields;

        $workTemplateStages = $workTemplate->templateStagesOrdered()->get($workTemplateStageFields)->toArray();
        $newWTSWorkStage = WorkStage::find($newWorkStageParameters['work_stage_id']);
        if ($newWTSWorkStage == null) {
            throw new ValidationException(__('Work stage id is invalid'));
        }

        $reorderedWTS = [];

        $reorderedWTS[$newWorkStageParameters['work_stage_id']] = [
            "work_template_id" => $workTemplate->id,
            "work_stage_id"    => $newWTSWorkStage->id,
            "rank"             => $newWorkStageParameters['rank'] > count($workTemplateStages) ? count($workTemplateStages) + 1
                : $newWorkStageParameters['rank'],
            "day_number"       => (int)$newWTSWorkStage->to_send
        ];

        $rankNumber = 1;
        foreach ($workTemplateStages as $workTemplateStage) {
            if ($workTemplateStage['rank'] < $newWorkStageParameters['rank']) {
                $workTemplateStage['rank'] = $rankNumber;
                $reorderedWTS[$workTemplateStage['work_stage_id']] = $workTemplateStage;
                $rankNumber++;
                continue;
            }
            if ($rankNumber < $newWorkStageParameters['rank']) {
                $rankNumber = $newWorkStageParameters['rank'];
            }

            $workTemplateStage['rank'] = $rankNumber + 1;
            $reorderedWTS[$workTemplateStage['work_stage_id']] = $workTemplateStage;
            $rankNumber++;
        }
        return $reorderedWTS;
    }


    public function addWorkStageUsersToTemplateStage(WorkTemplate $workTemplate, array $newWorkStageParameters)
    {
        $workStage = $workTemplate->workStagesOrdered()->where(
            'work_template_stages.work_stage_id',
            $newWorkStageParameters['work_stage_id']
        )->with('users')->first();
        $workTemplateStageUsers = [];
        foreach ($workStage->users as $user) {
            $workTemplateStageUsers[$user->id] = [
                'work_stage_id'          => $workStage->id,
                'work_template_stage_id' => $workStage->pivot->id,
                'work_template_id'       => $workTemplate->id,
                'user_id'                => $user->id,
                'price'                  => $user->pivot->price,
                'rank'                   => $user->pivot->rank
            ];
        }
        $workTemplate->templateStagesOrdered()->where(
            'work_template_stages.work_stage_id',
            $newWorkStageParameters['work_stage_id']
        )->where('work_template_stages.work_template_id', $workTemplate->id)->first()->workStageUsersOrdered()->sync(
            $workTemplateStageUsers
        );
    }


    public function reorderStages(WorkTemplate $workTemplate)
    {
        $workTemplateStageFields = $this->workTemplateStageFields;

        $workTemplateStages = $workTemplate->templateStagesOrdered()->get($workTemplateStageFields)->toArray();

        $reorderedWTS = [];

        $rankNumber = 1;
        foreach ($workTemplateStages as $workTemplateStage) {
            $workTemplateStage['rank'] = $rankNumber;
            $reorderedWTS[$workTemplateStage['work_stage_id']] = $workTemplateStage;
            $rankNumber++;
        }
        return $workTemplate->workStagesOrdered()->sync($reorderedWTS);
    }


    private function validateUpdatingWorkStagesRequest(WorkTemplate $workTemplate, $validatedRequestData)
    {
        //unique ranks
        $requestRanks = array_column($validatedRequestData['work_template_stages'], 'rank');
        $uniqueRequestRanks = array_unique($requestRanks);
        if ($requestRanks !== $uniqueRequestRanks) {
            throw new ValidationException('Invalid ranks');
        }
        //unique work stages
        $requestWorkStagesIds = array_column($validatedRequestData['work_template_stages'], 'work_stage_id');
        $uniqueRequestWorkStagesIds = array_unique($requestWorkStagesIds);
        if ($uniqueRequestWorkStagesIds !== $requestWorkStagesIds) {
            throw new ValidationException('Invalid work stages id');
        }

        $workTemplateStages = $workTemplate->templateStagesOrdered()->get();

        //same work stage in request as in db
        $workStageUsersIds = $workTemplateStages->pluck('work_stage_id')->toArray();

        if (count($workTemplateStages) !== count($requestWorkStagesIds)
            || count(array_intersect($requestWorkStagesIds, $workStageUsersIds)) !== count($workStageUsersIds)
        ) {
            throw new ValidationException('Updating work stages not the same as on work template.');
        }

        return $workTemplateStages;
    }
}
