<?php

namespace App\Http\Controllers;

use App\Exceptions\Api\AccessDeniedException;
use App\Exceptions\Api\ServerError;
use App\Exceptions\Api\ValidationException;
use App\Http\Requests\WorkTemplateStagesUpdateRequest;
use App\Http\Requests\WorkTemplateStageUserDeleteRequest;
use App\Http\Requests\WorkTemplateStageUserRequest;
use App\Http\Requests\WorkTemplateStageUsersIndexRequest;
use App\Http\Requests\WorkTemplateStageUsersUpdateRequest;
use App\Http\Resources\WorkTemplateStageUserCollection;
use App\Http\Resources\WorkTemplateStageUserResource;
use App\Models\WorkTemplate;
use App\Models\WorkTemplateStageUser;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Http\Response;

/**
 * @group Sabloane - utilizatori etape
 *
 * APIs for managing work templates
 */
class WorkTemplateStageUserController extends Controller
{
    private $workTemplateStageUserFields
        = [
            'work_template_stage_id',
            'work_stage_id',
            'user_id',
            'rank',
            'price',
        ];


    /**
     * Listing of work template stage users
     *
     * @apiResourceCollection   App\Http\Resources\WorkTemplateStageUserCollection
     * @apiResourceModel App\Models\WorkTemplateStageUser
     *
     * @param  WorkTemplateStageUsersIndexRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function index(WorkTemplateStageUsersIndexRequest $request)
    {
        if ($request->user()->cannot('work_templates.read')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.read.missing.permission'));
        }

        $validated = $request->validated();
        $workTemplate = WorkTemplate::where('id', $validated['work_template_id'])->firstOrFail();
        $workTemplateStageUsers = $workTemplate->templateStagesOrdered()->where('work_stage_id', $validated['work_stage_id'])->first()
            ->workStageUsersOrdered()->get()->loadMissing(['roles']);
        return response(['data' => new WorkTemplateStageUserCollection($workTemplateStageUsers)]);
    }


    /**
     * Creates a work template stage user.
     *
     * @apiResource  App\Http\Resources\WorkTemplateStageUserResource
     * @apiResourceModel App\Models\WorkTemplateStageUser
     *
     * @param  WorkTemplateStageUserRequest  $request
     *
     * @return Application|ResponseFactory|Response
     */
    public function store(WorkTemplateStageUserRequest $request)
    {
        if ($request->user()->cannot('work_templates.update')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.update.missing.permission'));
        }
        $validated = $request->validated();

        $workTemplate = WorkTemplate::where('id', $validated['work_template_id'])->firstOrFail();
        $workTemplateStage = $workTemplate->templateStagesOrdered()->where('work_stage_id', $validated['work_stage_id'])->firstOrFail();
        $wtsUser = $workTemplateStage->templateStageUsersOrdered()->where('user_id', $validated['user_id'])->first();
        if ($wtsUser !== null) {
            throw new ValidationException('User already exist. Please use update.');
        }

        $user = $this->userCreatingData($validated);

        $workTemplateStage->workStageUsersOrdered()->syncWithoutDetaching($user);
        $wtsUser = $workTemplateStage->workStageUsersOrdered()->where('user_id', $validated['user_id'])->first();
        return response(['data' => new WorkTemplateStageUserResource($wtsUser)], 201);
    }


    /**
     * Remove a work template stage user.
     *
     * @param  \App\Models\WorkTemplateStage  $workTemplateStage
     *
     * @response
     * @return Response
     */
    public function destroy(WorkTemplateStageUserDeleteRequest $request)
    {
        if ($request->user()->cannot('work_templates.update')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.update.missing.permission'));
        }

        $validated = $request->validated();

        $wtsUser = WorkTemplateStageUser::where('work_stage_id', $validated['work_stage_id'])->where(
            'work_template_id',
            $validated['work_template_id']
        )->where('user_id', $validated['user_id'])->firstOrFail();
        $wtsUser->forceDelete();

        $this->reorderTemplateStageUsers($validated['work_template_id'], $validated['work_stage_id']);

        return response(['data' => __("work template stage user has been deleted")], 204);
    }

    /**
     * WTS update work template stages.
     *
     * @apiResourceCollection   App\Http\Resources\WorkTemplateStageUserCollection
     * @apiResourceModel App\Models\WorkTemplateStageUser
     *
     * @param  WorkTemplateStagesUpdateRequest  $request
     *
     * @return Application|ResponseFactory|Response
     * @throws AccessDeniedException
     * @throws ValidationException
     */
    public function updateWorkTemplateStageUsers(WorkTemplateStageUsersUpdateRequest $request)
    {
        if ($request->user()->cannot('work_templates.update')) {
            throw new AccessDeniedException(__('lang_permissions.work_templates.update.missing.permission'));
        }

        $validated = $request->validated();

        $wtsUsers = $this->validateUpdatingWTSUsersRequest($validated);

        foreach ($validated['work_template_stage_users'] as $rank => $wtsUser) {
            $updatingWTSUser = $wtsUsers->where('user_id', $wtsUser['user_id'])->first();
            $updatingWTSUser->rank = $rank+1;
            $updatingWTSUser->price = $wtsUser['price']??$updatingWTSUser->price;
            $updatingWTSUser->save();
        }

        $template = WorkTemplate::where('id', $validated['work_template_id'])->firstOrFail();
        $wtStage = $template->templateStagesOrdered()->where('work_stage_id', $validated['work_stage_id'])->firstOrFail();
        $wtsUsers = $wtStage->workStageUsersOrdered()->get();
        $this->reorderTemplateStageUsers($validated['work_template_id'], $validated['work_stage_id']);

        return response(['data' => new WorkTemplateStageUserCollection($wtsUsers)]);
    }

    /**
     * @param $workTemplateStageId
     * @param $workTemplateId
     *
     * @return mixed
     */
    public function lastRankForTemplateWorkStage($workTemplateId, $workTemplateStageId)
    {
        return WorkTemplateStageUser::where('work_stage_id', $workTemplateStageId)->where('work_template_id', $workTemplateId)->get()
            ->pluck('rank')->max();
    }

    /**
     * @param $workTemplateStageId
     * @param $workTemplateId
     *
     * @return int
     */
    public function nextRankForTemplateWorkStage($workTemplateId, $workTemplateStageId): int
    {
        $nextRank = 0;
        $lastRank = $this->lastRankForTemplateWorkStage($workTemplateId, $workTemplateStageId);
        if ($lastRank !== null) {
            $nextRank = $lastRank + 1;
        }
        return $nextRank;
    }

    public function userCreatingData(array $parameters): array
    {
        $user[$parameters['user_id']] = [];
        $user[$parameters['user_id']]['rank'] = $this->nextRankForTemplateWorkStage(
            $parameters['work_template_id'],
            $parameters['work_stage_id']
        );

        foreach ($parameters as $keyName => $keyValue) {
            $user[$parameters['user_id']][$keyName] = $keyValue;
        }

        return $user;
    }

    /**
     * @param  int  $workTemplateId
     * @param  int  $workTemplateStageId
     *
     * @throws ServerError
     */
    public function reorderTemplateStageUsers(int $workTemplateId, int $workTemplateStageId): void
    {
        try {
            $wtsUsers = WorkTemplateStageUser::where('work_stage_id', $workTemplateStageId)->where('work_template_id', $workTemplateId)
                ->orderBy('rank')->get();

            foreach ($wtsUsers as $rank => $wtsUser) {
                $wtsUser->rank = $rank+1;
                $wtsUser->save();
            }
        } catch (Exception $e) {
            throw new ServerError($e->getMessage());
        }
    }


    private function validateUpdatingWTSUsersRequest($validatedRequestData)
    {
        //unique ranks
        $requestRanks = array_column($validatedRequestData['work_template_stage_users'], 'rank');
        $uniqueRequestRanks = array_unique($requestRanks);
        if ($requestRanks !== $uniqueRequestRanks) {
            throw new ValidationException('Invalid ranks');
        }
        //unique users
        $requestUsersIds = array_column($validatedRequestData['work_template_stage_users'], 'user_id');
        $uniqueRequestUsersIds = array_unique($requestUsersIds);
        if ($requestUsersIds !== $uniqueRequestUsersIds) {
            throw new ValidationException('Invalid users id');
        }

        $wtsUsers = WorkTemplateStageUser::where('work_stage_id', $validatedRequestData['work_stage_id'])->where(
            'work_template_id',
            $validatedRequestData['work_template_id']
        )->orderBy('rank')->get();

        //same users in request as in db
        $wtsUsersIds = $wtsUsers->pluck('user_id')->toArray();

        if (count(array_intersect($requestUsersIds, $wtsUsersIds)) !== count($wtsUsersIds)) {
            throw new ValidationException('Updating users not the same as on work template stage.');
        }

        return $wtsUsers;
    }
}
