<?php

namespace App\Models;

use App\Scopes\OrderAdditionalPriceScope;
use App\Scopes\OrderScope;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Spatie\MediaLibrary\HasMedia;
use Spatie\MediaLibrary\InteractsWithMedia;

class Order extends Model implements HasMedia
{
    use HasFactory;
    use SoftDeletes;
    use InteractsWithMedia;

    //default name for order files collection
    public const ORDER_FILES_COLLECTION_NAME = 'order-files';


    public static $searchable
        = [
            'id',
            'order_no',
            'status',
            'order_status',
            'date_added',
            'start_date',
            'collaborator',
            'dentist',
            'patient',
            'collaborator_id',
            'payment_document_id',
            'cancelled'
        ];

    /**
     * The attributes that should be cast.
     *
     * @var array
     */
    protected $casts
        = [
            'date_added'          => 'custom_datetime:Y-m-d',
            'start_added'         => 'custom_datetime:Y-m-d',
            'order_no'            => 'integer',
            'collaborator_id'     => 'integer',
            'dentist_id'          => 'integer',
            'patient_id'          => 'integer',
            'payment_document_id' => 'integer',
            'invoice_id'          => 'integer',
            'invoice_order'       => 'integer',
            'invoice_order_no'    => 'integer',
            'cancelled'           => 'boolean',
            'remake_id'           => 'integer',
            'stage_id'            => 'integer',
            'user_id'             => 'integer',
            'remake_of'           => 'integer',
            'has_order_details'   => 'boolean',
        ];

    protected $fillable
        = [
            'start_date',
            'collaborator_id',
            'dentist_id',
            'patient_id',
            'dentition_color',
            'dentition_shape',
            'face_shape',
            'order_details',
        ];

    protected $dates = ['date_added', 'start_added'];

    /**
     * The "booted" method of the model.
     *
     * @return void
     */
    protected static function booted()
    {
        static::addGlobalScope(new OrderScope);
    }

    public function user()
    {
        return $this->belongsTo(User::class);
    }

    public function invoice()
    {
        return $this->belongsTo(Invoice::class);
    }

    public function templates()
    {
        return $this->belongsToMany(WorkTemplate::class, 'order_work_templates', 'order_id', 'work_template_id')->withPivot('id');
    }

    public function works()
    {
        return $this->hasMany(OrderWorkTemplate::class, 'order_id');
    }

    public function worksNameAndSymbol()
    {
        return $this->hasMany(OrderWorkTemplate::class, 'order_id')->select(['order_id', 'name', 'symbol']);
    }

    public function payment_document()
    {
        return $this->belongsTo(PaymentDocument::class);
    }

    public function payments()
    {
        return $this->hasMany(Payment::class);
    }

    public function order_payments()
    {
        return $this->hasMany(OrderPayment::class);
    }

    public function discounts()
    {
        return $this->hasMany(Discount::class);
    }

    public function order_discounts()
    {
        return $this->hasMany(OrderDiscount::class);
    }

    public function patient()
    {
        return $this->belongsTo(Patient::class);
    }

    public function collaborator()
    {
        return $this->belongsTo(Collaborator::class);
    }

    public function dentist()
    {
        return $this->belongsTo(Dentist::class);
    }

    public function stages()
    {
        return $this->belongsToMany(WorkStage::class, 'order_work_stages', 'order_id', 'work_stage_id')->withPivot(
            ['id', 'day_number', 'order_work_template_id', 'ready_time', 'status']
        );
    }

    public function workStagesOrdered()
    {
        return $this->belongsToMany(WorkStage::class, 'order_work_stages', 'order_id', 'work_stage_id')->orderBy('order_work_stages.rank')
            ->withPivot(['id', 'rank', 'work_template_id', 'work_stage_id', 'day_number']);
    }

    public function stagesWithoutPivot()
    {
        return $this->hasMany(OrderWorkStage::class, 'order_id');
    }

    public function additionalPrices()
    {
        return $this->hasMany(OrderAdditionalPrice::class);
    }

    public function additionalPricesWithoutGlobalScope()
    {
        return $this->hasMany(OrderAdditionalPrice::class)->withoutGlobalScope(OrderAdditionalPriceScope::class);
    }

    public function orderStagesWithoutRemakes()
    {
        return $this->hasMany(OrderWorkStage::class, 'order_id')->where('remake_id', '<', 1);
    }

    public function scopeJoinPatients($query)
    {
        $query->addSelect(\DB::raw("CONCAT(patients.last_name, ' ', patients.first_name) as patient_name"));

        return $query->leftJoin('patients', 'orders.patient_id', '=', 'patients.id');
    }

    public function scopeJoinCollaborators($query)
    {
        $query->addSelect("collaborators.name as collaborator_name");

        return $query->leftJoin('collaborators', 'orders.collaborator_id', '=', 'collaborators.id');
    }

    public function scopeJoinDentists($query)
    {
        $query->addSelect(\DB::raw("CONCAT(dentists.last_name, ' ', dentists.first_name) as dentist_name"));

        return $query->leftJoin('dentists', 'orders.dentist_id', '=', 'dentists.id');
    }

    public function hasRemake()
    {
        return $this->belongsTo(Order::class, 'remake_id', 'id');
    }

    public function remakeOf()
    {
        return $this->hasOne(Order::class, 'id', 'remake_of');
    }

    public function scopeJoinFirstAvailableStage($query)
    {
        $firstByOrderSql = OrderWorkStage::select(
            'order_id',
            \DB::raw('MIN(`rank`) AS `rank`'),
            \DB::raw('MIN(`order_work_template_id`) AS `order_work_template_id`')
        )->where('status', '<>', \DB::raw("'finished'"))->whereRaw(
                'order_work_template_id IN ( select MIN(order_work_template_id) AS order_work_template_id
                                         from order_work_stages
                                         where `status` <> "finished"
                                         group by `order_id` )'
            )->groupBy('order_id')->toSql();

        $ifs = "IF(
                    `order_work_stages`.`type` = 'trial' || `order_work_stages`.`type` = 'external',
                    IF(
                        `order_work_stages`.`status` = 'working',
                        'in_trial',
                        'waiting_for_trial'
                        ),
                    IF(
                        `order_work_stages`.`status` = 'working',
                        'working',
                        'waiting'
                        )
                ) as `order_status`";

        $stages = OrderWorkStage::select(
            'order_work_stages.rank as stage_rank',
            'order_work_stages.id as stage_id',
            'order_work_stages.user_id as user_id',
            'order_work_stages.order_id',
            'order_work_stages.status',
            'order_work_stages.type',
            \DB::raw($ifs)
        )->join(\DB::raw('('.$firstByOrderSql.') as first_by_order'), function ($join) {
                $join->on('order_work_stages.order_id', '=', 'first_by_order.order_id')->on(
                        'order_work_stages.rank',
                        '=',
                        'first_by_order.rank'
                    )->on('order_work_stages.order_work_template_id', '=', 'first_by_order.order_work_template_id');
            })->where('status', '<>', \DB::raw("'finished'"))->toSql();

        $orderIfs = "IF(
                    `orders`.`status` = 'creating' || `orders`.`status` = 'complete',
                    `orders`.`status`,
                        IFNULL(first.order_status, 'expecting_completion')
                    ) as order_status";

        $query->addSelect(\DB::raw($orderIfs));
        $query->addSelect(\DB::raw('first.stage_id as stage_id'));
        $query->addSelect(\DB::raw('first.user_id as user_id'));

        return $query->leftJoin(\DB::raw('('.$stages.') as first'), 'first.order_id', '=', 'orders.id');
    }

    public function getFilesAttribute()
    {
        return $this->getMedia(self::ORDER_FILES_COLLECTION_NAME);
    }

    /**
     * must have for files registration
     */
    public function registerMediaCollections(): void
    {
        $this->addMediaCollection(static::ORDER_FILES_COLLECTION_NAME);
    }


    /**
     * Remake order file
     *
     * @param  Request  $request
     * @param $mediaUUID
     *
     * @return \Illuminate\Contracts\Routing\ResponseFactory|\Illuminate\Http\Response
     */
    public function remakeFile(Request $request, $mediaUUID)
    {
        $order = Order::findOrFail(22);
        $mediaItems = $order->getMedia();
        dd($mediaItems);
    }

}
