<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class PaymentDocument extends Model
{
    use HasFactory;
    use SoftDeletes;

    protected $fillable
    = [
        'collaborator_id',
        'dentist_id',
        'lab_invoice_profile_id',
        'invoice_id',
        'invoice_series',
        'invoice_no',
        'invoice_date',
        'proforma_no',
        'proforma_series',
    ];

    protected $dates = [
        'invoice_date',
        'created_at',
        'updated_at',
    ];

    public function getDentistNameAttribute()
    {
        $all = new \stdClass();
        $all->dentist_name = ((int)$this->dentist_id < 1) ? __('Toti') : $this->dentist->last_name . ' ' . $this->dentist->first_name;
        return $all;
    }

    public function getInvoiceDateFormattedAttribute()
    {
        return $this->invoice_date ? date('Y-m-d', strtotime($this->invoice_date)) : null;
    }

    public function getCreatedAtFormattedAttribute()
    {
        return $this->created_at ? date('Y-m-d', strtotime($this->created_at)) : null;
    }

    public function getAllReceiptsAreIssuedAttribute()
    {
        if (!count($this->payments)) return null;

        foreach ($this->payments as $payment) {
            if ($payment->type === 'cash' && (!$payment['receipt_series'] || !$payment['receipt_no'])) {
                return false;
            }
        }
        return true;
    }

    public function orders()
    {
        return $this->hasMany(Order::class);
    }

    public function discounts()
    {
        return $this->hasMany(Discount::class);
    }

    public function collaborator()
    {
        return $this->belongsTo(Collaborator::class);
    }

    public function dentist()
    {
        return $this->belongsTo(Dentist::class);
    }

    public function payments()
    {
        return $this->hasMany(Payment::class);
    }

    public function cashPayments()
    {
        return $this->hasMany(Payment::class)->where('payments.type', 'cash');
    }

    public function labInvoiceProfile()
    {
        return $this->belongsTo(LabInvoiceProfile::class);
    }

    public function invoice()
    {
        return $this->belongsTo(Invoice::class);
    }

    public function newQuery($excludeDeleted = true)
    {
        return parent::newQuery($excludeDeleted)->select(
            'payment_documents.*',
            \DB::raw("IFNULL(orders.value, 0) - IFNULL(discounts.value, 0) as value"),
            \DB::raw("IFNULL(payments.value, 0) as payed"),
            \DB::raw("IFNULL(orders.value, 0) - IFNULL(discounts.value, 0) - IFNULL(payments.value, 0) as to_pay")
        )
            ->leftJoin(
                \DB::raw(
                    '(SELECT payment_document_id, SUM(value) as value FROM orders WHERE orders.deleted_at IS NULL GROUP BY orders.payment_document_id) as orders'
                ),
                function ($join) {
                    $join->on('payment_documents.id', '=', 'orders.payment_document_id');
                }
            )->leftJoin(
                \DB::raw(
                    '(SELECT payment_document_id, SUM(value) as value FROM payments WHERE payments.deleted_at IS NULL GROUP BY payments.payment_document_id) as payments'
                ),
                function ($join) {
                    $join->on('payment_documents.id', '=', 'payments.payment_document_id');
                }
            )->leftJoin(
                \DB::raw(
                    '(SELECT payment_document_id, SUM(value) as value FROM discounts WHERE discounts.deleted_at IS NULL GROUP BY discounts.payment_document_id) as discounts'
                ),
                function ($join) {
                    $join->on('payment_documents.id', '=', 'discounts.payment_document_id');
                }
            )->groupBy('payment_documents.id');
    }

    public function scopeJoinCollaborators($query)
    {
        $query->addSelect("collaborators.name as collaborator");
        $query->addSelect("collaborators.cui as cui");
        $query->addSelect("collaborators.nr_reg_comertului as nr_reg_comertului");
        $query->addSelect("collaborators.address as address");
        $query->addSelect("collaborators.region_id as region_id");
        $query->addSelect("collaborators.city as city");
        $query->addSelect("collaborators.phone_fax as phone_fax");
        $query->addSelect("collaborators.mobile as mobile");
        $query->addSelect("collaborators.contact_mobile as contact_mobile");
        $query->addSelect("collaborators.email as email");
        $query->addSelect("collaborators.website as website");

        return $query->leftJoin('collaborators', 'payment_documents.collaborator_id', '=', 'collaborators.id');
    }

    public function scopeJoinDentists($query)
    {
        $query->addSelect(
            \DB::raw("IF(ISNULL(dentists.first_name), 'Toti', CONCAT_WS(' ', dentists.last_name, dentists.first_name)) as dentist")
        );
        return $query->leftJoin('dentists', 'payment_documents.dentist_id', '=', 'dentists.id');
    }
}
