<?php

namespace App\Models;

use App\Exceptions\Api\ValidationException;
use Exception;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Collection;
use Illuminate\Support\Str;
use Laravel\Sanctum\HasApiTokens;
use Laravel\Sanctum\NewAccessToken;
use Spatie\MediaLibrary\HasMedia;
use Spatie\MediaLibrary\InteractsWithMedia;
use Spatie\Permission\Models\Role;
use Spatie\Permission\Traits\HasRoles;

class User extends Authenticatable implements HasMedia
{
    use HasApiTokens;
    use HasFactory;
    use HasRoles;
    use Notifiable;
    use InteractsWithMedia;


    /**
     * Sorting default when we use
     * Spatie\QueryBuilder\QueryBuilder.
     *
     * sort asc ['id'] , sort desc ['-id']
     *
     * @var array
     */
    public static $defaultSort = ['last_name', 'first_name'];

    public static $sortable
    = [
        'id',
        'username',
        'first_name',
        'last_name',
        'email',
        'mobile',
        'phone_fax',
        'website',
        'address',
        'status'
    ];

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable
    = [
        'username',
        'first_name',
        'last_name',
        'email',
        'mobile',
        'phone_fax',
        'website',
        'address',
        'status',
        'use_profile_pic',
        'profile_pic',
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden
    = [
        'password',
        'remember_token',
        'created_at',
        'updated_at',
        'two_factor_secret',
        'two_factor_recovery_codes',
        'deleted_at'
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts
    = [
        'email_verified_at' => 'datetime',
        'use_profile_pic'   => 'boolean',
        'on_vacation'       => 'boolean',
        'technicians_on_vacation' => 'int'
    ];

    /**
     * @param  User  $user
     *
     * @return Collection
     */
    public static function userPermissionsNames(User $user)
    {
        return $user->getAllPermissions()->pluck('name');
    }


    /**
     * @param $query
     *
     * @return mixed
     */
    public function scopeNotSuperAdmin($query)
    {
        return $query->whereHas(
            'roles',
            function ($query) {
                $query->where('roles.name', '<>', config('custom.api.super_admin_role'));
            }
        )->orDoesntHave('roles');
    }

    /**
     * Create a new personal access token for the user.
     *
     * @param  string  $username
     * @param  array  $abilities
     *
     * @return NewAccessToken
     */
    public function createToken(string $username, array $abilities = ['*'])
    {
        try {
            $identificator = explode(".", $username)[0];
        } catch (Exception $e) {
            abort(500);
        }
        $plainTextToken = $identificator . '|' . Str::random(40 - strlen($identificator));
        $token = $this->tokens()->create(
            [
                'name'      => $username,
                'token'     => hash('sha256', $plainTextToken),
                'abilities' => $abilities,
            ]
        );

        return new NewAccessToken($token, $token->id . '|' . $plainTextToken);
    }

    public function workStageUsers()
    {
        return $this->hasMany(WorkStageUser::class);
    }

    public function department()
    {
        return $this->belongsTo(Department::class);
    }

    /**
     * @param  User  $user
     * @param  array  $rolesIds
     *
     * @throws ValidationException
     */
    public function updateUserRoles(User $user, array $rolesIds): void
    {
        $rolesNames = Role::where('name', '<>', config('custom.api.super_admin_role'))->whereIn('id', $rolesIds)->get();
        if ($rolesNames->count() !== count($rolesIds)) {
            throw new ValidationException('Could not attach roles. Invalid roles ids.');
        }
        $user->syncRoles([$rolesNames->pluck('name')]);
    }

    /**
     * @return Collection
     */
    public function userPermissions()
    {
        if ($this->hasRole([config('custom.api.super_admin_role')])) {
            return Collection::make(Permission::all());
        }

        return $this->getAllPermissions();
    }

    /**
     * Get the user suffix.
     *
     * @return string
     */
    public function getUsernameSuffixAttribute()
    {
        return explode(tenant('id') . config('custom.api.username_identifier_delimiter'), $this->username)[1] ?? null;
    }

    /**
     * Get the user suffix.
     *
     * @return string
     */
    public function getUsernameIdentifierAttribute()
    {
        return tenant('id');
    }

    /**
     * Get the user suffix.
     *
     * @return string
     */
    public function getRolesNameAttribute()
    {
        return $this->roles->pluck('name');
    }

    public function registerMediaCollections(): void
    {
        $this->addMediaCollection('picture')->singleFile();
    }

    public function getProfilePicAttribute()
    {
        $picture = $this->getFirstMedia('picture');
        return $picture !== null ? $picture->getFullUrl() : $picture;
    }
}
