<?php

namespace App\Models;

use App\Casts\FloatTwoDecimal;
use App\Exceptions\Api\ServerError;
use App\Traits\HasMultiFactorDentition;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class WorkTemplate extends Model
{
    use HasFactory;
    use SoftDeletes;
    use HasMultiFactorDentition;

    protected $touches
    = [
        'workStagesOrdered',
        'templateStagesOrdered'
    ];

    public static $searchable
    = [
        'id',
        'name',
        'min_duration',
        'estimated_duration',
        'cost',
        'extra_cost',
        'price_type'
    ];

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable
    = [
        'name',
        'min_duration',
        'estimated_duration',
        'cost',
        'extra_cost',
        'description',
        'type',
        'price_type',
        'symbol',
        'multi_factor_dentition',
    ];

    protected $casts
    = [
        'min_duration'       => "integer",
        'estimated_duration' => "integer",
    ];

    public function getDentistPreferentialPriceAttribute()
    {
        return $this->hasOne(DentistPrice::class, 'id', $this->id) ? true : false;
    }

    public function workStagesOrdered()
    {
        return $this->belongsToMany(WorkStage::class, 'work_template_stages', 'work_template_id', 'work_stage_id')->orderBy('work_template_stages.rank')
            ->withPivot(['id', 'rank', 'work_template_id', 'work_stage_id', 'day_number']);
    }

    public function templateStagesOrdered()
    {
        return $this->hasMany(WorkTemplateStage::class, 'work_template_id', 'id')->orderBy('rank');
    }

    public function getStagesWithUsers($stageId = null)
    {
        $minRank = \DB::table('work_template_stage_users')
            ->selectRaw('MIN(`work_template_stage_users`.`rank`) as minRank')
            ->get()->first()->minRank ?? 1;
        if ($minRank === 0) {
            //if rank is 0 we will have problems sorting correctly the first user of the work template stage 22.08.2021
            throw  new ServerError(__('#WTSU0002 Template work stage user should have minim ran of 1'));
        }

        $query = $this->workStagesOrdered()
            ->select(
                'work_stages.id as id',
                'work_stages.name as name',
                'work_stages.type as type',
                'work_stages.remake_cost as remake_cost',
                'work_stages.special_cost as special_cost',
                'work_stages.extra_cost_applied as extra_cost_applied',
                'work_stages.price_type as price_type',
                'work_stages.min_duration as min_duration',
                'work_stages.multi_factor_dentition as multi_factor_dentition',
                'work_stages.to_send as to_send',
                'work_template_stages.work_template_id',
                'work_template_stages.work_stage_id as stage_id',
                'work_stages.confirmation_required as confirmation_required',
                \DB::RAW('IFNULL(assigned_users.user_id, 0) as user_id'),
                \DB::RAW('CONCAT_WS(" ", users.first_name, users.last_name) as user_name'),
                \DB::RAW('IFNULL(work_template_stage_users.price, 0) as cost'),
                'work_template_stages.day_number as day_number'
            )
            //first user on template work stage
            ->leftJoin(
                \DB::raw(
                    '(SELECT MIN(`user_id`) user_id,`work_template_stage_id` FROM `work_template_stage_users` WHERE `work_template_stage_users`.`rank` = ' . $minRank . ' GROUP BY `work_template_stage_id` ) assigned_users'
                ),
                function ($join) {
                    $join->on('assigned_users.work_template_stage_id', '=', 'work_template_stages.id');
                }
            )
            ->leftJoin('users', 'users.id', '=', 'assigned_users.user_id')
            ->leftJoin('work_template_stage_users', function ($join) {
                $join->on('work_template_stage_users.work_stage_id', '=', 'work_stages.id')
                    ->on('work_template_stage_users.work_template_id', '=', 'work_template_stages.work_template_id')
                    ->on('work_template_stage_users.user_id', '=', 'assigned_users.user_id');
            })
            ->orderBy('work_template_stages.rank')
            ->orderBy('work_template_stages.id', 'DESC');

        if (!is_null($stageId)) {
            $query->where('work_stages.id', $stageId);
        }
        return $query->get()->toArray();
    }

    public function dentistsPrice()
    {
        return $this->belongsToMany(Dentist::class, 'dentist_prices')->withPivot(['price', 'locked'])
            // ->orderBy("name")
            ->withTimestamps();
    }
}
