<?php

namespace App\Services;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;

class ImportCollaboratorService
{
    /**
     * Set the request parameters.
     *
     * @param string $cif
     */
    public static function setRequestParams(string $cif)
    {
        $full_path  = "https://api.openapi.ro/api/companies/" . $cif;
        $http       = new Client();

        try {
            $request    = $http->get($full_path, [
                'headers'         => [
                    'cache-control' => 'no-cache',
                    'content-type'  => 'application/x-www-form-urlencoded',
                    'x-api-key'     => env('OPENAPI_API_KEY'),
                ],
                'timeout'         => 30,
                'connect_timeout' => true,
                'http_errors'     => true,
                'verify'          => false,
            ]);
        } catch (ClientException $e) {
            $request    = $e->getResponse();
        }

        return $request;
    }

    /**
     * Request company data from Openapi.
     *
     * @param string $cif
     */
    private static function requestCompanyData(string $cif)
    {
        $request    = self::setRequestParams($cif);
        $response   =  $request->getBody()->getContents();

        if ($response !== 'null') {
            return $response;
        }

        return null;
    }

    /**
     * Get company details.
     *
     * @param string $cif
     * @return array
     */
    public static function getCompanyDetails(string $cif)
    {
        $json       = self::requestCompanyData($cif);
        $jsonArray  = json_decode($json, true);

        return $jsonArray;
    }

    /**
     * Get company data.
     *
     * @param string $cif
     * @return array
     */
    public static function getCompanyData(string $cif)
    {
        $noPrefixCUI = preg_replace('/^(RO|ro|Ro|rO)/', '', $cif);
        $companyDetails = self::getCompanyDetails($noPrefixCUI);

        if ($companyDetails['error'] ?? false) {
            $companyDetails['message'] = __($companyDetails['error']['description']) ?? __($companyDetails['description']);
            $companyDetails['status']  = $companyDetails['error']['status'] ?? $companyDetails['status'];
            $companyDetails['errors']  = [
                'description' => [
                    $companyDetails['message']
                ]
            ];
            unset($companyDetails['error']);
            return $companyDetails;
        }
        $postalCode     = $companyDetails['cod_postal'] ? (', CP ' . $companyDetails['cod_postal']) : '';

        $company['name']                = $companyDetails['denumire'];
        $company['cui']                 = strtoupper($cif);
        $company['nr_reg_comertului']   = $companyDetails['numar_reg_com'];
        $company['address']             = $companyDetails['adresa'] . $postalCode;
        $company['region']              = str_replace("Municipiul ", "", $companyDetails['judet']);
        $company['phone_fax']           = $companyDetails['fax'];
        $company['mobile']              = $companyDetails['telefon'];

        return $company;
    }
}
