<?php

namespace App\Services;

use App\Exceptions\Api\AccessDeniedException;
use App\Exceptions\Api\NotFound;
use App\Exceptions\Api\ValidationException;
use App\Models\Order;
use App\Models\OrderPayment;
use App\Services\Reports\OrdersService;

class OrderPaymentService
{

    public static function addPayment(Order $order, array $paymentParameters)
    {
        self::canAddPayment($order, $paymentParameters['value'], true);
        OrderPayment::create($paymentParameters);
    }

    public static function canAddPayment(Order $order, $value, $throwException = false)
    {
        if (!request()->user()->can(['payment_documents.update'])) {
            if ($throwException) {
                throw new AccessDeniedException(__('lang_permissions.payment_documents.update.missing.permission'));
            }
            return false;
        }

        OrdersService::addPaymentSummaryToOrder($order, false);
        if ($order->paymentSummary['to_pay'] < $value) {
            if ($throwException) {
                throw new ValidationException(__('Valoarea platii depaseste restul de plata'));
            }
            return false;
        }

        return true;
    }

    /**
     * Delete the payment.
     *
     * @param Order $order
     * @param integer $paymentId
     * @return Order
     */
    public static function deletePayment(Order $order, int $paymentId): Order
    {
        $order->loadMissing('order_payments');
        self::canDeletePaymentId($order, $paymentId);
        $payment = OrderPayment::where('order_id', $order->id)->where('id', $paymentId)->firstOrFail();
        $payment->delete();
        return $order;
    }

    /**
     * Check if the user can delete the payment.
     *
     * @param Order $order
     * @param integer $paymentId
     * @param boolean $throwException
     * @return boolean
     */
    public static function canDeletePaymentId(Order $order, int $paymentId, $throwException = false)
    {
        if (!request()->user()->can(['payment_documents.update'])) {
            if ($throwException) {
                throw new AccessDeniedException(__('lang_permissions.payment_documents.update.missing.permission'));
            }
            return false;
        }

        $payment = $order->order_payments()->where('id', $paymentId)->first();

        if ($payment === null) {
            if ($throwException) {
                throw new NotFound(__('Payment could not be found'));
            }
            return false;
        }

        return true;
    }
}
