<?php


namespace App\Services;


use App\Configuration\LabConfiguration;
use App\Exceptions\Api\NotFound;
use App\Exceptions\Api\PossibleUnSyncException;
use App\Exceptions\Api\ServerError;
use App\Exceptions\Api\ValidationException;
use App\Models\Order;
use App\Models\OrderAdditionalPrice;
use App\Models\OrderWorkStage;
use App\Models\OrderWorkTemplate;
use App\Scopes\OrderScope;
use http\Client\Request;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\DB;
use StdClass;
use App\Exceptions\Api\AccessDeniedException;

class OrderService
{


    /**
     * @return int
     */
    public static function getNextOrderNumber(): int
    {
        $lastOrder = Order::orderBy('order_no', 'DESC')->withTrashed()->withoutGlobalScope(OrderScope::class)->first();
        $orderNumber = 1;
        if ($lastOrder !== null) {
            $orderNumber = $lastOrder->order_no + 1;
        }
        return $orderNumber;
    }

    /**
     * calculate order dates based on working days
     *
     * @param $order
     * @param $works
     *
     * @return array
     */
    public static function calculateDays($order, $works): array
    {
        $startDate = $order->start_date;
        $preferredDate = $startDate;

        $orderDates = [];

        foreach ($works as $work) {
            $estimatedDate = OrderDatesCalculatorService::calculateDays($startDate, $work['estimated_duration']);
            $preferredDate = OrderDatesCalculatorService::calculateDays($startDate, $work['preferred_duration']);

            $orderDates[$work['id']] = [
                'min_date'       => $startDate,
                'estimated_date' => $estimatedDate,
                'preferred_date' => $preferredDate
            ];

            $startDate = $preferredDate;
        }

        return [
            'finishDate' => $preferredDate,
            'works'      => $orderDates
        ];
    }


    /**
     * @param  Order  $order
     *
     * @return Order
     */
    public static function addTotalPriceWithOrder(Order $order,$updateRecord = true)
    {
        $total = 0;

        foreach ($order->works as $work) {
            $etapePreturiSuplimentare = [];
            $work->etape_preturi_suplimentare = [];

            $work->prices = new StdClass();
            $work->prices->total = 0;
            $work->prices->stages = 0;
            $work->prices->checkboxStages = 0;
            $work->prices->extra = 0;
            $work->prices->extra_prices = 0;
            $work->prices->combinations = 0;
            $work->prices->base = $work->price * $work->multi_factor;

            // basic price
            $work->prices->total = $work->price * $work->multi_factor;
            $total += $work->price * $work->multi_factor;

            // extra price from preferred duration
            if (!$work->ignore_extra_price) {
                $extra = ($work->estimated_duration - $work->preferred_duration) * $work->extra_price * $work->multi_factor;
                $work->prices->total += $extra;
                $work->prices->extra = $extra;
                $total += $extra;
            }

            // add combinations
            $combinations = $work->combinations ?? [];
            foreach ($combinations as $combination) {
                $work->prices->combinations += $combination->price * $combination->multi_factor;
                $total += $combination->price * $combination->multi_factor;
            }


            $stages = $work->stagesWithoutPivot ?? [];

            foreach ($stages as $stage) {
                if (!$stage->remake_id
                    && !((bool)($stage->extra_cost_applied))
                    && $stage->from_template
                ) {
                    continue;
                }

                if ($stage->remake_id && (!$stage->collaborator_fault_id || $stage->lab_paying)) {
                    continue;
                }

                $multi = 1;
                if ($stage->price_type !== 'fix') {
                    $multi = $stage->multi_factor;
                }

                if (self::isSpecialStage($stage->type)) {
                    if ($stage->price_type === 'fix' && !$stage->confirmed) {
                        $multi = 0;
                    }
                    $work->prices->checkboxStages += $stage->price * $multi;
                } else {
                    $work->prices->stages += $stage->price * $multi;
                    $etapePreturiSuplimentare[] = $stage;
                }

                $work->prices->total += $stage->price * $multi;
                $total += $stage->price * $multi;
                //used on calculations
                $stage->pret_unitar = $stage->price;
                $stage->cantitate = $multi;
                $stage->pret_total = $stage->price * $multi;
            }
            $work->etape_preturi_suplimentare = $etapePreturiSuplimentare;
        }

        $additionalPrices = $order->additionalPrices ?? [];
        foreach ($additionalPrices as $price) {
            $work->prices->extra = $extra ?? 0;
            $total += $price->price;
        }

        $order->temp_value = $total;
        if($updateRecord){
            $order->save();
        }
        $order->total = $total;
        return $order;
    }

    public static function calculateTotalPrice($orderId,$updateRecord = false)
    {
        return self::calculateTotalPriceForIds([$orderId],$updateRecord)[$orderId];
    }

    public static function calculateTotalPriceForIds(array $orderIds,$updateRecord = true)
    {
        $orders = Order::whereIn('id', $orderIds)->with([
                                                             'works' => function ($q) {
                                                                 $q->with('stagesWithoutPivot')->with('combinations')->where('combination_id', 0);
                                                             }
                                                         ])->with('additionalPrices')->withoutGlobalScope(OrderScope::class)->get();
        if($orders->count() !== count($orderIds)){
            throw new ValidationException(__('Could not find all orders'));
        }
        $ordersTotals = [];
        $orders->each(function (&$order) use(&$ordersTotals,$updateRecord){
            $ordersTotals[$order->id] = OrderService::addTotalPriceWithOrder($order,$updateRecord)->total;
        });

        return $ordersTotals;

    }

    public static function orderIdCanBeUpdated(int $orderId, $throwException = true): bool
    {
        $order = Order::find($orderId);
        if ($order === null) {
            if ($throwException) {
                throw new NotFound(__('Order with id :orderId could not be found', ['orderId' => $orderId]));
            }
            return false;
        }
        return self::orderCanBeUpdated($order, $throwException);
    }

    public static function orderCanBeUpdated(Order $order, $throwException = true): bool
    {
        if (!request()->user()->can(['orders.update'])) {
            if ($throwException) {
                throw new AccessDeniedException(__('lang_permissions.orders.update.missing.permission'));
            }
            return false;
        }

        if ($order->invoice_order_no > 0) {
            if ($throwException) {
                throw new ValidationException(__('The order is billed'));
            }

            return false;
        }

        if ($order->status === 'complete') {
            if ($throwException) {
                throw new ValidationException(__('The order is completed'));
            }
            return false;
        }

        return true;
    }

    public static function isOrderComplete(Order $order, $throwException = false): bool
    {
        if ($order->status !== 'complete') {
            if ($throwException) {
                throw new ValidationException(__('The order is not completed'));
            }
            return false;
        }

        return true;
    }

    public static function isOrderStatusIn(Order $order, array $possibleStatuses, $throwException = false): bool
    {
        if (in_array($order->status, $possibleStatuses)) {
            return true;
        }

        if ($throwException) {
            throw new ValidationException(
                __('The order status is not in :possibleStatuses', [
                    'possibleStatuses' => implode(
                        ',',
                        $possibleStatuses
                    )
                ])
            );
        }

        return false;
    }

    public static function isOrderIdStatusIn($orderId, array $possibleStatuses, $throwException = false): bool
    {
        $order = Order::findOrFail($orderId);
        return self::isOrderStatusIn($order, $possibleStatuses, $throwException);
    }

    public static function calculation(Order $order): Order
    {
        $order = self::calculationSpecialStages($order);
        $order = self::calculationWorks($order);
        $order = self::calculationUrgency($order);
        $order = self::calculationPricesTab($order);
        $order = self::calculationSuplementaryPrice($order);
        $order = self::calculationAdditionalPriceTab($order);
        $order = self::orderFilesTab($order);
        $order->actions = OrderService::possibleOrderActions($order);

        return $order;
    }

    public static function calculationUrgency(Order $order)
    {
        $calculation = [];
        foreach ($order->works as $work) {
            $calculation = $work->calculation;

            $calculation["pret_urgenta"] = [
                'id_order_work'         => $work->id,
                'nume'                  => __('Valoare urgenta'),
                'price_type'            => $work->price_type,
                'type'                  => null,
                'icon'                  => null,
                'pret'                  => [
                    'elemente_lucrare'     => $work->elements,
                    'cantitate_urgenta'    => $work->multi_factor,
                    'zile_urgenta'         => ($work->estimated_duration - $work->preferred_duration),
                    'pret_urgenta_de_baza' => $work->extra_price,
                    'pret_urgenta_total'   => $work->prices->extra,
                ],
                "ignora_pret_urgenta"   => (boolean)$work->ignore_extra_price,
                "durata_minima"         => $work->min_duration,
                "data_durata_minima"    => $work->minDate,
                "data_durata_estimata"  => $work->estimatedDate,
                "durata_estimata"       => $work->estimated_duration,
                "data_durata_preferata" => $work->preferredDate,
                "durata_preferata"      => $work->preferred_duration,
                'elemente'              => null

            ];
            $work->calculation = $calculation;
        }

        return $order;
    }

    public static function calculationWorks(Order $order)
    {
        foreach ($order->works as $work) {
            $calculation = $work->calculation;
            $calculation['pret_lucrare'] = [
                'pret_sablon_lucrare' => $work->price,
                'elemente_lucrare'    => $work->elements,
                'cantitate_lucrare'   => $work->multi_factor,
                'pret_lucrare'        => $work->prices->base,
                'pret_combinatii'     => $work->prices->combinations,
                'pret_suplimentar'    => $work->prices->stages,
                'pret_etape_speciale' => $work->prices->checkboxStages,
                'pret_urgenta'        => $work->prices->extra,
                'preturi_extra'       => $work->prices->extra_prices,
                'pret_total_lucrare'  => $work->prices->total + $work->prices->combinations
            ];
            $calculation["sumar_lucrare"] = [];
            foreach ($work->combinations as $combination) {
                $calculation["sumar_lucrare"][] = [
                    'id_order_work' => $combination->id,
                    'nume'          => __('Combinatie: ').$combination->original_name,
                    'descriere'     => __('Tip pret: ').__($combination->price_type),
                    'price_type'    => $combination->price_type,
                    'type'          => null,
                    'icon'          => null,
                    'pret'          => [
                        'pret_sablon_lucrare' => $combination->price,
                        'elemente_lucrare'    => $combination->elements,
                        'cantitate_lucrare'   => $combination->multi_factor,
                        'pret_lucrare'        => $combination->price * $combination->multi_factor
                    ],
                    'elemente'      => null

                ];
            }
            $calculation["sumar_lucrare"][] = [
                'id_order_work' => $work->id,
                'nume'          => __('Lucrare: ').$work->name,
                'descriere'     => __('Tip pret: ').__($work->price_type),
                'price_type'    => $work->price_type,
                'type'          => null,
                'icon'          => null,
                'pret'          => [
                    'pret_sablon_lucrare' => $work->price,
                    'elemente_lucrare'    => $work->elements,
                    'cantitate_lucrare'   => $work->multi_factor,
                    'pret_lucrare'        => $work->prices->base,
                    'pret_combinatii'     => $work->prices->combinations,
                    'pret_suplimentar'    => $work->prices->stages,
                    'pret_etape_speciale' => $work->prices->checkboxStages,
                    'pret_urgenta'        => $work->prices->extra,
                    'preturi_extra'       => $work->prices->extra_prices,
                    'pret_total_lucrare'  => $work->prices->total + $work->prices->combinations
                ],
                'elemente'      => null

            ];
            $work->calculation = $calculation;
        }

        return $order;
    }

    public static function calculationSpecialStages(Order $order)
    {
        $stageTypes = LabConfiguration::config('work_stages_types', 'value', 'assoc');
        foreach ($order->works as $work) {
            $calculation["etape_speciale"] = [];
            $stages = $work->stagesWithoutPivot ?? [];
            foreach ($stages as $stage) {
                if (!self::isSpecialStage($stage->type)) {
                    //nu este etapa speciala
                    continue;
                }

                $stage = self::calculatePricesForStage($stage, $stageTypes);

                $calculation["etape_speciale"][] = [
                    'id_order_work_stage'               => $stage->id,
                    'se_adauga_pretul_daca_e_in_sablon' => $stage->extra_cost_applied,
                    'price_type'                        => $stage->price_type,
                    'type'                              => $stage->type,
                    'icon'                              => [
                        "color"  => $stageTypes[$stage->type]['color'],
                        "symbol" => $stageTypes[$stage->type]['symbol'],
                    ],
                    'nume'                              => $stage->name,
                    'descriere'                         => 'daca e cazul',
                    'pret'                              => [
                        'pret_unitar' => $stage->pret_unitar,
                        'cantitate'   => $stage->cantitate,
                        'pret_total'  => $stage->pret_total
                    ],
                    'elemente'                          => $stage->elemente

                ];
            }
            $work->calculation = $calculation;
        }
        return $order;
    }

    public static function calculatePricesForStage(OrderWorkStage $stage, $stageTypes)
    {
        $cantitate = 1;
        $pretUnitar = $stage->price;
        $elemente = null;

        if ($stage->price_type !== 'fix') {
            $cantitate = $stage->multi_factor;
        }

        // check if confirmed
        if ($stage->type === 'implant' || $stage->type === 'special') {
            if ($stage->price_type === 'fix' && !$stage->confirmed) {
                $cantitate = 0;
            }
        }

        if (self::isSpecialStage($stage->type)) {
            // is a checkbox
            if ($stage->price_type === 'fix' && !$stage->confirmed) {
                $cantitate = 0;
            }
        }

        $total = $pretUnitar * $cantitate;

        if (!$stage->remake_id
            && !((bool)($stage->extra_cost_applied))
            && $stage->from_template
        ) {
            $total = 0;
        }

        if ($stage->remake_id && (!$stage->collaborator_fault_id || $stage->lab_paying)) {
            $total = 0;
        }

        $stage->pret_unitar = $pretUnitar;
        $stage->cantitate = $cantitate;
        $stage->pret_total = $total;
        $stage->elemente = (object)OrderWorkStagesService::getStageDentitionElements($stage);

        return $stage;
    }

    public static function calculationPricesTab(Order $order)
    {
        $stageTypes = LabConfiguration::config('work_stages_types', 'value', 'assoc');
        $pricesTab = [];
        foreach ($order->works as $work) {
            if (!isset($work->calculation['pret_lucrare'])) {
                throw new ServerError(
                    __("#SEMC Missing calculation for work id: :workId from order id: :orderId",
                       ["workId" => $work->id, "orderId" => $order->id])
                );
            }

            $workPrices = $work->calculation['pret_lucrare'];
            $workDetails = [
                "order_work_id"      => $work->id,
                "work_name"          => $work->name,
                "cantitate_lucrare"  => $workPrices['cantitate_lucrare'],
                "pret_total_lucrare" => $workPrices['pret_total_lucrare'],
            ];


            $stages = $work->stagesWithoutPivot ?? [];
            foreach ($stages as $stage) {
                $elemente = [];
                if (self::isSpecialStage($stage->type)) {
                    //adding elements if special stage
                    $elemente = $stage->elemente;
                }
                $stage = OrderService::calculatePricesForStage($stage, $stageTypes);
                $workDetails['etape'][] = [
                    'id_order_work_stage'               => $stage->id,
                    'rank'                              => $stage->rank,
                    'status'                            => $stage->status,
                    'status_translation'                => OrderWorkStagesService::translateStageStatus($stage->status),
                    'se_adauga_pretul_daca_e_in_sablon' => $stage->extra_cost_applied,
                    'price_type'                        => $stage->price_type,
                    'type'                              => $stage->type,
                    'etapa_se_trimite'                  => (bool)$stage->to_send,
                    'date'                              => $stage->date,
                    'time'                              => $stage->time,
                    'icon'                              => [
                        "color"  => $stageTypes[$stage->type]['color'],
                        "symbol" => $stageTypes[$stage->type]['symbol'],
                    ],
                    'nume'                              => $stage->name,
                    'etapa_refacuta'                    => (boolean)$stage->remake_id,
                    'etapa_refacuta_de_etapa_cu_id'     => $stage->remake_id,
                    'cine_plateste'                     => [
                        'vina_tehnicianului'   => (boolean)$stage->user_fault_id,
                        'vina_colaboratorului' => (boolean)$stage->collaborator_fault_id,
                        'colaborator'          => $order->collaborator,
                        'plateste_laboratorul' => (boolean)$stage->lab_paying,
                    ],
                    'etapa_sablon'                      => [
                        'id_sablon'                  => $stage->from_template,
                        'etapa_apartine_unui_sablon' => (bool)$stage->from_template
                    ],
                    'stage_actions'                     => OrderWorkStagesService::possibleStageActions(request(), $stage),
                    'stage_asigned_user'                => [
                        'user_id' => $stage->user_id,
                        'name'    => $stage->user_name,
                        'cost'    => [
                            'cantitate'  => $stage->multi_factor,
                            'elemente'   => $stage->elements,
                            'cost_baza'  => $stage->cost,
                            'cost_total' => number_format((float) $stage->multi_factor * $stage->cost, 2, '.', ''),
                        ]
                    ],
                    'stage_possible_users'              => self::possibleUsersForStage($stage),
                    'elemente'                          => $elemente,
                    "pret"                              => [
                        'pret_inclus_in_sablon' => OrderWorkStagesService::isStagePriceIncludedInTemplate($stage),
                        'pret_unitar'           => $stage->pret_unitar,
                        'cantitate'             => $stage->cantitate,
                        'elemente'              => $stage->elements,
                        'pret_total'            => $stage->pret_total,
                    ],

                ];
            }
            $pricesTab[] = $workDetails;
        }

        $order->price_tab = $pricesTab;

        return $order;
    }

    public static function calculationSuplementaryPrice(Order $order)
    {
        $stageTypes = LabConfiguration::config('work_stages_types', 'value', 'assoc');
        foreach ($order->works as $work) {
            if (!isset($work->etape_preturi_suplimentare)) {
                throw new ServerError(
                    __("#SEMSP Missing suplementary prices stages for work id: :workId from order id: :orderId",
                       ["workId" => $work->id, "orderId" => $order->id])
                );
            }
            $calculation = $work->calculation;
            $calculation['preturi_suplimentare'] = [];

            $stages = $work->etape_preturi_suplimentare;
            foreach ($stages as $stage) {
                $stage = OrderService::calculatePricesForStage($stage, $stageTypes);
                $calculation["preturi_suplimentare"][] = [
                    'id_order_work_stage'               => $stage->id,
                    'se_adauga_pretul_daca_e_in_sablon' => $stage->extra_cost_applied,
                    'price_type'                        => $stage->price_type,
                    'type'                              => $stage->type,
                    'icon'                              => [
                        "color"  => $stageTypes[$stage->type]['color'],
                        "symbol" => $stageTypes[$stage->type]['symbol'],
                    ],
                    'nume'                              => $stage->name,
                    'pret'                              => [
                        'pret_unitar' => $stage->pret_unitar,
                        'cantitate'   => $stage->cantitate,
                        'pret_total'  => $stage->pret_total
                    ]

                ];
            }
            $work->calculation = $calculation;
        }


        return $order;
    }

    public static function possibleUsersForStage(OrderWorkStage $stage)
    {
        $users = $stage->workStage->workTemplateStageUsers ?? [];
        $possibleUsers = [];
        foreach ($users as $user) {
            if ((int)$user->pivot->work_template_id === (int)$stage->work->work_template_id) {
                $possibleUsers[] = [
                    "first_name"    => $user->first_name,
                    "last_name"     => $user->last_name,
                    "cost"          => $user->pivot->price,
                    "rank"          => $user->pivot->rank,
                    "user_id"       => (integer)$user->pivot->user_id,
                    "work_stage_id" => $user->pivot->work_stage_id,
                ];
            }
        }
        return $possibleUsers;
    }

    public static function sendOrderStagesToProd(Order $order)
    {
        DB::transaction(
            function () use ($order) {
                $stages = $order->refresh()->stages()->get();
                foreach ($stages as $stage) {
                    $stage->pivot->status = 'not_ready';
                    $stage->save();
                    $stage->pivot->save();
                }
            },
            2
        );

        return true;
    }

    public static function hasAllStagesWithUserAssigned($order, $returnException = false): bool
    {
        //todo validation for combinations 2021.06.10

        $order = $order->with([
                                  'works' => function ($q) {
                                      $q->with('stagesWithoutPivot')->with('combinations')->where('combination_id', 0);
                                  }
                              ])->withoutGlobalScope(OrderScope::class)->findOrFail($order->id);

        foreach ($order->works as $work) {
            foreach ($work->stagesWithoutPivot as $stage) {
                if (strlen($stage->user_name) && (int)$stage->user_id > 0) {
                    continue;
                }
                if ($returnException) {
                    throw new ValidationException(
                        __("Order stage with id: :orderStageId from order id: :orderId does not have a user assigned",
                           ['orderStageId' => $stage->id, "orderId" => $order->id])
                    );
                }
                return false;
            }
        }
        return true;
    }

    public static function updateStatus($orderId)
    {
        // get first ready stage
        $readyStage = OrderWorkStage::where('order_id', $orderId)->orderBy('order_work_template_id')->orderBy('rank')->where(
            'status',
            '<>',
            'finished'
        )->first();

        $status = 'expecting_completion';

        if ($readyStage !== null) {
            $status = 'waiting';
            if ($readyStage->status === 'working') {
                $status = 'working';
            }

            if ($readyStage->type === 'trial') {
                $status = 'waiting_for_trial';
                if ($readyStage->status === 'working') {
                    $status = 'in_trial';
                }
            }
        }

        return Order::where('id', '=', $orderId)->update(['status' => $status]);
    }

    public static function isSpecialStage($stageType): bool
    {
        return OrderWorkStagesService::isSpecialStage($stageType);
    }

    public static function possibleOrderActions(Order $order): array
    {
        return [
            'can_be_edited'             => self::orderCanBeUpdated($order, false),
            'can_be_deleted'            => self::orderCanBeDeleted($order, false),
            'can_be_canceled'           => self::orderCanBeCanceled($order, false),
            //todo can be remaked ATM is equivalent with can Be Canceled. and double the queries required !!!!!
            'can_be_remaked'            => self::orderCanBeRemaked($order, false),
            'can_be_send_in_production' => self::orderCanBeSentToProd($order, false),
            'can_be_finished'           => self::orderCanBeFinished($order, false),
            'can_see_calendar'          => true,
            'can_see_overview'          => true,
        ];
    }

    public static function orderCanBeDeleted(Order $order, $throwException = true): bool
    {
        if (!request()->user()->can(['orders.delete'])) {
            if ($throwException) {
                throw new AccessDeniedException(__('lang_permissions.orders.delete.missing.permission'));
            }
            return false;
        }

        if ($order->status !== 'creating') {
            if ($throwException) {
                throw new ValidationException(__('The order status does not permit deletion'));
            }
            return false;
        }

        return true;
    }

    public static function hasWorkStagesInProgress(Order $order, $throwException)
    {
        $hasWorkStageInProgress = true;
        $oswInProgress = OrderWorkStage::where('order_id', $order->id)->where('status', \DB::raw("'working'"))->first();
        if (empty($oswInProgress)) {
            $hasWorkStageInProgress = false;
        }
        if ($throwException && $hasWorkStageInProgress) {
            throw new ValidationException(__('validation.work_stage_in_progress'));
        }
        return $hasWorkStageInProgress;
    }

    public static function orderCanBeCanceled(Order $order, $throwException = true): bool
    {
        if (!OrderService::orderCanBeUpdated($order, $throwException)) {
            return false;
        }

        // check order status
        if ($order->status === 'creating') {
            if ($throwException) {
                throw new PossibleUnSyncException(__('order_in_configuration'));
            }
            return false;
        }

        // check order status
        if ($order->status === 'working') {
            if ($throwException) {
                throw new PossibleUnSyncException(__('validation.work_stage_in_progress'));
            }
            return false;
        }

        //todo refactor cand aflam exact ce vrea sa zica poetul
//        if (OrderService::isPayed($order->id)) {
//            if ($throwException) {
//                throw new ValidationException(__('Comanda e in plata'));
//            }
//            return false;
//        }

        //todo to be implemented
        if ($order->external_invoice) {
            if ($throwException) {
                throw new ValidationException(__('Comanda e in facturare'));
            }
            return false;
        }

        if (self::hasWorkStagesInProgress($order, $throwException)) {
            return false;
        }

        return true;
    }

    public static function isPayed($orderId)
    {
        $isPaid = false;
        $o = Order::where('id', $orderId)->with('payment_document')->first();
        if (isset($o->payment_document) && $o->payment_document->status == 'paid') {
            $isPaid = true;
        }
        return $isPaid;
    }

    public static function orderCanBeRemaked(Order $order, $throwException = true): bool
    {
        return self::orderCanBeCanceled($order, $throwException);
    }

    public static function orderCanBeSentToProd(Order $order, $throwException = true): bool
    {
        if (!self::orderCanBeUpdated($order, $throwException)) {
            return false;
        }

        if (!(request()->user()->can(['orders.send_in_production']))) {
            if ($throwException) {
                throw new AccessDeniedException(__('lang_permissions.orders.send_in_production.missing.permission'));
            }
            return false;
        }

        if ($order->status !== 'creating') {
            if ($throwException) {
                throw new PossibleUnSyncException(__('Order status does not permit sending to production (not creating)'));
            }
            return false;
        }

        $works = self::getOrderWorks($order);
        if (is_null($works) && $throwException) {
            throw new ValidationException(__('Orders without works can not be send to production'));
        }

        return true;
    }

    public static function orderCanBeFinished(Order $order, $throwException = true)
    {
        if (!(request()->user()->can(['orders.confirm_finish_order']))) {
            if ($throwException) {
                throw new AccessDeniedException(__('lang_permissions.orders.confirm_finish_order.missing.permission'));
            }
            return false;
        }

        if ($order->status !== 'expecting_completion') {
            if ($throwException) {
                throw new PossibleUnSyncException(
                    __('Order status is not compatible with finish operation: :orderStatus', [
                        'orderStatus' => $order->status
                    ])
                );
            }
            return false;
        }

        return true;
    }

    private static function calculationAdditionalPriceTab(Order $order)
    {
        $additionalPrices = $order->additionalPricesWithoutGlobalScope ?? [];
        $additionalPricesTab = [];
        $additionalPricesTab['total'] = 0;
        foreach ($additionalPrices as $additionalPrice) {
            if($additionalPrice->to_be_invoiced){
                $additionalPricesTab['total'] += $additionalPrice->price;
            }
            $additionalPricesTab['items'][] = [
                'id'                => $additionalPrice->id,
                'order_id'          => $additionalPrice->order_id,
                'name'              => $additionalPrice->name,
                'description'       => $additionalPrice->description,
                'total_price'       => $additionalPrice->price,
                'quantity'          => $additionalPrice->quantity,
                'pret_per_unitate'  => $additionalPrice->price_per_unit,
                'to_be_invoiced'    => (bool) $additionalPrice->to_be_invoiced,
            ];
        }
        $order->additional_prices_tab = $additionalPricesTab;
        return $order;
    }

    public static function orderFilesTab(Order $order)
    {
        //todo posibil refactoring la cum trimitem fisierele (resursa speciala?)
        $orderFiles = $order->files ?? [];
        $orderFilesTab = [];
        foreach ($orderFiles as $orderFile) {
            $orderFilesTab[] = $orderFile;
        }
        $order->order_files_tab = $orderFiles;
        return $order;
    }

    public static function getOrderIdAdditionalPricesListing(int $orderId)
    {
        return OrderAdditionalPrice::where('order_id', (int)$orderId)->get();
    }

    /**
     * @param $ordersIds
     *
     * @return array
     */
    public static function getExtraDetails($ordersIds): array
    {
        $checkboxStages = LabConfiguration::getCheckboxStages();
        $works = OrderWorkTemplate::whereIn('order_id', $ordersIds)->with([
                                                                              'stagesWithoutPivot' => function ($q) use ($checkboxStages) {
                                                                                  return $q->where('remake_id', 0)->where('price', '<>', 0)
                                                                                      ->where('multi_factor', '<>', 0)->where(
                                                                                          function ($q) use ($checkboxStages) {
                                                                                              $q->whereIn('type', $checkboxStages)->orWhere(
                                                                                                  'from_template',
                                                                                                  0
                                                                                              )->orWhere(function ($q) {
                                                                                                  $q->where('extra_cost_applied', 'true')
                                                                                                      ->orWhere(
                                                                                                          'extra_cost_applied',
                                                                                                          1
                                                                                                      );
                                                                                              });
                                                                                          });
                                                                              }
                                                                          ])->with('remakeStagesPrice')->with('order', function ($q) {
            return $q->with('dentist', function ($q) {
                return $q->select(['dentists.id', 'dentists.first_name', 'dentists.last_name']);
            })->with('collaborator', function ($q) {
                return $q->select(['collaborators.id', 'collaborators.name']);
            })->with('patient', function ($q) {
                return $q->select(['patients.id', 'patients.first_name', 'patients.last_name']);
            });
        })->get()->toArray();
        $res = [];
        foreach ($works as &$work) {
            $work['order']['works'][] = [
                'id'                 => $work['id'],
                'name'               => $work['original_name'],
                'elements'           => $work['elements'],
                'symbol'             => $work['symbol'],
                'price'              => $work['price'],
                'multi_factor'       => $work['multi_factor'],
                'stages'             => $work['stages_without_pivot'],
                'remakes_price'      => $work['remake_stages_price'],
                'ignore_extra_price' => $work['ignore_extra_price'],
                'estimated_duration' => $work['estimated_duration'],
                'preferred_duration' => $work['preferred_duration'],
                'extra_price'        => $work['extra_price']
            ];
            $res[$work['order_id']] = $work['order'];
        }

        $additionalPrices = OrderAdditionalPrice::whereIn('order_id', $ordersIds)->with('product')->get()->toArray();
        foreach ($additionalPrices as $additionalPrice) {
            $res[$additionalPrice['order_id']]['additionalPrices'][$additionalPrice['id']] = $additionalPrice;
        }


        return array_values($res);
    }

    /**
     * @param  Order  $order
     *
     * @return
     */
    public static function getOrderWorks(Order $order)
    {
        $seconds = 2;
        $identifier = tenancy()->tenant->id ?? __('no_use_only_so_you_can_generate_documentation_without_errors');
        $orderWorks = cache()->store('file')->remember('works'.($order->id).'_'.$identifier, $seconds, function () use ($order) {
            return $order->works()->first();
        });
        return $orderWorks;
    }

    /**
     * @param $ordersSql
     */
    public static function addOrderWorksSql($ordersSql)
    {
        $checkboxStages = LabConfiguration::getCheckboxStages();
        $ordersSql->with('works', function ($q) use ($checkboxStages) {
            $q->with([
                         'stagesWithoutPivot' => function ($q) use ($checkboxStages) {
                             return $q->where('remake_id', 0)->where('price', '<>', 0)->where('multi_factor', '<>', 0)->where(
                                 function ($q) use ($checkboxStages) {
                                     $q->whereIn('type', $checkboxStages)->orWhere(
                                         'from_template',
                                         0
                                     )->orWhere(function ($q) {
                                         $q->where('extra_cost_applied', 'true')->orWhere(
                                             'extra_cost_applied',
                                             1
                                         );
                                     });
                                 }
                             );
                         }
                     ])->with('remakeStagesPrice');
        });
    }

    public static function addPaymentsAndDiscountsToPaymentSummary(Collection $orders)
    {
        $orders->each(function ($item, $key) {
            OrderService::addPaymentsAndDiscountsToPaymentSummaryToOrder($item);
        });
        return $orders;
    }

    private static function addPaymentsAndDiscountsToPaymentSummaryToOrder(Order $order)
    {
        $order->loadMissing([
                                'order_payments',
                                'order_discounts.discount_type',
                                'payment_document.payments',
                                'payment_document.discounts.discount_type'
                            ]);
        $paymentSummary = $order->paymentSummary ?? [];
        $paymentSummary['payments'] = OrderService::getOrderPayments($order);
        $paymentSummary['discounts'] = OrderService::getOrderDiscounts($order);

        $order->paymentSummary = $paymentSummary;
        return $order;
    }

    private static function getOrderPayments(Order $order)
    {
        if ($order->payment_document !== null) {
            return $order->payment_document->payments->where('order_id', '!=', 0);
        }
        return $order -> order_payments;
    }

    private static function getOrderDiscounts(Order $order)
    {
        if ($order->payment_document !== null) {
            return $order->payment_document->discounts;
        }
        return $order -> order_discounts;
    }

//
//    public static function canAddOrderPayment(Order $order,$throwException = false)
//    {
//        $errorMessage = __('validation.cant_add_order_payment');
//
//        if (!request()->user()->can(['payment_documents.update'])) {
//            if ($throwException) {
//                throw new AccessDeniedException(__('lang_permissions.payment_documents.update.missing.permission'));
//            }
//            return true;
//        }
//
//        self::canAddOrderPaymentValue($order,$value);
//
//        if ($paymentDocument->to_pay < $value) {
//            $canAdd = false;
//            $errorMessage = __('Valoarea platii depaseste restul de plata');
//        }
//
//        if ($throwException && !$canAdd) {
//            throw new ValidationException($errorMessage);
//        }
//        return $canAdd;
//
//        return true;
//
//    }
//
//    private static function canAddOrderPaymentValue(Order $order)
//    {
//        $order ->loadMissing(['order_payments','order_discounts']);
//    }

}
