<?php


namespace App\Services;


use App\Configuration\LabConfiguration;
use App\Exceptions\Api\AccessDeniedException;
use App\Exceptions\Api\PossibleUnSyncException;
use App\Exceptions\Api\ServerError;
use App\Exceptions\Api\ValidationException;
use App\Http\Requests\CalendarOrderStagesIndexRequest;
use App\Models\Order;
use App\Models\OrderWorkStage;
use App\Models\OrderWorkTemplate;
use App\Models\UserWorks;
use Illuminate\Http\Request;
use App\Http\Resources\OrderWorkStageResource;

class OrderWorkStagesService
{


    /**
     * @param $orderId
     * @param  int  $minWorkId
     *
     * @return bool
     */
    public static function updateStagesDates($orderId, $minWorkId = 0)
    {
        $order = Order::findOrFail($orderId);
        $templates = $order->templates()->select(
            'order_work_templates.id as id',
            'order_work_templates.price as price',
            'order_work_templates.dentition as dentition',
            'order_work_templates.min_duration as min_duration',
            'order_work_templates.estimated_duration as estimated_duration',
            'order_work_templates.preferred_duration as preferred_duration',
            'order_work_templates.name as name',
            'work_template_id'
        )->where('order_work_templates.combination_id', 0)->orderBy('order_work_templates.created_at')->get()->toArray();
        $days = OrderService::calculateDays($order, $templates);

        $stages = OrderWorkStage::where('order_id', $order->id)->where('order_work_template_id', '>=', $minWorkId)->get();

        $lastStages = self::getLastStages($stages);

        foreach ($stages as $stage) {
            // check if it's last stage of the work
            if (!self::isTrialOrExternal($stage)) {
                continue;
            }

            if (in_array($stage->id, $lastStages)) {
                // check if it's last stage of the work
                $stage->date = $days['works'][$stage->order_work_template_id]['preferred_date'];
            } else {
                // check if it's last stage of the work
                if ($stage->day_number > 0) {
                    if (
                        $days['works'][$stage->order_work_template_id]['estimated_date']
                        != $days['works'][$stage->order_work_template_id]['preferred_date']
                    ) {
                        // reset all
                        $stage->date = null;
                    } else {
                        // calculate date
                        $startDate = $days['works'][$stage->order_work_template_id]['min_date'];
                        $date = OrderDatesCalculatorService::calculateDays($startDate, $stage->day_number);
                        $stage->date = $date;
                    }
                }
            }

            $stage->time = null;
            $stage->save();
        }

        return true;
    }

    /**
     * get last stages grouped by work id
     *
     * @param  array  $stages  stages
     *
     * @return array          list of ids
     */
    public static function getLastStages($stages)
    {
        $lastStages = [];

        foreach ($stages as $stage) {
            if (!isset($lastStages[$stage->order_work_template_id])) {
                $lastStages[$stage->order_work_template_id] = $stage;
            } elseif ($stage->rank > $lastStages[$stage->order_work_template_id]->rank) {
                $lastStages[$stage->order_work_template_id] = $stage;
            }
        }

        $res = [];

        foreach ($lastStages as $stage) {
            $res[] = $stage->id;
        }

        return $res;
    }

    public static function markReadyTime($orderId)
    {
        $stages = OrderWorkStage::where('order_id', $orderId)->orderBy('order_work_template_id')->orderBy('rank')->get();

        $readyFound = 0;
        foreach ($stages as $stage) {
            // skip finished stages
            if ($stage['status'] === 'finished') {
                continue;
            }

            if ($readyFound) {
                $stage->ready_time = null;
                $stage->save();
                continue;
            }

            $readyFound = $stage->id;
            // ready found
            if (is_null($stage->ready_time)) {
                $stage->ready_time = date('Y-m-d H:i:s');
                $stage->save();
            }
        }

        return true;
    }


    /**
     * calculate multi factor for regular stages
     *
     * @param $stageType
     * @param $stagePriceType
     * @param $dentition
     * @param  object  $arcadaDentition
     *
     * @return int
     */
    public static function calculateMultiFactor($stageType, $stagePriceType, $dentition, object $arcadaDentition)
    {
        $checkboxStages = LabConfiguration::getCheckboxStages();
        $multiFactor = 0;
        if (in_array($stageType, $checkboxStages)) {
            return $multiFactor;
        }

        switch ($stagePriceType) {
            case 'fix':
                $multiFactor = 1;
                break;
            case 'arcada':
                $elements = 0;
                if (isset($arcadaDentition->upper) && $arcadaDentition->upper) {
                    $elements++;
                }
                if (isset($arcadaDentition->lower) && $arcadaDentition->lower) {
                    $elements++;
                }

                $multiFactor = $elements;
                break;
            default:
                //dentition might come as an object
                $multiFactor = count((array)($dentition ?? []));
        }

        return $multiFactor;
    }

    public static function getDentConfig(OrderWorkTemplate $work, array $stage)
    {
        $checkboxStages = LabConfiguration::getCheckboxStages();
        $dentitionOptions = config('lab_configuration.dentition');
        $arcadaDentition = json_decode($work->arcada_dentition);

        if (in_array($stage['type'], $checkboxStages)) {
            return [
                'elements'         => 0,
                'multi_factor'     => 0,
                'dentition'        => '[]',
                'arcada_dentition' => json_encode($arcadaDentition)
            ];
        }

        $workDentition = json_decode($work->dentition);
        $stageMFDent = json_decode($stage['multi_factor_dentition']);
        $regular = [];

        if (!is_null($workDentition)) {
            foreach ($workDentition as $key => $dent) {
                if ($dent->form != 0) {
                    $form = $dent->form;

                    if (isset($stageMFDent->{$form}) && $stageMFDent->{$form}) {
                        $regular[$key] = true;

                        // arcada dentition
                        if (in_array($key, $dentitionOptions[0]['options'])) {
                            $arcadaDentition->upper = true;
                        }
                        if (in_array($key, $dentitionOptions[1]['options'])) {
                            $arcadaDentition->lower = true;
                        }
                    }
                }
            }
        }

        $multiFactor = count($regular);
        if ($stage['price_type'] == 'fix' && count($regular)) {
            $multiFactor = 1;
        }

        return [
            'elements'         => count($regular),
            'multi_factor'     => $multiFactor,
            'dentition'        => json_encode($regular),
            'arcada_dentition' => json_encode($arcadaDentition)
        ];
    }

    public static function getWorksByWorkId($id)
    {
        $works = OrderWorkTemplate::where('id', $id)->orWhere('combination_id', $id)->get()->toArray();

        $return = [];

        foreach ($works as $work) {
            $return[] = $work['id'];
        }

        return $return;
    }

    public static function getStageDentitionElements(OrderWorkStage $stage)
    {
        $elemente = new \stdClass();
        switch ($stage->price_type) {
            case 'fix':
                $elemente->fix = (bool)$stage->confirmed;
                break;
            case 'arcada':
                $elemente = $stage->arcada_dentition_array;
                break;
            case 'elemente':
                $elemente = $stage->dentition_array;
                break;
            default:
                throw new ServerError(__("Error #MPTOWS "));
                break;
        }
        return $elemente;
    }

    public static function isStageStarted(OrderWorkStage $orderStage, $completeVerification = false)
    {
        $isStarted = $orderStage->status === 'working';
        if ($completeVerification) {
            $logsCount = UserWorks::where('order_work_stage_id', $orderStage->id)->whereNull('stop_time')->get()->count();
            $isStarted = (bool)$logsCount;
        }
        return $isStarted;
    }

    public static function canStageBeFinished(OrderWorkStage $orderStage, $completeVerification = false, $throwException = false)
    {
        $canBeFinished = $orderStage->status === 'working' || $orderStage->status === 'paused';
        if ($completeVerification) {
            $logsCount = UserWorks::where('order_work_stage_id', $orderStage->id)->get()->count();
            $canBeFinished = (bool)$logsCount;
        }

        if ($throwException && !$canBeFinished) {
            throw new PossibleUnSyncException(__('Stage can not be finished'));
        }

        return $canBeFinished;
    }

    public static function canStageBeDeleted(OrderWorkStage $orderStage, $throwException = false)
    {
        $canBeDeleted = $orderStage->status !== 'finished';

        if ($throwException && !$canBeDeleted) {
            throw new PossibleUnSyncException(__('Stage is finished. Can not be deleted.'));
        }

        return $canBeDeleted;
    }

    public static function startStageTimer(OrderWorkStage $orderStage)
    {
        $log = new UserWorks();
        $log->order_work_stage_id = $orderStage->id;
        $log->start_time = \DB::raw('NOW()');
        $log->user_id = auth()->user()->id;
        $log->save();
        $log->refresh();
        return $log;
    }

    public static function stopStageTimer(OrderWorkStage $orderStage)
    {
        $log = UserWorks::where('order_work_stage_id', $orderStage->id)->whereNull('stop_time')->firstOrFail();
        $log->stop_time = \DB::raw('NOW()');
        $log->save();
        $log->refresh();
        return $log;
    }

    public static function endAllTimersForStage(OrderWorkStage $orderStage)
    {
        $logs = UserWorks::where('order_work_stage_id', $orderStage->id)->get();;
        foreach ($logs as $log) {
            if (is_null($log->stop_time)) {
                $log->stop_time = \DB::raw('NOW()');
                $log->save();
            }
        }

        return $logs;
    }

    public static function canStageUserBeModified(OrderWorkStage $orderStage, $throwException = false): bool
    {
        if (request()->user()->can(['orders_stages.update_stage_anytime'])) {
            return true;
        }

        if (OrderService::orderIdCanBeUpdated($orderStage->order_id, $throwException)) {
            return true;
        }

        if ($orderStage->status !== 'finished') {
            return true;
        }

        if ($throwException) {
            throw new PossibleUnSyncException(__('lang_permissions.orders_stages.update_stage_anytime.missing.permission'));
        }


        return false;
    }


    /**
     * @param  Request  $request
     * @param  OrderWorkStage  $orderStage
     * @param  false  $throwException
     *
     * @return bool
     * @throws PossibleUnSyncException
     */
    public static function canStageUserBeManuallyModified(Request $request, OrderWorkStage $orderStage, $throwException = false): bool
    {
        self::canStageUserBeModified($orderStage, $throwException);

        if ($request->user()->can(['orders_stages.update_user_manually'])) {
            return true;
        }

        if ($throwException) {
            throw new PossibleUnSyncException(__('lang_permissions.orders_stages.update_user_manually.missing.permission'));
        }


        return false;
    }

    public static function canStageChangeRank(OrderWorkStage $orderStage, $throwException = false)
    {
        $canChangeRank = $orderStage->status !== 'finished';

        if ($throwException && !$canChangeRank) {
            throw new PossibleUnSyncException(__('Stage can not change rank'));
        }

        return $canChangeRank;
    }

    public static function possibleStageActions(Request $request, OrderWorkStage $owStage)
    {
        return [
            'order_stage_id'           => $owStage->id,
            'can_be_deleted'           => self::canStageBeDeleted($owStage),
            'can_change_user'          => self::canStageUserBeModified($owStage),
            'can_change_user_manually' => self::canStageUserBeManuallyModified($request, $owStage),
            'can_change_rank'          => self::canStageChangeRank($owStage),
            'can_be_remaked'           => self::canStageBeRemaked($owStage),
            'etapa_in_lucru'           => self::isStageInWork($owStage)
        ];
    }

    public static function canStageBeRemaked(OrderWorkStage $stage, $throwException = false)
    {
        $seconds = 10;
        $identifier = tenancy()->tenant->id ?? __('no_use_only_so_you_can_generate_documentation_without_errors');
        //using cache for order
        $order = cache()->store('file')->remember('order_' . $stage->order_id . $identifier, $seconds, function () use ($stage) {
            return Order::find($stage->order_id);
        });


        if ($order === null) {
            if ($throwException) {
                throw new PossibleUnSyncException(__('Could not find order of stage'));
            }
            return false;
        }

        if (!OrderService::orderCanBeUpdated($order, $throwException)) {
            return false;
        }

        $orderStatusesWhereWeCantRemake = config('lab_configuration.default_order_statuses_where_we_cant_remake');
        if (OrderService::isOrderStatusIn($order, $orderStatusesWhereWeCantRemake)) {
            if ($throwException) {
                throw new PossibleUnSyncException(
                    __(
                        'Order status: :orderStatus does not permit remaking stages.',
                        [
                            'orderStatus' => $order->status
                        ]
                    )
                );
            }
            return false;
        }

        if (!self::isStageFinished($stage)) {
            if ($throwException) {
                throw new PossibleUnSyncException(__('Stage is not finished'));
            }
            return false;
        }

        if (self::isStageInWork($stage, $throwException)) {
            return false;
        }

        if (!self::isStageARemake($stage, $throwException)) {
            return false;
        }


        return true;
    }

    public static function isStageInWork(OrderWorkStage $stage, $throwException = false)
    {
        if ($stage->status == 'working') {
            if ($throwException) {
                throw new ValidationException(__('validation.stage_ongoing'));
            }
            return true;
        }
        return false;
    }

    public static function isStageARemake(OrderWorkStage $stage, $throwException = false)
    {
        if ($stage->status === 'finished') {
            if ($stage->remake_id) {
                if ($throwException) {
                    throw new ValidationException(__('Stage is a remake.'));
                }
                return false;
            }
        }

        return true;
    }

    public static function isStageFinished(OrderWorkStage $stage, $throwException = false)
    {
        if ($stage->status !== 'finished') {
            if ($throwException) {
                throw new ValidationException(__('Stage is not finished remake.'));
            }
        }

        return true;
    }

    public static function specialStageIsNotFinished(OrderWorkStage $stage, $throwException = false)
    {
        if ($stage->status === 'finished') {
            if ($throwException) {
                throw new PossibleUnSyncException(__("Order work stage is finished. Can not be modified"));
            }
            return true;
        }

        return false;
    }

    public static function isTrialOrExternal(OrderWorkStage $orderWorkStage)
    {
        return in_array($orderWorkStage->type, ['trial', 'external']);
    }

    public static function translateStageStatus($orderStagestatus, $throwException = false)
    {
        $stagePossibleStatuses = LabConfiguration::orderStageStatuses();
        foreach ($stagePossibleStatuses as $stagePossibleStatus) {
            if ($stagePossibleStatus['value'] === $orderStagestatus) {
                return $stagePossibleStatus['title'];
            }
        }
        if ($throwException) {
            throw new ValidationException(__('order stage status :status can not be translated', ['status' => $orderStagestatus]));
        }

        return $orderStagestatus;
    }

    public static function isSpecialStage($stageType)
    {
        $stageTypes = LabConfiguration::config('work_stages_types', 'value', 'assoc');

        return (isset($stageTypes[$stageType]['order_checkbox']) && $stageTypes[$stageType]['order_checkbox']);
    }

    public static function isStagePriceIncludedInTemplate(OrderWorkStage $stage, $throwException = false)
    {
        // include price if it doesn't bring additional cost & it's a template stage; if it's a remake, is without the collaborator's fault 
        if (!$stage->extra_cost_applied && $stage->from_template && $stage->remake_id === 0) {
            return true;
        }

        if ($stage->collaborator_fault_id === 0 && $stage->remake_id !== 0) {
            return true;
        }

        if ($throwException) {
            throw new PossibleUnSyncException(__("Stage price is not included in template"));
        }

        return false;
    }

    /**
     * @param  array  $validated
     * @param  OrderWorkStage  $orderWorkStage
     *
     * @return OrderWorkStage
     * @throws \Throwable
     */
    static function updateDateAndTime(array $validated, OrderWorkStage $orderWorkStage): OrderWorkStage
    {
        $orderWorkStage->date = date('Y-m-d', strtotime($validated['date']));
        $time = is_null($validated['time']) ? null : date('H:i', strtotime($validated['time']));
        $orderWorkStage->time = $time;
        $orderWorkStage->saveOrFail();
        $orderWorkStage->refresh();
        return $orderWorkStage;
    }

    /**
     * @param  CalendarOrderStagesIndexRequest  $request
     * @param  Order  $order
     *
     * @return \Illuminate\Contracts\Routing\ResponseFactory|\Illuminate\Http\Response
     */
    public static function showFinishedOrToSentOWS(CalendarOrderStagesIndexRequest $request, Order $order)
    {
        if (!$request->user()->can(['orders.read'])) {
            throw new AccessDeniedException(__('lang_permissions.orders.read.missing.permission'));
        }
        $validated = $request->validated();
        $orderStages = OrderWorkStage::where('order_id', $order->id)
            ->where(function ($query) use ($validated) {
                $query->where(function ($query) use ($validated) {
                    $query->where('status', 'finished')->where('finished_time', '>=', $validated['start_date'])->where('finished_time', '<=', $validated['end_date']);
                });
                $query->orWhere(function ($query) use ($validated) {
                    $query->where('to_send', 1)->where('date', '>=', $validated['start_date'])->where('date', '<=', $validated['end_date']);
                });
            });
        $orderStages = $orderStages->get();
        return response(["data" => OrderWorkStageResource::collection($orderStages)], 200);
    }


    /**
     * @param $orderId
     * @param $orderFinishDate
     * @param $orderStartDate
     * @param $orderWorktemplateId
     * @throws ValidationException
     */
    public static function checkIfExceedDateRange($orderId, $orderFinishDate, $orderStartDate, $orderWorktemplateId): void
    {
        $workStages = OrderWorkStage::select('date')->where([
            ['order_id', '=', $orderId],
            ['to_send', '=', 1],
            ['order_work_template_id', $orderWorktemplateId],
            ['date', '>', $orderFinishDate]
        ])->get();
        if (count($workStages)) {
            throw new ValidationException(__('validation.exceed_date_range') . ' (' . date('Y-m-d', strtotime($orderStartDate)) . ' - ' . $orderFinishDate . ')');
        }
    }
}
