<?php


namespace App\Services;

use App\Exceptions\Api\AccessDeniedException;
use App\Exceptions\Api\ValidationException;
use App\Models\Invoice;
use App\Models\LabInvoiceProfile;
use App\Models\Order;
use App\Models\PaymentDocument;
use App\Models\Discount;
use App\Models\Receipt;
use App\Models\Payment;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Collection;

class PaymentDocumentService
{

    /**
     * @param  int  $collaboratorId
     * @param  int  $dentistId
     *
     * @return mixed
     */
    public static function sqlAvailableOrdersForCollaborator($collaboratorId, $dentistId, $filters = [])
    {
        $sql = Order::select('orders.*')->where('orders.collaborator_id', $collaboratorId)->where(
            'orders.payment_document_id',
            0
        )->where(function ($query) {
            $query->where('orders.appears_on_payment_document', 1)->orWhere('orders.cancelled', 0);
        });
        $sql->with(['dentist', 'collaborator', 'patient', 'additionalPrices'])->joinPatients()->joinCollaborators()->joinDentists()->where(
            'orders.status',
            '<>',
            'creating'
        );

        if ($dentistId > 0) {
            $sql->where('orders.dentist_id', $dentistId);
        }

        foreach ($filters as $filter => $filterValue) {
            switch ($filter) {
                case 'date_filtering':
                    if (isset($filterValue['start_date'])) {
                        $sql->where('orders.start_date', '>=', date('Y-m-d', strtotime($filterValue['start_date']['interval_start'])))
                            ->where(
                                'orders.start_date',
                                '<=',
                                date('Y-m-d', strtotime($filterValue['start_date']['interval_end']))
                            );
                    }
                    if (isset($filterValue['end_date'])) {
                        $sql->where('orders.finish_date', '>=', date('Y-m-d', strtotime($filterValue['end_date']['interval_start'])))
                            ->where(
                                'orders.finish_date',
                                '<=',
                                date('Y-m-d', strtotime($filterValue['end_date']['interval_end']))
                            );
                    }
                    break;
                case 'dentist_filtering':
                    $sql->where(
                        \DB::raw("CONCAT(dentists.first_name, ' ', dentists.last_name)"),
                        'like',
                        '%' . $filterValue['dentist_name'] . '%'
                    );
                    break;
                case 'patient_filtering':
                    $sql->where(
                        \DB::raw("CONCAT(patients.first_name, ' ', patients.last_name)"),
                        'like',
                        '%' . $filterValue['patient_name'] . '%'
                    );
                    break;
                default:
                    //in the future we might need other filters
                    break;
            }
        }

        return $sql;
    }

    public static function canBeDeleted(PaymentDocument $paymentDocument, $throwException = false)
    {
        $canBeDeleted = false;
        $errorMessage = __('validation.payment_document_cant_be_deleted');

        $worksCount = Order::select('orders.*')->where('orders.payment_document_id', $paymentDocument->id)->count();

        if ($worksCount > 0) {
            $canBeDeleted = false;
            $errorMessage = __('validation.payment_document_orders_attached');
        }

        if ($throwException && !$canBeDeleted) {
            throw new ValidationException($errorMessage);
        }

        return $canBeDeleted;
    }

    public static function canBeUpdated(PaymentDocument $paymentDocument, $throwException = false)
    {
        if (!request()->user()->can(['payment_documents.update'])) {
            throw new AccessDeniedException(__('lang_permissions.payment_documents.update.missing.permission'));
        }

        $canBeUpdated = true;
        $errorMessage = __('validation.payment_document_cant_be_updated');

        if ($paymentDocument->status !== 'config') {
            $canBeUpdated = false;
            $errorMessage = __('validation.payment_document_not_on_configuration');
        }

        if ($paymentDocument->status === 'locked') {
            $canBeUpdated = false;
            $errorMessage = __('validation.payment_document_locked');
        }

        if ($throwException && !$canBeUpdated) {
            throw new ValidationException($errorMessage);
        }

        return $canBeUpdated;
    }

    public static function canAddPayment(PaymentDocument $paymentDocument, $value, $throwException = false)
    {
        if (!request()->user()->can(['payment_documents.update'])) {
            throw new AccessDeniedException(__('lang_permissions.payment_documents.update.missing.permission'));
        }

        $canAdd = true;
        $errorMessage = __('validation.cant_add_payment');

        if ($paymentDocument->to_pay < $value) {
            $canAdd = false;
            $errorMessage = __('Valoarea platii depaseste restul de plata');
        }

        if ($throwException && !$canAdd) {
            throw new ValidationException($errorMessage);
        }
        return $canAdd;
    }

    public static function canRemovePayment($paymentId, $throwException = false)
    {
        if (!request()->user()->can(['payment_documents.update'])) {
            throw new AccessDeniedException(__('lang_permissions.payment_documents.update.missing.permission'));
        }

        $canRemove = true;
        $errorMessage = __('validation.cant_remove_payment');
        $receipts = Receipt::where('payment_id', $paymentId)->get();

        if (count($receipts) > 0) {
            $canRemove = false;
            $errorMessage = __('A fost deja emisa chitanta');
        }

        if ($throwException && !$canRemove) {
            throw new ValidationException($errorMessage);
        }
        return $canRemove;
    }

    /**
     * Delete a payment from the payment document.
     *
     * @param integer $paymentId
     * @param PaymentDocument $paymentDocument
     * @return integer
     */
    public static function removePaymentFromPD(int $paymentId, PaymentDocument $paymentDocument): int
    {
        return Payment::where('id', $paymentId)
            ->where('payment_document_id', $paymentDocument->id)
            ->delete();
    }

    public static function canGenerateInvoice(PaymentDocument $paymentDocument, $invoiceDate, $seriesId, $services, $throwException = false)
    {
        $canGenerateInvoice = true;
        $errorMessage = __('validation.invoice_cant_be_generated');

        if ($paymentDocument->status == 'config') {
            $canGenerateInvoice = false;
            $errorMessage = __('validation.payment_document_on_configuration');
        }

        if ($paymentDocument->status !== 'locked') {
            $canGenerateInvoice = false;
            $errorMessage = __('validation.payment_document_not_locked');
        }

        if ($paymentDocument->invoice_id > 0) {
            $canGenerateInvoice = false;
            $errorMessage = __('validation.payment_document_has_invoice');
        }

        if ($paymentDocument->lab_invoice_profile_id == 0) {
            $canGenerateInvoice = false;
            $errorMessage = __('validation.payment_document_no_lab_profile');
        }

        if (count($services) < 1) {
            $canGenerateInvoice = false;
            $errorMessage = __('validation.no_services_attached');
        }

        if (!InvoiceService::verifyNoInvoiceIssuedAfterSpecifiedDate($invoiceDate, $seriesId)) {
            $canGenerateInvoice = false;
            $errorMessage = __('validation.invoice_with_same_series_and_greater_date');
        };

        if ($throwException && !$canGenerateInvoice) {
            throw new ValidationException($errorMessage);
        }

        return $canGenerateInvoice;
    }

    /**
     * @param  PaymentDocument  $paymentDocument
     * @param $paymentId
     * @param $value
     * @param $receiptDate
     * @param $seriesId
     * @param  bool  $throwException
     *
     * @return bool
     * @throws AccessDeniedException
     * @throws ValidationException
     */
    public static function canGenerateReceipt(
        PaymentDocument $paymentDocument,
        $paymentId,
        $receiptDate,
        $seriesId,
        $throwException = false
    ) {
        $canGenerateReceipt = true;
        $errorMessage = __('validation.receipt_cant_be_generated');
        if (!request()->user()->can(['payment_documents.update'])) {
            throw new AccessDeniedException(__('lang_permissions.payment_documents.update.missing.permission'));
        }

        if ($paymentDocument->invoice_id == 0) {
            $canGenerateReceipt = false;
            $errorMessage = __('Documentul de plata nu are factura');
        }
        $invoice = Invoice::findOrFail($paymentDocument->invoice_id);
        if ($invoice->status == 'cancelled') {
            $canGenerateReceipt = false;
            $errorMessage = __('Factura anulata');
        }

        if (!ReceiptService::verifyValidValue($paymentDocument->invoice_id, $paymentId)) {
            $canGenerateReceipt = false;
            $errorMessage = __('Valoarea tuturor chitantelor depaseste valoarea facturii');
        }

        $payment = Payment::findOrFail($paymentId);
        if ($payment->type != 'cash') {
            $canGenerateReceipt = false;
            $errorMessage = __('Plata nu a fost facuta cash');
        }

        if (!is_null($payment->receipt_series)) {
            $canGenerateReceipt = false;
            $errorMessage = __('Plata are deja o chitanta generata');
        }

        if (!ReceiptService::verifyNoreceiptIssuedAfterSpecifiedDate($receiptDate, $seriesId)) {
            $canGenerateReceipt = false;
            $errorMessage = __('validation.receipt_with_same_series_and_greater_date');
        };

        if ($throwException && !$canGenerateReceipt) {
            throw new ValidationException($errorMessage);
        }
        return $canGenerateReceipt;
    }

    /**
     * @param  PaymentDocument  $paymentDocument
     *
     * @throws ValidationException
     */
    public static function canBeMarkedAsPaid(PaymentDocument $paymentDocument, $throwException = false): bool
    {
        $canBeMarked = self::canBeUpdated($paymentDocument, $throwException);
        $message = __('validation.undefined_error');

        if ($paymentDocument->to_pay === null || $paymentDocument->to_pay !== 0) {
            $canBeMarked = false;
            $message = __('validation.payment_document_to_pay');
        }

        if ($throwException && $canBeMarked) {
            throw new ValidationException($message, 409);
        }
        return $canBeMarked;
    }

    public static function canBeLocked(PaymentDocument $paymentDocument, $throwException = false): bool
    {
        return self::canBeUpdated($paymentDocument, $throwException);
    }

    public static function calculatePaymentDocumentValue($paymentDocumentId)
    {
        $sqlWorks = Order::select(
            'orders.*',
            'order_work_templates.name as work',
            'order_work_templates.id as work_id'
        )->where('orders.payment_document_id', $paymentDocumentId)
            // ->scopeJoinPatients()
            // ->scopeJoinContributors()
            // ->scopeJoinDentists()
            ->join('order_work_templates', function ($join) {
                $join->on('orders.id', '=', 'order_work_templates.order_id')->on('order_work_templates.combination_id', '=', \DB::raw('0'));
            })->get()->toArray();

        $orders = [];
        foreach ($sqlWorks as $work) {
            if (!isset($orders[$work['id']])) {
                $orders[$work['id']] = $work;
                $orders[$work['id']]['works'] = [];
            }
            if (!empty($work['work'])) {
                $orders[$work['id']]['works'][] = [
                    'name' => $work['work'],
                    'id'   => $work['work_id']
                ];
            }
        }

        // to array
        $ordersArr = [];
        $total = 0;
        foreach ($orders as $order) {
            $ordersArr[] = $order;
            $total += $order['value'];
        }

        // discounts
        $discounts = Discount::where('payment_document_id', $paymentDocumentId)->get()->toArray();

        foreach ($discounts as $discount) {
            $total -= $discount['value'];
        }
        return $total;
    }

    public static function generateProformaNumberBySeries(LabInvoiceProfile $labInvoiceProfile)
    {
        $startNo = $labInvoiceProfile->proforma_start_no;
        $proformaNr = PaymentDocument::select('proforma_no as proforma_max_no')->where(
            'proforma_series',
            $labInvoiceProfile->proforma_series
        )->where('lab_invoice_profile_id', $labInvoiceProfile->id)->orderBy('proforma_no', 'DESC')->first();

        if (is_null($proformaNr)) {
            $no = $startNo;
        } else {
            $no = $proformaNr->proforma_max_no + 1;
        }
        return $no;
    }

    /**
     * Check if user can add payment document.
     *
     * @param  integer  $collaboratorId
     * @param  boolean  $throwException
     *
     * @return boolean
     */
    public static function canAddPaymentDocument(int $collaboratorId, $throwException = false)
    {
        if (!request()->user()->can(['payment_documents.create'])) {
            throw new AccessDeniedException(__('lang_permissions.payment_documents.create.missing.permission'));
        }

        $canAddPaymentDocument = true;
        $errorMessage = __('validation.payment_document_cant_be_added');
        $openPaymentDocuments = PaymentDocument::where('collaborator_id', $collaboratorId)->where('status', '!=', 'locked')->get()->toArray();

        if (count($openPaymentDocuments) > 0) {
            $canAddPaymentDocument = false;
            $errorMessage = __('validation.collaborator_has_an_open_pd');
        }

        if ($throwException && !$canAddPaymentDocument) {
            throw new ValidationException($errorMessage);
        }

        return $canAddPaymentDocument;
    }

    /**
     * @param  PaymentDocument  $paymentDocument
     * @param  array  $validated
     *
     * @throws AccessDeniedException
     * @throws ValidationException
     */
    public static function addPayment(PaymentDocument $paymentDocument, array $paymentParameters)
    {
        PaymentService::canAddPayment($paymentDocument, $paymentParameters['value'], true);
        $paymentParameters['payment_document_id'] = $paymentDocument->id;
        Payment::create($paymentParameters);
        return $paymentDocument;
    }

    public static function addDiscount(PaymentDocument $paymentDocument, array $paymentParameters)
    {
        DiscountService::canAddDiscount($paymentDocument, $paymentParameters['value'], true);
        $paymentParameters['payment_document_id'] = $paymentDocument->id;
        Discount::create($paymentParameters);
        return $paymentDocument;
    }

    /**
     * Delete payment.
     *
     * @param PaymentDocument $paymentDocument
     * @param integer $paymentId
     * @return PaymentDocument
     */
    public static function deletePayment(PaymentDocument $paymentDocument, int $paymentId): PaymentDocument
    {
        self::canRemovePayment($paymentDocument, true);
        $payment = $paymentDocument->payments()->where('payments.id', $paymentId)->firstOrFail();
        self::removePaymentFromPD($payment->id, $paymentDocument);
        return $paymentDocument;
    }

    public static function deleteDiscount(PaymentDocument $paymentDocument, int $discountId)
    {
        DiscountService::canRemoveDiscount($paymentDocument, true);
        $discount = $paymentDocument->discounts()->where('discounts.id', $discountId)->firstOrFail();
        DiscountService::deleteDiscount($discount->id);
        return $paymentDocument;
    }

    /**
     * Get the unlocked payment document.
     *
     * @param Order $order
     * @return object|null
     */
    public static function getTheUnlockedPaymentDocument(Order $order)
    {
        return PaymentDocument::where('collaborator_id', $order->collaborator_id)
            ->where('status', '<>', 'locked')
            ->first();
    }

    /**
     * Create the payment document.
     *
     * @param array $validated
     * @return PaymentDocument
     */
    public static function createThePaymentDocument(array $validated)
    {
        $validated['dentist_id'] = $validated['dentist_id'] ?? 0;

        $labInvoiceProfile = LabInvoiceProfile::findOrFail($validated['lab_invoice_profile_id']);
        $proformaNo = PaymentDocumentService::generateProformaNumberBySeries($labInvoiceProfile);
        $validated['proforma_no'] = $proformaNo;
        $validated['proforma_series'] = $labInvoiceProfile->proforma_series;

        $paymentDocument = PaymentDocument::create($validated);
        $paymentDocument->refresh();

        return $paymentDocument;
    }

    /**
     * Distribute payment to orders. 
     *
     * @param Collection $orders
     * @param array $parameters
     * @param PaymentDocument $paymentDocument
     * @return array
     */
    public static function distributePaymentToOrders(Collection $orders, array $parameters, PaymentDocument $paymentDocument)
    {
        foreach ($orders as $order) {
            $orderPaymentsValue = $order->payments->sum('value');
            $orderValue = $order->value;

            while ($orderPaymentsValue < $orderValue && $parameters['value'] > 0) {
                $paymentsTotal = $orderPaymentsValue + $parameters['value'];
                $orderRest = $orderValue - $orderPaymentsValue;

                $paymentsArray[] = [
                    'payment_document_id'   => $paymentDocument->id,
                    'order_id'              => $order->id,
                    'name'                  => $parameters['name'],
                    'value'                 => $paymentsTotal > $orderValue ? $orderRest : $parameters['value'],
                    'date'                  => $parameters['date'],
                    'type'                  => $parameters['type'],
                    'created_at'            => Carbon::now()->toDateTimeString(),
                    'updated_at'            => Carbon::now()->toDateTimeString(),
                ];

                $parameters['value'] -= $orderRest;
                break;
            }
        }

        return $paymentsArray ?? [];
    }

    /**
     * Distribute the payments.
     *
     * @param Collection $payments
     * @param Collection $orders
     * @param PaymentDocument $paymentDocument
     * @return array
     */
    public static function distributePayments(Collection $payments, Collection $orders, PaymentDocument $paymentDocument)
    {
        foreach ($payments as $payment) {
            $remainingToPay = $payment->value;

            foreach ($orders as $order) {
                $paymentValue = $remainingToPay < $order->value ? $remainingToPay : $order->value;

                if ($remainingToPay > 0 && $paymentValue > 0) {
                    $paymentsArray[] = [
                        'payment_document_id'   => $paymentDocument->id,
                        'order_id'              => $order->id,
                        'name'                  => "Plata distribuita automat. Descriere: {$payment->name}, valoare: {$payment->value}, data: {$payment->date}",
                        'value'                 => $paymentValue,
                        'date'                  => $payment->date,
                        'type'                  => $payment->type,
                        'created_at'            => Carbon::now()->toDateTimeString(),
                        'updated_at'            => Carbon::now()->toDateTimeString(),
                    ];

                    $remainingToPay -= $paymentValue;
                    $order->value -= $paymentValue;
                }
            }
        }

        return $paymentsArray ?? [];
    }
}
