<?php


namespace App\Services\Reports;

use App\Models\OrderWorkStage;
use App\Models\User;
use App\Models\WorkStage;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Facades\DB;
use stdClass;

class CompletedStagesChartService
{
    /**
     * Calculate total value per month.
     *
     * @param User $user
     * @param WorkStage $workStage
     * @param integer $numberOfMonths
     * @return Builder
     */
    public static function calculateTotalValuePerMonth(User $user, WorkStage $workStage, int $numberOfMonths): Builder
    {
        $startDate = Carbon::now()->subMonths($numberOfMonths);

        $orderWorkStages = OrderWorkStage::select(
            DB::raw('DATE_FORMAT(finished_time, "%Y-%m") as month'),
            DB::raw('ROUND(SUM(cost * multi_factor), 2) as value'),
        )
            ->whereNotNull('finished_time')
            ->where('finished_time', '>=', $startDate->format('Y-m'))
            ->where('user_id', $user->id)
            ->groupBy('month');

        if ($workStage->id !== null) {
            $orderWorkStages->where('work_stage_id', DB::raw("'" . (int)$workStage->id . "'"));
        }

        return $orderWorkStages;
    }

    /**
     * Generate interval dates with default values if order work stages dates are not provided.
     *
     * @param Builder $orderWorkStages
     * @param integer $numberOfMonths
     * @return array
     */
    public static function generateIntervalDatesWithDefaultValues(Builder $orderWorkStages, int $numberOfMonths): array
    {
        $dates = [];

        $orderWorkStages = $orderWorkStages->get()->mapWithKeys(function ($item) {
            return [$item['month'] => $item['value']];
        });

        for ($i = $numberOfMonths - 1; $i >= 0; $i--) {
            $date = Carbon::now()->subMonthNoOverflow($i);
            $dateOfInterval = new stdClass();
            $dateOfInterval->month = $date->format('Y-m');

            isset($orderWorkStages[$date->format('Y-m')])
                ? $dateOfInterval->value = $orderWorkStages[$date->format('Y-m')]
                : $dateOfInterval->value = "0";

            $dates[] = $dateOfInterval;
            unset($dateOfInterval);
        }

        return $dates;
    }

    /**
     * Calculate the user's order work stages total.
     *
     * @param User $user
     * @param integer $numberOfMonths
     * @return object
     */
    public static function calculateUserTotalPerOrderWorkStage(User $user, int $numberOfMonths): object
    {
        $startDate = Carbon::now()->subMonths($numberOfMonths);

        return OrderWorkStage::select(
            'work_stage_id as id',
            'name',
            DB::raw('ROUND(SUM(cost * multi_factor), 2) as value'),
        )
            ->whereNotNull('finished_time')
            ->where('finished_time', '>=', $startDate->format('Y-m'))
            ->where('user_id', $user->id)
            ->whereRaw('( cost * multi_factor ) > 0')
            ->groupBy('work_stage_id')
            ->orderBy('value', 'DESC')
            ->get();
    }
}
