<?php

namespace App\Services\Reports;

use App\Exceptions\Api\PossibleUnSyncException;
use App\Exceptions\Api\ServerError;
use App\Models\Order;
use App\Services\OrderService;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;

class OrdersService
{

    /**
     * @param  array  $filters
     *
     * @return mixed
     * @throws ServerError
     */
    public static function getOrdersNotCanceled(array $filters = [])
    {
        $orders = self::getOrdersFiltered($filters);
        $orders = self::addPaymentSummaryToOrders($orders);
        $orders = OrderService::addPaymentsAndDiscountsToPaymentSummary($orders);
        $orders = self::addPaymentFilters($orders, $filters);
        return $orders;
    }

    public static function getTotals($ids): array
    {
        $totals = [];

        foreach ($ids as $id) {
            $totals[$id] = OrderService::calculateTotalPrice($id);
        }
        return $totals;
    }

    /**
     * @param  array  $filters
     * @param  Builder  $orders
     */
    private static function addFilters(array $filters, Builder $orders): void
    {
        if (isset($filters['collaborator_id'])) {
            $orders->where('collaborator_id', (int)$filters['collaborator_id']);
        }

        if (isset($filters['dentist_id']) && $filters['dentist_id'] !== 0) {
            $orders->where('dentist_id', (int)$filters['dentist_id']);
        }

        if (isset($filters['order_id'])) {
            $orders->where('id', (int)$filters['order_id']);
        }

        if (isset($filters['status']['is_complete'])) {
            if ($filters['status']['is_complete'] === true) {
                $orders->where('status', 'complete');
            } else {
                $orders->where('status', '<>', 'complete');
            }
        }

        if (isset($filters['invoice'])) {
            if (isset($filters['invoice']['only_fiscal']) && $filters['invoice']['only_fiscal'] === true) {
                $orders->whereHas('payment_document.invoice');
            }
            if (isset($filters['invoice']['only_proforma']) && $filters['invoice']['only_proforma'] === true) {
                $orders->whereHas('payment_document')->whereDoesntHave('payment_document.invoice');
            }
            if (isset($filters['invoice']['only_no_invoice']) && $filters['invoice']['only_no_invoice'] === true) {
                $orders->whereDoesntHave('payment_document');
            }
        }

        if (isset($filters['zero_value_orders'])) {
            //todo might not be the final version (check if requirements are ok for this)
            if ($filters['zero_value_orders'] === true) {
                $orders->where('orders.value', 0);
            }
            if ($filters['zero_value_orders'] === false) {
                $orders->where('orders.value', '!=', 0);
            }
        }


        if (isset($filters['date_filtering']['start_date']['interval_start'])
            && $filters['date_filtering']['start_date']['interval_start'] !== ''
        ) {
            $intervalStartDate = new Carbon($filters['date_filtering']['start_date']['interval_start']);
            $orders->where('start_date', '>=', $intervalStartDate);
        }

        if (isset($filters['date_filtering']['start_date']['interval_end'])
            && $filters['date_filtering']['start_date']['interval_end']
        ) {
            $intervalEndDate = new Carbon($filters['date_filtering']['start_date']['interval_end']);
            $intervalEndDate->addDay();
            $orders->where('start_date', '<', $intervalEndDate);
        }

        if (isset($filters['date_filtering']['end_date']['interval_start'])
            && $filters['date_filtering']['end_date']['interval_start'] !== ''
        ) {
            $intervalStartDate = new Carbon($filters['date_filtering']['end_date']['interval_start']);
            $orders->where('finish_date', '>=', $intervalStartDate);
        }

        if (isset($filters['date_filtering']['end_date']['interval_end']) && $filters['date_filtering']['end_date']['interval_end']) {
            $intervalEndDate = new Carbon($filters['date_filtering']['end_date']['interval_end']);
            $intervalEndDate->addDay();
            $orders->where('finish_date', '<', $intervalEndDate);
        }
    }

    /**
     * @param  array  $filters
     *
     * @return mixed
     */
    private static function getOrdersFiltered(array $filters)
    {
        $orders = Order::where('status', '<>', 'creating')
                    ->with([
                                  'order_payments',
                                  'order_discounts.discount_type',
                                  'dentist',
                                  'patient',
                                  'collaborator.region',
                                  'payment_document.dentist',
                                  'payment_document.collaborator.region',
                                  'payment_document.payments',
                                  'payment_document.discounts.discount_type',
                                  'payment_document.invoice.paymentDocument'
                              ]);
        OrderService::addOrderWorksSql($orders);

        self::addFilters($filters, $orders);

        $orders = $orders->orderBy('date_added', 'DESC');
        $orders = $orders->get();

        return $orders;
    }

    /**
     * @param $orders
     *
     * @return mixed
     */
    public static function addPaymentSummaryToOrders($orders)
    {
        $orders->each(function ($order) {
            return self::addPaymentSummaryToOrder($order);
        });

        return $orders;
    }

    private static function addPaymentFilters($orders, array $filters)
    {
        //removing orders based on filtering
        $orders->each(function ($item, $key) use ($orders, $filters) {
            if (isset($filters['is_paid_fully'])) {
                //we need this field to filter the orders
                if (!isset($item['paymentSummary'])) {
                    throw new ServerError(__('validation.exceptions.reports.missing_payment_summary'));
                }

                if (($filters['is_paid_fully'] === true) && $item['paymentSummary']['to_pay'] > 0) {
                    unset($orders[$key]);
                }
                if (($filters['is_paid_fully'] === false) && $item['paymentSummary']['to_pay'] <= 0) {
                    unset($orders[$key]);
                }
            }
            return $item;
        });

        return $orders;
    }

    /**
     * @param  Order  $order
     * @param  bool  $formatPaymentSummary
     *
     * @return Order
     */
    public static function addPaymentSummaryToOrder(Order $order, bool $formatPaymentSummary = true): Order
    {
        $paymentSummary['payed'] = 0;
        $paymentSummary['discounts_value'] = 0;
        $payments = $order->payment_document !== null ? $order->payment_document->payments : $order->order_payments;
        $discounts = $order->payment_document !== null ? $order->payment_document->discounts : $order->order_discounts;

        foreach ($payments as $payment) {
            $paymentSummary['payed'] += $payment['value'];
        }

        foreach ($discounts as $discount) {
            $paymentSummary['payed'] += $discount['value'];
            $paymentSummary['discounts_value'] += $discount['value'];
        }
        $hasOrderValueCalculated = $order->status === 'complete' || $order->payment_document !== null;
        if ($hasOrderValueCalculated) {
            $paymentSummary['to_pay'] = round(($order->value - $paymentSummary['payed']), 2);
            $paymentSummary['to_pay'] = $formatPaymentSummary ? number_format($paymentSummary['to_pay'], 2) : $paymentSummary['to_pay'];
        } else {
            $paymentSummary['to_pay'] = null;
        }

        $order['paymentSummary'] = [
            'payed'                   => $formatPaymentSummary ? number_format($paymentSummary['payed'], 2) : $paymentSummary['payed'],
            'discounts_value'         => $formatPaymentSummary ? number_format($paymentSummary['discounts_value'], 2)
                : $paymentSummary['discounts_value'],
            'to_pay'                  => $paymentSummary['to_pay'],
            'payments'                => null,
            'discounts'               => null,
            'send_to_invoicing'       => $order->payment_document ? false : true,
            'has_invoice'             => $order->payment_document && $order->payment_document->invoice ? true : false,
            'has_proforma'            => $order->payment_document ? true : false,
            'has_no_invoice'          => $order->payment_document ? false : true,
            'needs_value_calculation' => !$hasOrderValueCalculated,
        ];
        return $order;
    }

    /**
     * Check if order can be sent to invoicing.
     *
     * @param  Order  $order
     *
     * @return boolean
     * @throws PossibleUnSyncException
     */
    public static function canBeSentToInvoicing(Order $order): bool
    {
        if ($order->payment_document) {
            throw new PossibleUnSyncException(__('Comanda este deja atribuita unui document de plata.'));
        }

        return true;
    }

}
