<?php


namespace App\Services;


use App\Exceptions\Api\AccessDeniedException;
use App\Models\OrderWorkStage;
use App\Models\User;
use App\Models\WorkStageUser;
use App\Models\WorkTemplateStageUser;
use Illuminate\Http\Request;

class UserService
{
    public static function userCanStartStage(User $user, OrderWorkStage $stage, $throwException = false)
    {
        if ($user->id === $stage->user_id) {
            return true;
        }

        if ($user->can(['orders_stages.other.trial_stages_dashboard']) && OrderWorkStagesService::isTrialOrExternal($stage)) {
            return true;
        }

        if ($user->can(['orders_stages.start_others_stages'])) {
            return true;
        }

        if ($throwException) {
            throw new AccessDeniedException(
                __(
                    'User can not start/pause/finish order stage timer (missing permissions or not assigned to stage or stage is not trial or external)'
                )
            );
        }


        return false;
    }

    public static function userCanReadOthersStages(User $user, $throwException = true)
    {
        if ($user->can(['orders_stages.others_stages.read'])) {
            return true;
        }

        if ($throwException) {
            throw new AccessDeniedException(__('lang_permissions.orders_stages.others_stages.read.missing.permission'));
        }

        return false;
    }

    public static function userCanReadOrderDashboardOrdersNewAndFinished(User $user, $throwException = true)
    {
        if ($user->can(['production.orders.new_and_finish'])) {
            return true;
        }

        if ($throwException) {
            throw new AccessDeniedException(__('lang_permissions.production.orders.new_and_finish.missing.permission'));
        }

        return false;
    }

    public static function canSkipAssigendWorkStages(User $user, $throwException = true)
    {
        if ($user->can(['orders_stages.assigned_stage.skip'])) {
            return true;
        }

        if ($throwException) {
            throw new AccessDeniedException(__('lang_permissions.orders_stages.assigned_stage.skip.missing.permission'));
        }

        return false;
    }

    public static function canSkipAnyWorkStages(User $user, $throwException = true)
    {
        if ($user->can(['orders_stages.any_stage.skip'])) {
            return true;
        }

        if ($throwException) {
            throw new AccessDeniedException(__('lang_permissions.orders_stages.any_stage.skip.missing.permission'));
        }

        return false;
    }

    public static function userCanSkipStage(User $user, OrderWorkStage $stage, $throwException = true)
    {
        if (OrderWorkStagesService::isStageInWork($stage)) {
            return false;
        };

        if ($user->id === $stage->user_id && self::canSkipAssigendWorkStages($user, $throwException)) {
            return true;
        }

        if ($user->id !== $stage->user_id && self::canSkipAnyWorkStages($user, $throwException)) {
            return true;
        }

        if ($throwException) {
            throw new AccessDeniedException(
                __(
                    'User can not skip order stage'
                )
            );
        }
        return false;
    }

    /**
     * End user collaboration.
     *
     * @param User $user
     * @param array $validated
     * @param Request $request
     * @return User $user
     */
    public static function endCollaboration(User $user, array $validated, Request $request)
    {
        $user->status = $validated['end_collaboration'] === true ? 'locked' : 'active';

        if ($user->status === 'locked') {
            WorkStageUser::where('user_id', $user->id)->delete();
            WorkTemplateStageUser::where('user_id', $user->id)->delete();

            $orderWorkStages = OrderWorkStage::where('user_id', $user->id)
                ->where('status', '<>', 'finished')
                ->whereNull('finished_time')
                ->get();

            foreach ($orderWorkStages as $orderWorkStage) {
                UserSkipService::skipStageUser($request, OrderWorkStage::findOrFail($orderWorkStage->id));
            }
        }

        $user->save();
        return $user;
    }
}
